// --------------------------------------------------------------------------
#include <vcl\vcl.h>
#pragma hdrstop

// ==========================================================================
// File:            TDirectDraw.CPP
// Description:     This file contains the code for the TDirectDraw component
//
// (c) 1998 A.S.C. Alternate Software Concepts, Sydney, Australia.
// All Rights Reserved. Refer to the file TDirectX_Licence.Txt for detailed
// licence & copyright information.
// ==========================================================================

// --------------------------------------------------------------------------
// Include the TDirectDraw H File
// --------------------------------------------------------------------------
#include "TDirectDraw.h"

// --------------------------------------------------------------------------
// Method:          DDEnumXXXXCallback
// Description:     Forward declarations for various global callback
//                  functions used during execution of the
//                  TDirectDraw::EnumXXXX() methods
// Notes:           This function uses the standard TDirectX callback
//                  implementation (described elsewhere) to redirect the
//                  DirectX callbacks to the TDirectDraw::OnEnumXXXXCallback
//                  event methods
// --------------------------------------------------------------------------
long                __stdcall   DDEnumCallback( GUID FAR *lpGUID,
                                                LPSTR lpDriverDescription,
                                                LPSTR lpDriverName,
                                                LPVOID lpContext );

long                __stdcall   DDEnumCallbackEx( GUID FAR *lpGUID,
                                                LPSTR lpDriverDescription,
                                                LPSTR lpDriverName,
                                                LPVOID lpContext,
                                                HMONITOR hm );

long                __stdcall   DDEnumModesCallback(
                                            LPDDSURFACEDESC lpDDSurfaceDesc,
                                            LPVOID lpContext );

long                __stdcall   DDEnumSurfacesCallback(
                                            LPDIRECTDRAWSURFACE lpDDSurface,
                                            LPDDSURFACEDESC lpDDSurfaceDesc,
                                            LPVOID lpContext );

long                __stdcall   DDEnumVideoCallback(
                                        LPDDVIDEOPORTCAPS lpDDVideoPortCaps,
                                        LPVOID lpContext );

// --------------------------------------------------------------------------
// Globals:         DDEnumXXXXOwner
// Description:     These are temporary variables used by the
//                  EnumXXXXCallback methods
// --------------------------------------------------------------------------
TDirectDraw*        DDEnumCallbackOwner = NULL;
TDirectDraw*        DDEnumCallbackExOwner = NULL;
TDirectDraw*        DDEnumModesCallbackOwner = NULL;
TDirectDraw*        DDEnumSurfacesCallbackOwner = NULL;
TDirectDraw*        DDEnumVideoCallbackOwner = NULL;

// --------------------------------------------------------------------------
// Method:          Constructor
// Description:     Sets the default values used within TDirectDraw and
//                  creates any miscellaneous internal objects
// --------------------------------------------------------------------------
                    __fastcall  TDirectDraw::TDirectDraw( TComponent* Owner )
	                                                    : TComponent( Owner )
{
// setup default parameters
fDefaultHandle = NULL;
fUseFormHandle = true;
fCreated = false;
fDirectDraw = NULL;
fDirectDraw2 = NULL;
fPrimarySurface = NULL;
fBackBuffer = NULL;
fDisplayMode = _800x600x16;
fWidth = 800;
fHeight = 600;
fBitsPerPixel = 16;
fRefreshRate = 60;
fDriverCaps = new TDDCaps();
fHelCaps = new TDDCaps();
FOnError = ErrorHandler;
FOnMessage = MessageHandler;
}

// --------------------------------------------------------------------------
// Method:          Destructor
// Description:     Release / delete anything created in the constructor
// --------------------------------------------------------------------------
                    __fastcall  TDirectDraw::~TDirectDraw()
{
// make sure everything is released
if (fCreated) Release();
// release default parameters
delete fHelCaps;
delete fDriverCaps;
}

// --------------------------------------------------------------------------
// Method:          Create
// Description:     Call this method to create the internals of the
//                  TDirectDraw object
// Notes:           Remember to call the Release() method when finished
// --------------------------------------------------------------------------
HRESULT             __fastcall  TDirectDraw::Create()
{
// caJUSTse the VCL sets the default handler params to NULL
if (FOnError==NULL) FOnError = ErrorHandler;
if (FOnMessage==NULL) FOnMessage = MessageHandler;

// check pre-requisites
if (fCreated)
    {
    if (FOnError) FOnError(this,"TDirectDraw::Create()","TDX_ALREADYCREATED",
                    "The TDirectDraw interface has already been created" );
    return TDX_ALREADYCREATED;
    }

// set the default owner handle
if (fUseFormHandle) fDefaultHandle = ((TWinControl*)Owner)->Handle;
else fDefaultHandle = Application->Handle;

// assume everything is going to work as expected (otherwise the methods
//                              called below won't work because !fCreated)
if (FOnMessage) FOnMessage(this,"TDirectDraw::Create()",
                                        ">>> Creating TDirectDraw object");
fCreated = true;

// create basic IDIRECTDRAW interface
if (FOnMessage) FOnMessage(this,"TDirectDraw::Create()",
                            "Attempting to create IDirectDraw interface");
fError = ::DirectDrawCreate( NULL, &fDirectDraw, NULL );
if (fError!=DD_OK)
    {
    if (FOnError) FOnError(this,"TDirectDraw::Create()",
            DirectDrawErrorString(fError),DirectDrawErrorMessage(fError));
    fCreated = false;
    return fError;
    }
if (FOnMessage) FOnMessage(this,"TDirectDraw::Create()",
                        "IDirectDraw interface was created successfully");

// make sure IDIRECTDRAW is in normal mode
if (FOnMessage) FOnMessage(this,"TDirectDraw::Create()",
                        "Setting IDirectDraw to normal cooperative level");
fError = SetCooperativeLevel( fDefaultHandle, DDSCL_NORMAL );
if (fError!=DD_OK)
    {
    if (FOnError) FOnError(this,"TDirectDraw::Create()",
            DirectDrawErrorString(fError),DirectDrawErrorMessage(fError));
    fCreated = false;
    return fError;
    }
if (FOnMessage) FOnMessage(this,"TDirectDraw::Create()",
            "IDirectDraw was set to normal cooperative level successfully");

// create the IDIRECTDRAW2 object
if (FOnMessage) FOnMessage(this,"TDirectDraw::Create()",
                            "Attempting to create IDirectDraw2 interface");
fError = fDirectDraw->QueryInterface(IID_IDirectDraw2,(LPVOID *)
                                                            &fDirectDraw2);
if (fError!=DD_OK)
    {
    if (FOnError) FOnError(this,"TDirectDraw::Create()",
            DirectDrawErrorString(fError),DirectDrawErrorMessage(fError));
    fCreated = false;
    return fError;
    }
if (FOnMessage) FOnMessage(this,"TDirectDraw::Create()",
                        "IDirectDraw2 interface was created successfully");

// get the current IDIRECTDRAW/2 capabilities
if (FOnMessage) FOnMessage(this,"TDirectDraw::Create()",
            "Attempting to retrieve the current Driver / HEL capabilities");
fError = GetCaps( fDriverCaps, fHelCaps );
if (fError!=DD_OK)
    {
    if (FOnError) FOnError(this,"TDirectDraw::Create()",
            DirectDrawErrorString(fError),DirectDrawErrorMessage(fError));
    }
else if (FOnMessage) FOnMessage(this,"TDirectDraw::Create()",
                "Driver / HEL capabilities were retrieved successfully");


// everything worked as expected
if (FOnMessage) FOnMessage(this,"TDirectDraw::Create()",
"Setting IDirectDraw2 / IDirectDraw to the user-selected property settings");

// setup the default states based on what has been selected within the object
//                                      inspector / assigned before Create()

// primary surface
if (fPrimarySurface!=NULL)
    {
    if (FOnMessage) FOnMessage(this,"TDirectDraw::Create()",
                            "The PrimarySurface property has been assigned");

    // set fullscreen / exclusive mode
    if (FOnMessage) FOnMessage(this,"TDirectDraw::Create()",
            "About to set the cooperative level to exclusive/fullscreen");
    fError = SetCooperativeLevel( fDefaultHandle, DDSCL_EXCLUSIVE |
                                                        DDSCL_FULLSCREEN );
    if (fError!=DD_OK)
        {
        if (FOnError) FOnError(this,"TDirectDraw::Create()",
            DirectDrawErrorString(fError),DirectDrawErrorMessage(fError));
        fCreated = false;
        return fError;
        }
    if (FOnMessage) FOnMessage(this,"TDirectDraw::Create()",
    "The cooperative level was set to exclusive/fullscreen successfully.");

    // create the primary surface
    if (FOnMessage) FOnMessage(this,"TDirectDraw::Create()",
                                    "About to create the primary surface");
    fError = CreateSurface( fPrimarySurface );
    if (fError!=DD_OK)
        {
        if (FOnError) FOnError(this,"TDirectDraw::Create()",
            DirectDrawErrorString(fError),DirectDrawErrorMessage(fError));
        fCreated = false;
        return fError;
        }
    if (FOnMessage) FOnMessage(this,"TDirectDraw::Create()",
                        "The primary surface has been created successfully");

    // create any attached back buffers (only if the properties
    //                      PrimarySurface and BackBuffer have been defined)
    if (fBackBuffer!=NULL)
        {
        if (FOnMessage) FOnMessage(this,"TDirectDraw::Create()",
                    "TDirectDraw::BackBuffer property has been assigned");
        if (FOnMessage) FOnMessage(this,"TDirectDraw::Create()",
                                "About to get the attached back buffer(s)");
//        fError = fPrimarySurface->GetAttachedSurface
//                                  (fBackBuffer->SCaps->Caps,fBackBuffer);
        if (fError!=DD_OK)
            {
            if (FOnError) FOnError(this,"TDirectDraw::Create()",
               DirectDrawErrorString(fError),DirectDrawErrorMessage(fError));
            fCreated = false;
            return fError;
            }
        if (FOnMessage) FOnMessage(this,"TDirectDraw::Create()",
                        "Back Buffer(s) have been retrieved successfully");
        }

    }

// default cooperative level
if (FOnMessage) FOnMessage(this,"TDirectDraw::Create()",
                                        "Setting default CooperativeLevel");
CooperativeLevel = DDSCL_NORMAL | DDSCL_FULLSCREEN;

// default display mode
if (FOnMessage) FOnMessage(this,"TDirectDraw::Create()",
                                            "Setting default DisplayMode");
DisplayMode = fDisplayMode;

// all finished... call the OnCreate event method
if (FOnCreate)
    {
    if (FOnMessage) FOnMessage(this,"TDirectDraw::Create()",
                                        "Calling the OnCreate event method");
    FOnCreate(this);
    }

// at this point fError must contain DD_OK, simply return it
if (FOnMessage) FOnMessage(this,"TDirectDraw::Create()",
                    ">>> The TDirectDraw object has been created correctly");
return fError;
}

// --------------------------------------------------------------------------
// Method:          CreateClipper
// Description:     Creates a DirectDrawClipper object
// --------------------------------------------------------------------------
HRESULT	            __fastcall	TDirectDraw::CreateClipper(
                                            TDirectDrawClipper* pClipper )
{
// check pre-requisites
if (!fCreated)
    {
    if (FOnError) FOnError(this,"TDirectDraw::CreateClipper()",
        "TDX_NOTCREATED","The TDirectDraw interface has not been created" );
    return TDX_NOTCREATED;
    }
if (pClipper==NULL)
    {
    if (FOnError) FOnError(this,"TDirectDraw::CreateClipper()",
                        "TDX_BADPARAMS","the 'pClipper' parameter is NULL");
    return TDX_BADPARAMS;
    }

/*
// create the internals for the TDirectDrawClipper object
if (FOnMessage) FOnMessage(this,"TDirectDraw::CreateClipper()",
                                "About to create clipper " + clipper->Name);

fError = fDirectDraw2->CreateClipper( clipper->Description->Ptr(),
                                clipper->PtrToDirectDrawClipperPtr, NULL );
if (fError!=DD_OK)
    {
    if (FOnError) FOnError(this,"TDirectDraw::CreateClipper()",
            DirectDrawErrorString(fError),DirectDrawErrorMessage(fError));
    return fError;
    }
// everything worked as expected
if (FOnMessage) FOnMessage(this,"TDirectDraw::CreateClipper()",
                                        "clipper was created successfully");

*/
// at this point fError must contain DD_OK, simply return it.
return fError;
}

// --------------------------------------------------------------------------
// Method:          CreatePalette
// Description:     Creates a DirectDrawPalette for this DirectDraw object
// Parameters:      Palette - address of a pointer for the address of the new
//                          DirectDrawPalette object if successfully created
// --------------------------------------------------------------------------
HRESULT	            __fastcall	TDirectDraw::CreatePalette(
                                            TDirectDrawPalette* pPalette )
{
// check pre-requisites
if (!fCreated)
    {
    if (FOnError) FOnError(this,"TDirectDraw::CreatePalette()",
        "TDX_NOTCREATED","The TDirectDraw interface has not been created" );
    return TDX_NOTCREATED;
    }
if (pPalette==NULL)
    {
    if (FOnError) FOnError(this,"TDirectDraw::CreatePalette()",
                        "TDX_BADPARAMS","the 'pPalette' parameter is NULL");
    return TDX_BADPARAMS;
    }

/*
// create the internals for the TDirectDrawPalette object
if (FOnMessage) FOnMessage(this,"TDirectDraw::CreatePalette()",
                                "About to create palette " + palette->Name);

fError = fDirectDraw2->CreatePalette( palette->Description->Ptr(),
                                palette->PtrToDirectDrawPalettePtr, NULL );
if (fError!=DD_OK)
    {
    if (FOnError) FOnError(this,"TDirectDraw::CreatePalette()",
            DirectDrawErrorString(fError),DirectDrawErrorMessage(fError));
    return fError;
    }
// everything worked as expected
if (FOnMessage) FOnMessage(this,"TDirectDraw::CreatePalette()",
                                        "palette was created successfully");

*/
// at this point fError must contain DD_OK, simply return it.
return fError;
}

// --------------------------------------------------------------------------
// Method:          CreateSurface
// Description:     Call this function to create a direct draw surface
// Parameters:      Surface - an existing TDirectDrawSurface that is to be
//                          created
// --------------------------------------------------------------------------
HRESULT	            __fastcall	TDirectDraw::CreateSurface(
                                            TDirectDrawSurface* pSurface )
{
// check pre-requisites
if (!fCreated)
    {
    if (FOnError) FOnError(this,"TDirectDraw::CreateSurface()",
        "TDX_NOTCREATED","The TDirectDraw interface has not been created" );
    return TDX_NOTCREATED;
    }
if (pSurface==NULL)
    {
    if (FOnError) FOnError(this,"TDirectDraw::CreateSurface()",
                        "TDX_BADPARAMS","the 'pSurface' parameter is NULL");
    return TDX_BADPARAMS;
    }

// create the internals for the TDirectDrawSurface
if (FOnMessage) FOnMessage(this,"TDirectDraw::CreateSurface()",
                                "About to create surface " + pSurface->Name);

LPDIRECTDRAWSURFACE lpdds;
fError = fDirectDraw2->CreateSurface( pSurface->Description->Ptr(),
                                                            &lpdds, NULL );
if (fError!=DD_OK)
    {
    if (FOnError) FOnError(this,"TDirectDraw::CreateSurface()",
            DirectDrawErrorString(fError),DirectDrawErrorMessage(fError));
    return fError;
    }
// everything worked as expected
pSurface->Ptr = (LPDIRECTDRAWSURFACE3) lpdds;
if (FOnMessage) FOnMessage(this,"TDirectDraw::CreateSurface()",
                                        "surface was created successfully");

// at this point fError must contain DD_OK, simply return it.
return fError;
}

// --------------------------------------------------------------------------
// Method:          DuplicateSurface
// Description:     Duplicates a DirectDrawSurface object
// Parameters:      SourceSurface   - address of structure to be duplicated
//                  DuplicateSurface- address of pointer to the newly created
//                                  duplicate structure
// --------------------------------------------------------------------------
HRESULT	            __fastcall	TDirectDraw::DuplicateSurface(
                                    TDirectDrawSurface* pSourceSurface,
                                    TDirectDrawSurface* pDuplicateSurface )
{
// check pre-requisites
if (!fCreated)
    {
    if (FOnError) FOnError(this,"TDirectDraw::DuplicateSurface()",
        "TDX_NOTCREATED","The TDirectDraw interface has not been created" );
    return TDX_NOTCREATED;
    }
if (pSourceSurface==NULL)
    {
    if (FOnError) FOnError(this,"TDirectDraw::DuplicateSurface()",
                "TDX_BADPARAMS","the 'pSourceSurface' parameter was NULL");
    return TDX_BADPARAMS;
    }
if (pDuplicateSurface==NULL)
    {
    if (FOnError) FOnError(this,"TDirectDraw::DuplicateSurface()",
            "TDX_BADPARAMS","the 'pDuplicateSurface' parameter was NULL");
    return TDX_BADPARAMS;
    }
// duplicate the surface
if (FOnMessage) FOnMessage(this,"TDirectDraw::DuplicateSurface()",
                                        "about to duplicate the surface");
//DJD
//fError = fDirectDraw2->DuplicateSurface( pSourceSurface->Ptr,
//                                                  pDuplicateSurface->Ptr );
if (fError!=DD_OK)
	{
    if (FOnError) FOnError(this,"TDirectDraw::DuplicateSurface()",
            DirectDrawErrorString(fError),DirectDrawErrorMessage(fError));
    return fError;
    }
// everything worked as expected
if (FOnMessage) FOnMessage(this,"TDirectDraw::DuplicateSurface()",
                                        "surface was duplicated correctly");
// at this point fError must contain DD_OK, simply return it.
return fError;
}

// --------------------------------------------------------------------------
// Method:          EnumDisplayModes
// Description:     Call this function to "enumerate" the current display
//                  modes
// Parameters:      flags   - can be used to modify the type of display mode
//                          enumerated
//                  desc    -  the options of this object can be used to
//                          determine the kind of display mode to be
//                          enumerated
//                  context - this is pointer to a user-defined object that
//                          will be passed directly to the OnEnumDisplayModes
//                          event method for each of the enumerated display
//                          modes
// Note:            TDirectDraw will call the OnEnumDisplayModes event method
//                  for each of the currently available display modes
// --------------------------------------------------------------------------
HRESULT	            __fastcall	TDirectDraw::EnumDisplayModes( dword pFlags,
                                                TDDSurfaceDesc* pSurfaceDesc,
                                                void* pContext )
{
// check pre-requisites
if (!fCreated)
    {
    if (FOnError) FOnError(this,"TDirectDraw::EnumDisplayModes()",
        "TDX_NOTCREATED","The TDirectDraw interface has not been created" );
    return TDX_NOTCREATED;
    }
if (pSurfaceDesc==NULL)
    {
    if (FOnError) FOnError(this,"TDirectDraw::EnumDisplayModes()",
                    "TDX_BADPARAMS","the 'pSurfaceDesc' parameter was NULL");
    return TDX_BADPARAMS;
    }

// enumerate the display modes (as an IDE event method)
if (FOnMessage) FOnMessage(this,"TTestDirectDraw::EnumDisplayModes()",
                                        "About to enumerate display modes");
// since we've had to declare the DDEnumModesCallback() function as a global
// function, and it still needs to know about this TDirectDraw object so that
// it can call the appropriate event method, we need to tell it
// which TDirectDraw object called it.
// The way we are doing it is to use a global TDirectDraw*, which all
// functions in this .CPP file know about
DDEnumModesCallbackOwner = this;
fError = fDirectDraw2->EnumDisplayModes( pFlags, pSurfaceDesc->Ptr(),
                                            pContext, DDEnumModesCallback );
if (fError!=DD_OK)
    {
    if (FOnError) FOnError(this,"TDirectDraw::EnumDisplayModes()",
            DirectDrawErrorString(fError),DirectDrawErrorMessage(fError));
    return fError;
    }
// make sure that it's no longer in use...
DDEnumModesCallbackOwner = NULL;
// everything worked as expected
if (FOnMessage) FOnMessage(this,"TDirectDraw::EnumDisplayModes()",
                                "display modes were enumerated correctly.");
// at this point fError must contain DD_OK, simply return it.
return fError;
}

// --------------------------------------------------------------------------
// Method:          EnumSurfaces
// Description:     Enumerates all of the existing or possible surfaces that
//                  meet the search criterion specified
// Parameters:      Flags       - type of enum search to be performed
//                  SurfaceDesc - address of a TDDSURFACEDESC structure that
//                              defines the surface of interest
//                  Context     - address of a structure that will be passed
//                              to each enumeration member
// --------------------------------------------------------------------------
HRESULT	            __fastcall	TDirectDraw::EnumSurfaces( dword pFlags,
                                            TDDSurfaceDesc* pSurfaceDesc,
                                            void* pContext )
{
// check pre-requisites
if (!fCreated)
    {
    if (FOnError) FOnError(this,"TDirectDraw::EnumSurfaces()",
        "TDX_NOTCREATED","The TDirectDraw interface has not been created" );
    return TDX_NOTCREATED;
    }
if (pSurfaceDesc==NULL)
    {
    if (FOnError) FOnError(this,"TDirectDraw::EnumSurfaces()",
                    "TDX_BADPARAMS","the 'pSurfaceDesc' parameter was NULL");
    return TDX_BADPARAMS;
    }

// enumerate the display modes (as an IDE event method)
if (FOnMessage) FOnMessage(this,"TTestDirectDraw::EnumSurfaces()",
                                            "About to enumerate surfaces");
// since we've had to declare the DDEnumModesCallback() function as a global
// function, and it still needs to know about this TDirectDraw object so that
// it can call the appropriate event method, we need to tell it which
// TDirectDraw object called it
// The way we are doing it is to use a global TDirectDraw*, which all
// functions in this .CPP file know about
DDEnumSurfacesCallbackOwner = this;
fError = fDirectDraw2->EnumSurfaces( pFlags, pSurfaceDesc->Ptr(), pContext,
                                                    DDEnumSurfacesCallback );
if (fError!=DD_OK)
    {
    if (FOnError) FOnError(this,"TDirectDraw::EnumSurfaces()",
            DirectDrawErrorString(fError),DirectDrawErrorMessage(fError));
    return fError;
    }
// make sure that it's no longer in use...
DDEnumSurfacesCallbackOwner = NULL;
// everything worked as expected
if (FOnMessage) FOnMessage(this,"TDirectDraw::EnumSurfaces()",
                                    "surfaces were enumerated correctly.");
// at this point fError must contain DD_OK, simply return it.
return fError;
}

// --------------------------------------------------------------------------
// Method:          ErrorHandler
// Description:     This is the TDirectDraw component's default error
//                  handling method which displays a simple dialog box with
//                  appropriate formatting of the error message
// Notes:           Use the OnError event to override default error handling
// --------------------------------------------------------------------------
void                __fastcall  TDirectDraw::ErrorHandler( TObject *Sender,
                                                AnsiString TheFunction,
                                                AnsiString TheError,
                                                AnsiString TheErrorMessage )
{
ShowMessage(TheFunction+" has encountered a DirectX error as follows :- \n\n"
                + "DirectX Error: " + TheError + "\n" +
                "DirectX Message: " + TheErrorMessage + "\n\n" +
                "continuing execution..." );
}

// --------------------------------------------------------------------------
// Method:          FlipToGDISurface
// Description:     Flips the currently displayed surface so that the surface
//                  used by the Windows GDI is displayed.
// --------------------------------------------------------------------------
HRESULT	            __fastcall	TDirectDraw::FlipToGDISurface()
{
// check pre-requisites
if (!fCreated)
    {
    if (FOnError) FOnError(this,"TDirectDraw::FlipToGDISurface()",
        "TDX_NOTCREATED","The TDirectDraw interface has not been created" );
    return TDX_NOTCREATED;
    }
if (FOnMessage) FOnMessage(this,"TDirectDraw::FlipToGDISurface()",
                                        "about to flip to the gdi surface");
fError = fDirectDraw2->FlipToGDISurface();
if (fError!=DD_OK)
    {
    if (FOnError) FOnError(this,"TDirectDraw::FlipToGDISurface()",
            DirectDrawErrorString(fError),DirectDrawErrorMessage(fError));
    return fError;
    }
if (FOnMessage) FOnMessage(this,"TDirectDraw::FlipToGDISurface()",
                            "the flip to the gdi surface worked correctly");
return fError;
}

// --------------------------------------------------------------------------
// Method:          GetAvailableVidMem
// Description:     Call this function to retrieve the total memory and free
//                  memory available for a particular type of surface
// Parameters:      scaps   - an existing TDDSCaps object describing the type
//                          of surface that we want to get information about
//                  total   - a pointer to a local variable that is to be
//                          filled with total available memory (in bytes)
//                  free    - a pointer to a local variable that is to be
//                          filled with free available memory  (in bytes)
// --------------------------------------------------------------------------
HRESULT	            __fastcall	TDirectDraw::GetAvailableVidMem(
                                                            TDDSCaps* pSCaps,
                                                            dword* pTotal,
                                                            dword* pFree )
{
// check pre-requisites
if (!fCreated)
    {
    if (FOnError) FOnError(this,"TDirectDraw::GetAvailableVidMem()",
        "TDX_NOTCREATED","The TDirectDraw interface has not been created" );
    return TDX_NOTCREATED;
    }
if (pSCaps==NULL)
    {
    if (FOnError) FOnError(this,"TDirectDraw::GetAvailableVidMem()",
                        "TDX_BADPARAMS","the 'pSCaps' parameter was NULL");
    return TDX_BADPARAMS;
    }
if (pTotal==NULL)
    {
    if (FOnError) FOnError(this,"TDirectDraw::GetAvailableVidMem()",
                        "TDX_BADPARAMS","the 'pTotal' parameter was NULL");
    return TDX_BADPARAMS;
    }
if (pFree==NULL)
    {
    if (FOnError) FOnError(this,"TDirectDraw::GetAvailableVidMem()",
                        "TDX_BADPARAMS","the 'pFree' parameter was NULL");
    return TDX_BADPARAMS;
    }
if (FOnMessage) FOnMessage(this,"TDirectDraw::GetAvailableVidMem()",
                                    "about to get available video memory");
fError = fDirectDraw2->GetAvailableVidMem( pSCaps->Ptr(), pTotal, pFree );
if (fError!=DD_OK)
	{
    if (FOnError) FOnError(this,"TDirectDraw::GetAvailableVidMem()",
            DirectDrawErrorString(fError),DirectDrawErrorMessage(fError));
    return fError;
    }
if (FOnMessage) FOnMessage(this,"TDirectDraw::GetAvailableVidMem()",
                        "get available video memory worked successfully");
return fError;
}

// --------------------------------------------------------------------------
// Method:          GetCaps
// Description:     Call this function to retrieve the current direct draw
//                  capabilities
// Parameters:      drivercaps  - an existing TDDCaps object that is to be
//                              filled with the current driver capabilities
//                  helcaps     - an existing TDDCaps object that is to be
//                              filled with the current hardware emulation
//                              layer capabilities
// --------------------------------------------------------------------------
HRESULT	            __fastcall	TDirectDraw::GetCaps( TDDCaps* pDriverCaps,
                                                        TDDCaps* pHelCaps )
{
// check pre-requisites
if (!fCreated)
    {
    if (FOnError) FOnError(this,"TDirectDraw::GetCaps()","TDX_NOTCREATED",
                        "The TDirectDraw interface has not been created" );
    return TDX_NOTCREATED;
    }
if (pDriverCaps==NULL)
    {
    if (FOnError) FOnError(this,"TDirectDraw::GetCaps()","TDX_BADPARAMS",
                                    "the 'pDriverCaps' parameter was NULL");
    return TDX_BADPARAMS;
    }
if (pHelCaps==NULL)
    {
    if (FOnError) FOnError(this,"TDirectDraw::GetCaps()","TDX_BADPARAMS",
                                        "the 'pHelCaps' parameter was NULL");
    return TDX_BADPARAMS;
    }

// get the capabilities of the surface into our TDDCaps wrappers
if (FOnMessage) FOnMessage(this,"TDirectDraw::GetCaps()",
                                "about to retrieve driver/hel capabilities");
fError = fDirectDraw2->GetCaps( pDriverCaps->Ptr(), pHelCaps->Ptr() );
if (fError!=DD_OK)
    {
    if (FOnError) FOnError(this,"TDirectDraw::GetCaps()",
            DirectDrawErrorString(fError),DirectDrawErrorMessage(fError));
    return fError;
    }
// everything worked as expected
if (FOnMessage) FOnMessage(this,"TDirectDraw::GetCaps()",
                    "driver/hel capabilities were retrieved successfully");

// at this point fError must contain DD_OK, simply return it.
return fError;
}

// --------------------------------------------------------------------------
// Method:          GetDisplayMode
// Description:     Returns current display mode
// Note:            Examples _640x480x8, _800x600x16, etc
// --------------------------------------------------------------------------
TDDDisplayMode        __fastcall  TDirectDraw::FGetDisplayMode()
{
return fDisplayMode;
}

// --------------------------------------------------------------------------
// Method:          GetFourCCCodes
// Description:     Retrieves FOURCC codes supported by the DirectDraw object
// Parameters:      NumCodes    - address with the number of codes supported
//                  Codes       - address of array for holding FOURCC codes
// Note:            This method can also retrieve the number of codes
//                  supported by setting Codes to NULL
// --------------------------------------------------------------------------
HRESULT	            __fastcall	TDirectDraw::GetFourCCCodes( dword* pNumCodes
                                                             ,dword* pCodes )
{
// check pre-requisites
if (!fCreated)
    {
    if (FOnError) FOnError(this,"TDirectDraw::GetFourCCCodes()",
        "TDX_NOTCREATED","The TDirectDraw interface has not been created" );
    return TDX_NOTCREATED;
    }
if (pNumCodes==NULL)
    {
    if (FOnError) FOnError(this,"TDirectDraw::GetFourCCCodes()",
                    "TDX_BADPARAMS","the 'pNumCodes' parameter was NULL");
    return TDX_BADPARAMS;
    }
if (pCodes==NULL)
    {
    if (FOnError) FOnError(this,"TDirectDraw::GetFourCCCodes()",
                        "TDX_BADPARAMS","the 'pCodes' parameter was NULL");
    return TDX_BADPARAMS;
    }
// get the fourCC codes
if (FOnMessage) FOnMessage(this,"TDirectDraw::GetFourCCCodes()",
                                    "about to retrieve the FourCC codes");
fError = fDirectDraw2->GetFourCCCodes( pNumCodes, pCodes );
if (fError!=DD_OK)
	{
    if (FOnError) FOnError(this,"TDirectDraw::GetFourCCCodes()",
            DirectDrawErrorString(fError),DirectDrawErrorMessage(fError));
    return fError;
    }
// everything worked as expected
if (FOnMessage) FOnMessage(this,"TDirectDraw::GetFourCCCodes()",
                                "the FourCC codes were retrieved correctly");
// at this point fError must contain DD_OK, simply return it.
return fError;
}

// --------------------------------------------------------------------------
// Method:          GetGDISurface
// Description:     Retrieves the surface memory that GDI is treating as the
//                  primary surface
// Parameters:      GDISurface  - Address of a pointer to the
//                              DirectDrawSurface object that currently
//                              controls GDIs primary surface memory
// --------------------------------------------------------------------------
HRESULT	            __fastcall	TDirectDraw::GetGDISurface(
                                            TDirectDrawSurface* pGDISurface )
{
// check pre-requisites
if (!fCreated)
    {
    if (FOnError) FOnError(this,"TDirectDraw::GetGDISurface()",
        "TDX_NOTCREATED","The TDirectDraw interface has not been created" );
    return TDX_NOTCREATED;
    }
if (pGDISurface==NULL)
    {
    if (FOnError) FOnError(this,"TDirectDraw::GetGDISurface()",
                    "TDX_BADPARAMS","the 'pGDISurface' parameter was NULL");
    return TDX_BADPARAMS;
    }
// get the gdi surface
if (FOnMessage) FOnMessage(this,"TDirectDraw::GetGDISurface()",
                                            "about to get the gdi surface");
//DJD
//LPDIRECTDRAWSURFACE gdi_surface;
//fError = fDirectDraw2->GetGDISurface( &gdi_surface );
if (fError!=DD_OK)
	{
    if (FOnError) FOnError(this,"TDirectDraw::GetGDISurface()",
            DirectDrawErrorString(fError),DirectDrawErrorMessage(fError));
    return fError;
    }
// everything worked as expected
if (FOnMessage) FOnMessage(this,"TDirectDraw::GetGDISurface()",
                                "the gdi surface was retrieved correctly");
// at this point fError must contain DD_OK, simply return it.
return fError;
}

// --------------------------------------------------------------------------
// Method:          GetMonitorFrequency
// Description:     Call this function to retrieve the current monitor
//                  frequency
// Parameters:      monitorfrequency    - a pointer to a local variable that
//                                      is to be filled with the current
//                                      monitor frequency
// --------------------------------------------------------------------------
HRESULT	            __fastcall	TDirectDraw::GetMonitorFrequency(
                                                dword* pMonitorFrequency )
{
// check pre-requisites
if (!fCreated)
    {
    if (FOnError) FOnError(this,"TDirectDraw::GetMonitorFrequency()",
        "TDX_NOTCREATED","The TDirectDraw interface has not been created" );
    return TDX_NOTCREATED;
    }
if (pMonitorFrequency==NULL)
    {
    if (FOnError) FOnError(this,"TDirectDraw::GetMonitorFrequency()",
                "TDX_BADPARAMS","the 'monitorfrequency' parameter was NULL");
    return TDX_BADPARAMS;
    }
// get the monitor frequency
if (FOnMessage) FOnMessage(this,"TDirectDraw::GetMonitorFrequency()",
                                            "getting the monitor frequency");
fError = fDirectDraw2->GetMonitorFrequency( pMonitorFrequency );
if (fError!=DD_OK)
	{
    if (FOnError) FOnError(this,"TDirectDraw::GetMonitorFrequency()",
            DirectDrawErrorString(fError),DirectDrawErrorMessage(fError));
    return fError;
    }
// everything worked as expected
if (FOnMessage) FOnMessage(this,"TDirectDraw::GetMonitorFrequency()",
                                "monitor frequency was retrieved correctly");
// at this point fError must contain DD_OK, simply return it.
return fError;
}

// --------------------------------------------------------------------------
// Method:          GetScanLine
// Description:     Call this function to retrieve the current monitor scan
//                  line
// Parameters:      scanline    - a pointer to a local variable that is to be
//                              filled with the current monitor scan line.
// --------------------------------------------------------------------------
HRESULT	            __fastcall	TDirectDraw::GetScanLine( dword* pScanLine )
{
// check pre-requisites
if (!fCreated)
    {
    if (FOnError) FOnError(this,"TDirectDraw::GetScanLine()",
        "TDX_NOTCREATED","The TDirectDraw interface has not been created" );
    return TDX_NOTCREATED;
    }
if (pScanLine==NULL)
    {
    if (FOnError) FOnError(this,"TDirectDraw::GetScanLine()",
                    "TDX_BADPARAMS","the 'pScanLine' parameter was NULL");
    return TDX_BADPARAMS;
    }
// get the scan line
if (FOnMessage) FOnMessage(this,"TDirectDraw::GetScanLine()",
                                            "about to get the scan line");
fError = fDirectDraw2->GetScanLine( pScanLine );
if (fError!=DD_OK)
	{
    if (FOnError) FOnError(this,"TDirectDraw::GetScanLine()",
            DirectDrawErrorString(fError),DirectDrawErrorMessage(fError));
    return fError;
    }
// everything worked as expected
if (FOnMessage) FOnMessage(this,"TDirectDraw::GetScanLine()",
                                        "scan line was retrieved correctly");
// at this point fError must contain DD_OK, simply return it.
return fError;
}

// --------------------------------------------------------------------------
// Method:          GetVerticalBlankStatus
// Description:     Call this function to retrieve the current vertical blank
//                  status
// Parameters:      scanline    - a pointer to a local boolean variable that
//                              is to be filled with the current vertical
//                              blank status
// Notes:           if the result of this call is true, the monitor is
//                  currently inside the vertical blank period
// --------------------------------------------------------------------------
HRESULT	            __fastcall	TDirectDraw::GetVerticalBlankStatus(
                                                bool* pVerticalBlankStatus )
{
// check pre-requisites
if (!fCreated)
    {
    if (FOnError) FOnError(this,"TDirectDraw::GetVerticalBlankStatus()",
        "TDX_NOTCREATED","The TDirectDraw interface has not been created" );
    return TDX_NOTCREATED;
    }
if (pVerticalBlankStatus==NULL)
    {
    if (FOnError) FOnError(this,"TDirectDraw::GetVerticalBlankStatus()",
            "TDX_BADPARAMS","the 'pVerticalBlankStatus' parameter was NULL");
    return TDX_BADPARAMS;
    }
// get the vertical blank status
if (FOnMessage) FOnMessage(this,"TDirectDraw::GetVerticalBlankStatus()",
                            "about to retrieve the vertical blank status");
fError = fDirectDraw2->GetVerticalBlankStatus( (int*) pVerticalBlankStatus );
if (fError!=DD_OK)
	{
    if (FOnError) FOnError(this,"TDirectDraw::GetVerticalBlankStatus()",
            DirectDrawErrorString(fError),DirectDrawErrorMessage(fError));
    return fError;
    }
// everything worked as expected
if (FOnMessage) FOnMessage(this,"TDirectDraw::GetVerticalBlankStatus()",
                            "vertical blank status was retrieved correctly");
// at this point fError must contain DD_OK, simply return it.
return fError;
}

// --------------------------------------------------------------------------
// Method:          MessageHandler
// Description:     This is the TDirectDraw component's default message
//                  handling method
// Note:            The default implementation of this function is that
//                  it completely ignores the message
//                  Use the OnMessage event to override default message
//                  handling
// --------------------------------------------------------------------------
void                __fastcall  TDirectDraw::MessageHandler( TObject *Sender,
                                                    AnsiString TheFunction,
                                                    AnsiString TheMessage )
{
// do nothing by default
/*
ShowMessage( "TDirectDraw reports:\n\n" +
             TheFunction + "\n" +
             TheMessage);
*/
}

// --------------------------------------------------------------------------
// Method:          Release
// Description:     Call this method to release the internals of the
//                  TDirectDraw object
// --------------------------------------------------------------------------
HRESULT             __fastcall  TDirectDraw::Release()
{
// if no OnError/OnMessage event handlers have been supplied, make sure we
//                                                  use the default handlers
if (FOnMessage==NULL) FOnMessage = MessageHandler;
if (FOnError==NULL) FOnError = ErrorHandler;

// call the OnRelease event method
if (FOnRelease)
    {
    if (FOnMessage) FOnMessage(this,"TDirectDraw::Release()",
                                "About to call the OnRelease event method");
    if (FOnRelease) FOnRelease(this);
    }

//  if the primary surface is not NULL, clean up anything to do with
//                                                      predefined surfaces
if (fPrimarySurface!=NULL)
    {
    // flip back to the GDI surface
    if (FOnMessage) FOnMessage(this,"TDirectDraw::Release()",
                                "Flipping back to the GDI display surface");
    fError = FlipToGDISurface();
    if (fError!=DD_OK)
        {
        if (FOnError) FOnError(this,"TDirectDraw::Release()",
            DirectDrawErrorString(fError),DirectDrawErrorMessage(fError));
        }
    else
        if (FOnMessage) FOnMessage(this,"TDirectDraw::Release()",
            "Flipping back to the GDI display surface has been successful");

    // release the primary surface.
    // nb: back buffers are released when you release the primary surface
    if (FOnMessage) FOnMessage(this,"TDirectDraw::Release()",
                                    "About to release the primary surface");
    fError = fPrimarySurface->Release();
    fPrimarySurface = NULL;
    if (fError!=DD_OK)
        {
        if (FOnError) FOnError(this,"TDirectDraw::Release()",
            DirectDrawErrorString(fError),DirectDrawErrorMessage(fError));
        }
    else
        if (FOnMessage) FOnMessage(this,"TDirectDraw::Release()",
                            "The primary surface was released successfully");

    // release anything internal to our wrapper for the back buffer.
    if (fBackBuffer!=NULL)
        {
        if (FOnMessage) FOnMessage(this,"TDirectDraw::Release()",
                                    "Also releasing back buffer internals");
        fError = fBackBuffer->Release();
        fBackBuffer = NULL;
        if (fError!=DD_OK)
            {
            if (FOnError) FOnError(this,"TDirectDraw::Release()",
               DirectDrawErrorString(fError),DirectDrawErrorMessage(fError));
            }
        else
            if (FOnMessage) FOnMessage(this,"TDirectDraw::Release()",
                    "The back buffer internals were released successfully");
        }
    }

// set to fullscreen / exclusive mode
if (FOnMessage) FOnMessage(this,"TDirectDraw::Release()",
                    "Setting the cooperative level to exclusive/fullscreen");
fError = SetCooperativeLevel( fDefaultHandle, DDSCL_FULLSCREEN |
                                                        DDSCL_EXCLUSIVE );
if (fError!=DD_OK)
    {
    if (FOnError) FOnError(this,"TDirectDraw::Release()",
            DirectDrawErrorString(fError),DirectDrawErrorMessage(fError));
    }
else
    if (FOnMessage) FOnMessage(this,"TDirectDraw::Release()",
            "The cooperative level has been set to exclusive/fullscreen");

// restore the original (application-startup) display mode
if (FOnMessage) FOnMessage(this,"TDirectDraw::Release()",
                                    "Restoring the original display mode");
fError = RestoreDisplayMode();
if (fError!=DD_OK)
    {
    if (FOnError) FOnError(this,"TDirectDraw::Release()",
            DirectDrawErrorString(fError),DirectDrawErrorMessage(fError));
    }
else
    if (FOnMessage) FOnMessage(this,"TDirectDraw::Release()",
                    "The original display mode was restored successfully");

// set cooperative level to normal
if (FOnMessage) FOnMessage(this,"TDirectDraw::Release()",
                            "Setting the cooperative level to normal mode");
fError = SetCooperativeLevel( fDefaultHandle, DDSCL_NORMAL );
if (fError!=DD_OK)
    {
    if (FOnError) FOnError(this,"TDirectDraw::Release()",
            DirectDrawErrorString(fError),DirectDrawErrorMessage(fError));
    }
else
    if (FOnMessage) FOnMessage(this,"TDirectDraw::Release()",
        "The cooperative level has been set to normal mode successfully");

// blow away IDIRECTDRAW2
if (fDirectDraw2!=NULL)
    {
    if (FOnMessage) FOnMessage(this,"TDirectDraw::Release()",
                                    "Releasing the IDirectDraw2 interface");
    fError = fDirectDraw2->Release();
    fDirectDraw2 = NULL;    // possible memory leak?
    if (fError!=DD_OK)
        {
        if (FOnError) FOnError(this,"TDirectDraw::Release()",
            DirectDrawErrorString(fError),DirectDrawErrorMessage(fError));
        }
    else
        if (FOnMessage) FOnMessage(this,"TDirectDraw::Release()",
                    "The IDirectDraw2 interface was released successfully");
    }

// blow away IDIRECTDRAW
if (fDirectDraw!=NULL)
    {
    if (FOnMessage) FOnMessage(this,"TDirectDraw::Release()",
                                    "Releasing the IDirectDraw interface");
    fError = fDirectDraw->Release();
    if (fError!=DD_OK)
        {
        if (FOnError) FOnError(this,"TDirectDraw::Release()",
            DirectDrawErrorString(fError),DirectDrawErrorMessage(fError));
        }
    else
        {
        fDirectDraw = NULL;
        if (FOnMessage) FOnMessage(this,"TDirectDraw::Release()",
                    "The IDirectDraw interface was released successfully");
        }
    }

// everything worked as expected
fCreated = false;
if (FOnMessage) FOnMessage(this,"TDirectDraw::Release()",
                    "the TDirectDraw object has been released successfully");

// at this point fError must contain DD_OK, simply return it.
return fError;
}

// --------------------------------------------------------------------------
// Method:          RestoreDisplayMode
// Description:     This method will restore the display mode to the mode
//                  that was set before the application first changed the
//                  display mode
// --------------------------------------------------------------------------
HRESULT	__fastcall	TDirectDraw::RestoreDisplayMode()
{
// check pre-requisites
if (!fCreated)
    {
    if (FOnError) FOnError(this,"TDirectDraw::RestoreDisplayMode()",
        "TDX_NOTCREATED","The TDirectDraw interface has not been created" );
    return TDX_NOTCREATED;
    }
// restore the display mode
if (FOnMessage) FOnMessage(this,"TDirectDraw::RestoreDisplayMode()",
                                        "about to restore the display mode");
fError = fDirectDraw2->RestoreDisplayMode();
if (fError!=DD_OK)
	{
    if (FOnError) FOnError(this,"TDirectDraw::RestoreDisplayMode()",
            DirectDrawErrorString(fError),DirectDrawErrorMessage(fError));
    return fError;
    }
// everything worked as expected
if (FOnMessage) FOnMessage(this,"TDirectDraw::RestoreDisplayMode()",
                                    "display mode was restored correctly");
// at this point fError must contain DD_OK, simply return it.
return fError;
}

// --------------------------------------------------------------------------
// Method:          SetCooperativeLevel
// Description:     Determines the top-level behavior of the application
// Parameters:      Flags   - CooperativeLevel settings
//                  Handle  - applications handle, can be NULL if mode normal
// --------------------------------------------------------------------------
HRESULT	            __fastcall	TDirectDraw::SetCooperativeLevel(
                                                            HWND pHandle,
                                                            dword pFlags )
{
// note:    this function is called by Create() between creation of
//          IDirectDraw and IDirectDraw2, hence, we still have a check if
//          (fDirectDraw2!=NULL)

// check pre-requisites
if (!fCreated)
    {
    if (FOnError) FOnError(this,"TDirectDraw::SetCooperativeLevel()",
        "TDX_NOTCREATED","The TDirectDraw interface has not been created" );
    return TDX_NOTCREATED;
    }
if (FOnMessage) FOnMessage(this,"TDirectDraw::SetCooperativeLevel()",
                                        "about to set cooperative level");
// if the supplied handle is not available, use the default handle instead.
if (pHandle==NULL) pHandle = fDefaultHandle;

// use IDirectDraw2 if possible, otherwise use IDirectDraw
fError = TDX_NOTCREATED;
if (fDirectDraw2!=NULL)
    {
    if (FOnMessage) FOnMessage(this,"TDirectDraw::SetCooperativeLevel()",
       "DirectDraw2 is available. Attempting to use DirectDraw2 interface.");
    fError = fDirectDraw2->SetCooperativeLevel( pHandle, pFlags );
    }
if (fError!=DD_OK)
    {
    if (FOnMessage) FOnMessage(this,"TDirectDraw::SetCooperativeLevel()",
       "Could not use DirectDraw2. Attempting to use DirectDraw interface.");
    fError = fDirectDraw->SetCooperativeLevel( pHandle, pFlags );
    if (fError!=DD_OK)
    	{
        if (FOnError) FOnError(this,"TDirectDraw::SetCooperativeLevel()",
            DirectDrawErrorString(fError),DirectDrawErrorMessage(fError));
        return fError;
        }
    }
if (FOnMessage) FOnMessage(this,"TDirectDraw::SetCooperativeLevel()",
                                "cooperative level was set successfully");
fCooperativeLevel = pFlags;
return fError;
}

// --------------------------------------------------------------------------
// Method:          SetDisplayMode
// Description:     This method can be used to change the current display
//                  mode settings
// Parameters:      width           - the width of the display mode to be set
//                  height          - the height of the display mode to be
//                                  set
//                  bitsperpixel    - the bits used per pixel of the display
//                                  mode to be set
//                  refreshrate     - the display mode refresh rate that will
//                                  be used
// --------------------------------------------------------------------------
HRESULT             __fastcall  TDirectDraw::SetDisplayMode( dword pWidth,
                                                        dword pHeight,
                                                        dword pBitsPerPixel,
                                                        dword pRefreshRate,
                                                        dword pFlags )
{
// check pre-requisites
if (!fCreated)
    {
    if (FOnError) FOnError(this,"TDirectDraw::SetDisplayMode()",
        "TDX_NOTCREATED","The TDirectDraw interface has not been created" );
    return TDX_NOTCREATED;
    }
// set the display mode (try IDirectDraw2 interface first)
if (FOnMessage) FOnMessage(this,"TDirectDraw::SetDisplayMode()",
                                            "about to set the display mode");
fError = fDirectDraw2->SetDisplayMode( pWidth, pHeight, pBitsPerPixel,
                                                    pRefreshRate, pFlags );
if (fError!=DD_OK)
	{
    // now try the IDirectDraw interface
    fError = fDirectDraw->SetDisplayMode( pWidth, pHeight, pBitsPerPixel );
    if (fError!=DD_OK)
        {
        if (FOnError) FOnError(this,"TDirectDraw::SetDisplayMode()",
            DirectDrawErrorString(fError),DirectDrawErrorMessage(fError));
        return fError;
        }
    }
// everything worked as expected
if (FOnMessage) FOnMessage(this,"TDirectDraw::SetDisplayMode()",
                                        "display mode was set correctly");

// make sure that we've updated the relevant display mode properties
// use temporary display mode var
TDDDisplayMode temp_display_mode = _CustomDisplayMode;
// scan through predefined display mode types to determine if the display
// mode settings are one of the standard TDDDisplayMode types....
for (int i=1;i<Num_DDDisplayModes;i++)
    if ((DDDisplayModeInfo.Width[i] == pWidth) &&
        (DDDisplayModeInfo.Height[i] == pHeight) &&
        (DDDisplayModeInfo.BitsPerPixel[i] == pBitsPerPixel ))
        {
        temp_display_mode = DDDisplayModeInfo.Mode[i];
        break;
        }
// set the rest of the related properties
fDisplayMode = temp_display_mode;
fWidth = pWidth;
fHeight = pHeight;
fBitsPerPixel = pBitsPerPixel;
fRefreshRate = pRefreshRate;

// at this point fError must contain DD_OK, simply return it.
return fError;
}

// --------------------------------------------------------------------------
// Method:          WaitForVerticalBlank
// Description:     Use this function to help synchronize the application
//                  with the vertical-blank interval
// Parameters:      Flags   - which portion of the vertical blank to wait for

// --------------------------------------------------------------------------
HRESULT	            __fastcall	TDirectDraw::WaitForVerticalBlank(
                                                            dword pFlags )
{
// check pre-requisites
if (!fCreated)
    {
    if (FOnError) FOnError(this,"TDirectDraw::WaitForVerticalBlank()",
        "TDX_NOTCREATED","The TDirectDraw interface has not been created" );
    return TDX_NOTCREATED;
    }
// wait for the vertical blank
if (FOnMessage) FOnMessage(this,"TDirectDraw::WaitForVerticalBlank()",
                                        "waiting for the vertical blank");
fError = fDirectDraw2->WaitForVerticalBlank( pFlags, NULL );
if (fError!=DD_OK)
	{
    if (FOnError) FOnError(this,"TDirectDraw::WaitForVerticalBlank()",
            DirectDrawErrorString(fError),DirectDrawErrorMessage(fError));
    return fError;
    }
// everything worked as expected
if (FOnMessage) FOnMessage(this,"TDirectDraw::WaitForVerticalBlank()",
                        "waiting for the vertical blank worked correctly");
// at this point fError must contain DD_OK, simply return it.
return fError;
}

// --------------------------------------------------------------------------
// Method:          GetBackBuffer
// Description:     Returns a pointer to the back buffer surface
// --------------------------------------------------------------------------
TDirectDrawSurface* __fastcall  TDirectDraw::FGetBackBuffer()
{
return fBackBuffer;
}

// --------------------------------------------------------------------------
// Method:          SetBackBuffer
// Description:     Sets a pointer to the back buffer surface
// --------------------------------------------------------------------------
void                __fastcall  TDirectDraw::FSetBackBuffer(
                                            TDirectDrawSurface* pBackBuffer )
{
if (pBackBuffer!=NULL) fBackBuffer = pBackBuffer;
}

// --------------------------------------------------------------------------
// Method:          GetBitsPerPixel
// Description:     Returns the default number of bits used per pixel
// Note:            The Create() method will attempt to use this value when
//                  setting the default display mode
// --------------------------------------------------------------------------
dword       __fastcall  TDirectDraw::FGetBitsPerPixel()
{
return fBitsPerPixel;
}

// --------------------------------------------------------------------------
// Method:          SetBitsPerPixel
// Description:     Sets the default number of bits used per pixel
// Note:            The Create() method will attempt to use this value when
//                  setting the default display mode
// --------------------------------------------------------------------------
void                __fastcall  TDirectDraw::FSetBitsPerPixel(
                                                        dword pBitsPerPixel )
{
fBitsPerPixel = pBitsPerPixel;
}

// --------------------------------------------------------------------------
// Method:          GetCooperativeLevel
// Description:     Returns the current CooperativeLevel of the internal
//                  IDIRECTDRAW2 interface
// Note:            Setting this property is equivalent to calling the
//                  SetCooperativeLevel() method with the default window
//                  handle as defined with the UseFormHandle property
// --------------------------------------------------------------------------
dword               __fastcall  TDirectDraw::FGetCooperativeLevel()
{
return fCooperativeLevel;
}

// --------------------------------------------------------------------------
// Method:          SetCooperativeLevel
// Description:     Sets the current CooperativeLevel of the internal
//                  IDIRECTDRAW2 interface
// Note:            Setting this property is equivalent to calling the
//                  SetCooperativeLevel() method with the default window
//                  handle as defined with the UseFormHandle property
// --------------------------------------------------------------------------
void                __fastcall  TDirectDraw::FSetCooperativeLevel(
                                                    dword pCooperativeLevel )
{
// the following code segments may seem a bit odd but...
//
// if the end-user sets the CooperativeLevel property during runtime, we need
// to actually set the directx cooperative level to reflect the change in the
// property. hence we call the SetCooperativeLevel() method if the end-user
// sets the CooperativeLevel property during design-time AND/OR directx is
// not available, the SetCooperativeLevel() call fails. yet we still need to
// adjust the cooperative level... hence the if (!fCreated)...
//
// notes:
//
// if the end-user calls the SetCooperativeLevel() method directly, it needs
// to set the fCooperativeLevel property to reflect the change. hence upon
// success SetCooperativeLevel() sets the fCooperativeLevel variable.
//
// during runtime, the Create() method sets directx to the appropriate
// design-time states.
//
// similar programming methods used throughout the tdirectx library

SetCooperativeLevel( NULL, pCooperativeLevel );
if (!fCreated) fCooperativeLevel = pCooperativeLevel;
}

// --------------------------------------------------------------------------
// Method:          GetCurrentDisplayModeInfo
// Description:     Returns a newly created TDDSurfaceDesc object containing
//                  the current display mode information
// Notes:           Returns NULL if the directx call failed
//                  Remember to 'delete' the newly created TDDSurfaceDesc*
// --------------------------------------------------------------------------
TDDSurfaceDesc*     __fastcall  TDirectDraw::FGetCurrentDisplayModeInfo()
{
TDDSurfaceDesc* desc = new TDDSurfaceDesc();
if (desc==NULL) return NULL;
GetDisplayMode( desc );
return desc;
}
// --------------------------------------------------------------------------
// Method:          SetCurrentDisplayModeInfo                                   //added set
// Description:     Sets a newly created TDDSurfaceDesc object containing
//                  display mode information
// --------------------------------------------------------------------------
void                __fastcall  TDirectDraw::FSetCurrentDisplayModeInfo(
                                    TDDSurfaceDesc* pCurrentDisplayModeInfo )
{
return;
}

// --------------------------------------------------------------------------
// Method:          GetDisplayMode
// Description:     Call this function to retrieve the current display mode
//                  description
// Parameters:      desc - an existing TDDSurfaceDesc object that is to be
//                      filled with the current display mode description
// --------------------------------------------------------------------------
HRESULT	            __fastcall	TDirectDraw::GetDisplayMode(
                                            TDDSurfaceDesc* pSurfaceDesc )
{
// check pre-requisites
if (!fCreated)
    {
    if (FOnError) FOnError(this,"TDirectDraw::GetDisplayMode()",
        "TDX_NOTCREATED","The TDirectDraw interface has not been created" );
    return TDX_NOTCREATED;
    }
if (pSurfaceDesc==NULL)
    {
    if (FOnError) FOnError(this,"TDirectDraw::GetDisplayMode()",
                    "TDX_BADPARAMS","the 'pSurfaceDesc' parameter was NULL");
    return TDX_BADPARAMS;
    }

// get the display mode
if (FOnMessage) FOnMessage(this,"TDirectDraw::GetDisplayMode()",
                                            "about to get the display mode");
fError = fDirectDraw2->GetDisplayMode( pSurfaceDesc->Ptr() );
if (fError!=DD_OK)
	{
    if (FOnError) FOnError(this,"TDirectDraw::GetDisplayMode()",
            DirectDrawErrorString(fError),DirectDrawErrorMessage(fError));
    return fError;
    }
// everything worked as expected
if (FOnMessage) FOnMessage(this,"TDirectDraw::GetDisplayMode()",
                                    "display mode was retrieved correctly");
// at this point fError must contain DD_OK, simply return it.
return fError;
}

// --------------------------------------------------------------------------
// Method:          SetDisplayMode
// Description:     Sets the current display mode to one of a list of
//                  standard display modes
// Note:            Examples _640x480x8, _800x600x16, etc
// --------------------------------------------------------------------------
void                __fastcall  TDirectDraw::FSetDisplayMode(
                                                TDDDisplayMode pDisplayMode )
{
// use current settings as the basis for the change
dword   temp_width = fWidth;
dword   temp_height = fHeight;
dword   temp_bits_per_pixel = fBitsPerPixel;
dword   temp_refresh_rate = fRefreshRate;

// quickly scan all modes except _CustomDisplayMode and get the appropriate
//                                                          mode information
for (int i=1;i<Num_DDDisplayModes;i++)
    {
    if (DDDisplayModeInfo.Mode[i] == pDisplayMode)
        {
        temp_width = DDDisplayModeInfo.Width[i];
        temp_height = DDDisplayModeInfo.Height[i];
        temp_bits_per_pixel = DDDisplayModeInfo.BitsPerPixel[i];
        temp_refresh_rate = 0;
        break;
        }
    }

// try to set the display mode
SetDisplayMode( temp_width, temp_height, temp_bits_per_pixel,
                                                        temp_refresh_rate );
// note: if the display mode was set correctly, the display mode properties
// will already be setup correctly. see SetCooperativeLevel() for reasoning
if (!fCreated)
    {
    fDisplayMode = pDisplayMode;
    fWidth = temp_width;
    fHeight = temp_height;
    fBitsPerPixel = temp_bits_per_pixel;
    fRefreshRate = temp_refresh_rate;
    }
}

// --------------------------------------------------------------------------
// Method:          GetHeight
// Description:     Returns the current default height of the display
// Note:            The Create() method will attempt to use this value when
//                  setting the default display mode
// --------------------------------------------------------------------------
dword       __fastcall  TDirectDraw::FGetHeight()
{
return fHeight;
}

// --------------------------------------------------------------------------
// Method:          SetHeight
// Description:     Sets the default height of the display
// Note:            The Create() method will attempt to use this value when
//                  setting the default display mode
// --------------------------------------------------------------------------
void                __fastcall  TDirectDraw::FSetHeight( dword pHeight )
{
fHeight = pHeight;
}

// --------------------------------------------------------------------------
// Method:          GetMonitorFrequency
// Description:     Returns the current monitor frequency setting
// Note:            Returns 0 if the directx call failed
// --------------------------------------------------------------------------
dword               __fastcall  TDirectDraw::FGetMonitorFrequency()
{
dword monitor_frequency = 0;
GetMonitorFrequency( &monitor_frequency );
return monitor_frequency;
}

// --------------------------------------------------------------------------
// Method:          SetMonitorFrequency                                         //added set
// Description:     Sets the monitor frequency
// --------------------------------------------------------------------------
void                __fastcall  TDirectDraw::FSetMonitorFrequency( dword
                                                        pMonitorFrequency )
{
return;
}

// --------------------------------------------------------------------------
// Method:          GetPrimarySurface
// Description:     Returns a pointer to the primary surface
// --------------------------------------------------------------------------
TDirectDrawSurface* __fastcall  TDirectDraw::FGetPrimarySurface()
{
return fPrimarySurface;
}

// --------------------------------------------------------------------------
// Method:          SetPrimarySurface
// Description:     Sets a pointer to the primary surface
// --------------------------------------------------------------------------
void                __fastcall  TDirectDraw::FSetPrimarySurface(
                                        TDirectDrawSurface* pPrimarySurface )
{
if (pPrimarySurface!=NULL) fPrimarySurface = pPrimarySurface;
}

// --------------------------------------------------------------------------
// Method:          GetPtr
// Description:     Returns a pointer to access the directx LPDIRECTDRAW2
//                  variable
// --------------------------------------------------------------------------
LPDIRECTDRAW2       __fastcall  TDirectDraw::FGetPtr()
{
return fDirectDraw2;
}

// --------------------------------------------------------------------------
// Method:          SetPtr
// Description:     Sets a pointer to access the directx LPDIRECTDRAW2
//                  variable
// --------------------------------------------------------------------------
void                __fastcall  TDirectDraw::FSetPtr( LPDIRECTDRAW2 pPtr )
{
if (pPtr!=NULL)
    {
    if (fCreated) Release();
    fDirectDraw2 = pPtr;
    fCreated = true;
    }
else
    {
    fDirectDraw2 = NULL;
    fCreated = false;
    }
}

// --------------------------------------------------------------------------
// Method:          GetRefreshRate
// Description:     Returns the default refresh rate for the display
// Note:            The Create() method will attempt to use this value when
//                  setting the default display mode
// --------------------------------------------------------------------------
dword       __fastcall  TDirectDraw::FGetRefreshRate()
{
return fRefreshRate;
}

// --------------------------------------------------------------------------
// Method:          SetRefreshRate
// Description:     Sets the default refresh rate for the display
// Note:            The Create() method will attempt to use this value when
//                  setting the default display mode
// --------------------------------------------------------------------------
void                __fastcall  TDirectDraw::FSetRefreshRate(
                                                        dword pRefreshRate )
{
fRefreshRate = pRefreshRate;
}

// --------------------------------------------------------------------------
// Method:          GetScanLine
// Description:     Returns the current scan line
// Note:            Returns 0 if the directx call failed
// --------------------------------------------------------------------------
dword               __fastcall  TDirectDraw::FGetScanLine()
{
dword scan_line = 0;
GetScanLine( &scan_line );
return scan_line;
}

// --------------------------------------------------------------------------
// Method:          SetScanLine                                                 //added set
// Description:     Sets the current scan line
// --------------------------------------------------------------------------
void                __fastcall  TDirectDraw::FSetScanLine( dword pScanLine )
{
return;
}

// --------------------------------------------------------------------------
// Method:          GetUseFormHandle
// Description:     Returns the variable indicating which window handle is
//                  used when setting the CooperativeLevel property
// Note:            If true, this TDirectDraw object is "owned" by the form,
//                  meaning that all the default CooperativeLevel values are
//                  set using the parent Form's ->Handle
//                  If false, this TDirectDraw object is "owned" by the
//                  application, meaning that the default CooperativeLevel
//                  values are set using the Application's ->Handle
// --------------------------------------------------------------------------
bool                __fastcall  TDirectDraw::FGetUseFormHandle()
{
return fUseFormHandle;
}

// --------------------------------------------------------------------------
// Method:          SetUseFormHandle
// Description:     Sets the variable indicating which window handle is used
//                  when setting the CooperativeLevel property
// Note:            If true, this TDirectDraw object is "owned" by the form,
//                  meaning that all the default CooperativeLevel values are
//                  set using the parent Form's ->Handle
//                  If false, this TDirectDraw object is "owned" by the
//                  application, meaning that the default CooperativeLevel
//                  values are set using the Application's ->Handle
// --------------------------------------------------------------------------
void                __fastcall  TDirectDraw::FSetUseFormHandle(
                                                        bool pUseFormHandle )
{
if (!fCreated) fUseFormHandle = pUseFormHandle;
}

// --------------------------------------------------------------------------
// Method:          GetVerticalBlankStatus
// Description:     Returns the current status of the vertical blank
// Note:            Returns 0 if the directx call failed
// --------------------------------------------------------------------------
bool                __fastcall  TDirectDraw::FGetVerticalBlankStatus()
{
bool vertical_blank_status = false;
GetVerticalBlankStatus( &vertical_blank_status );
return vertical_blank_status;
}

// --------------------------------------------------------------------------
// Method:          SetVerticalBlankStatus                                      //added set
// Description:     Sets the status of the vertical blank
// --------------------------------------------------------------------------
void                __fastcall  TDirectDraw::FSetVerticalBlankStatus( bool
                                                    pVerticalBlankStatus )
{
return;
}

// --------------------------------------------------------------------------
// Method:          GetWidth
// Description:     Returns the current default width of the display
// Note:            The Create() method will attempt to use this value when
//                  setting the default display mode
// --------------------------------------------------------------------------
dword       __fastcall  TDirectDraw::FGetWidth()
{
return fWidth;
}

// --------------------------------------------------------------------------
// Method:          SetWidth
// Description:     Sets the default width of the display
// Note:            The Create() method will attempt to use this value when
//                  setting the default display mode
// --------------------------------------------------------------------------
void                __fastcall  TDirectDraw::FSetWidth( dword pWidth )
{
fWidth = pWidth;
}

// --------------------------------------------------------------------------
long                __stdcall   DDEnumCallback( GUID FAR *lpGUID,
                                                LPSTR lpDriverDescription,
                                                LPSTR lpDriverName,
                                                LPVOID lpContext )
{
/*
if (FOnEnumCallback)
    {
    AnsiString description = *lpDriverDescription;
    AnsiString drivername = *lpDriverName;
    FOnEnumCallback( this, lpGUID, description, drivername, lpContext );
    return true;
    }
*/
return false;
}

// --------------------------------------------------------------------------
long                __stdcall   DDEnumCallbackEx( GUID FAR *lpGUID,
                                                LPSTR lpDriverDescription,
                                                LPSTR lpDriverName,
                                                LPVOID lpContext,
                                                HMONITOR hm )
{
/*
if (FOnEnumCallbackEx)
    {
    AnsiString description = *lpDriverDescription;
    AnsiString drivername = *lpDriverName;
    FOnEnumCallbackEx( this, lpGUID, description, drivername, lpContext, hm );
    return true;
    }
*/
return false;
}

// --------------------------------------------------------------------------
long                __stdcall   DDEnumModesCallback(
                                            LPDDSURFACEDESC lpDDSurfaceDesc,
                                            LPVOID lpContext )
{
if (DDEnumModesCallbackOwner==NULL) return false;   // caJUSTse
if (DDEnumModesCallbackOwner->OnEnumModesCallback!=NULL)
    {
    TDDSurfaceDesc* description = new TDDSurfaceDesc();
    if (description!=NULL) CopyMemory( description->Ptr(), lpDDSurfaceDesc,
                                                    description->Size() );
    DDEnumModesCallbackOwner->OnEnumModesCallback( DDEnumModesCallbackOwner,
                                                    description, lpContext );
    delete description;
    return true;
    }
return false;
}

// --------------------------------------------------------------------------
long                __stdcall   DDEnumSurfacesCallback(
                                            LPDIRECTDRAWSURFACE lpDDSurface,
                                            LPDDSURFACEDESC lpDDSurfaceDesc,
                                            LPVOID lpContext )
{
if (DDEnumSurfacesCallbackOwner==NULL) return false;   // caJUSTse
if (DDEnumSurfacesCallbackOwner->OnEnumSurfacesCallback!=NULL)
    {
    // scan through all components in the application and find the
    // appropriate surface
    TDirectDrawSurface* surface = NULL;
    for (int form=0;form<Screen->FormCount;form++)
        for
        (int control=0;control<Screen->Forms[form]->ControlCount;control++)
            if (Screen->Forms[form]->Controls[control]->ClassNameIs
                                                    ("TDirectDrawSurface"))
                {
                TDirectDrawSurface* tmpsurface = (TDirectDrawSurface*)
                                    Screen->Forms[form]->Controls[control];
                if (tmpsurface->Ptr == (LPDIRECTDRAWSURFACE3)lpDDSurface)
                    {
                    surface = tmpsurface;
                    break;
                    }
                }

    // create a temporary description
    TDDSurfaceDesc* description = new TDDSurfaceDesc();
    if (description!=NULL) CopyMemory( description->Ptr(), lpDDSurfaceDesc,
                                                    description->Size() );
    // call our VCL callback wrapper
    DDEnumSurfacesCallbackOwner->OnEnumSurfacesCallback(
            DDEnumSurfacesCallbackOwner, surface, description, lpContext );
    // delete temporary description
    delete description;
    return true;
    }
return false;
}

// --------------------------------------------------------------------------
long                __stdcall   DDEnumVideoCallback(
                                        LPDDVIDEOPORTCAPS lpDDVideoPortCaps,
                                        LPVOID lpContext )
{
/*
typedef __fastcall  void (__closure* TDirectDrawEnumVideoCallback)
                         ( TObject*             Sender,
                           TDDVideoPortCaps*    TheVideoPortCaps,
                           void*                TheContext );
*/
return false;
}
// --------------------------------------------------------------------------

