// --------------------------------------------------------------------------
#include <vcl\vcl.h>
#pragma hdrstop

// ==========================================================================
// File:            TDirectDrawSurface.CPP
// Description:     This file contains the code for the TDirectDrawSurface
//                  component
//
// (c) 1998 A.S.C. Alternate Software Concepts, Sydney, Australia.
// All Rights Reserved. Refer to the file TDirectX_Licence.Txt for detailed
// licence & copyright information.
// ==========================================================================

// --------------------------------------------------------------------------
// Include the TDirectDrawSurface H File
// --------------------------------------------------------------------------
#include "TDirectDrawSurface.h"

// --------------------------------------------------------------------------
// Method:          Constructor
// Description:     Sets the default values used within TDirectDrawSurface
//                  and creates any miscellaneous internal objects
//---------------------------------------------------------------------------
                    __fastcall TDirectDrawSurface::TDirectDrawSurface
                                    (TComponent* Owner) : TComponent(Owner)
{
//ShowMessage("Creating TDirectDrawSurface");
fCreated = false;
fError = DD_OK;
fSurfacePtr = NULL;

fDescription = new TDDSurfaceDesc();
fCKDestBlt = new TDDColorKey();
fCKDestOverlay = new TDDColorKey();
fCKSrcBlt = new TDDColorKey();
fCKSrcOverlay = new TDDColorKey();
fPixelFormat = new TDDPixelFormat();

Type = _BackBuffer;

Height = 640;
Width = 480;
BackBufferCount = 0;
AlphaBitDepth = 0;
MipMapCount = 0;        // note:    dwMipMapCount, dwZBufferBitDepth and
ZBufferBitDepth = 0;    //          dwRefreshRate are union'd, meaning that
RefreshRate = 0;        //          if you change one inside the property
                        //          editor, they will all change

// quickly scan other components to see if a TDirectDraw object hass already
// been placed onto the form. If there is one available, set it up
// automatically for (dword i=0;i<Screen->ActiveForm->Designer->Form->
//                                                        ComponentCount;i++)
// if (Screen->ActiveForm->Designer->Form->Components[i]->
//                                                ClassNameIs("TDirectDraw"))
//        DirectDraw = (TDirectDraw*) Screen->ActiveForm->Designer->Form->
//                                                             Components[i];
}

// --------------------------------------------------------------------------
// Method:          Destructor
// Description:     Release / delete anything created in the constructor
// --------------------------------------------------------------------------
                    __fastcall TDirectDrawSurface::~TDirectDrawSurface()
{
//ShowMessage("Destroying TDirectDrawSurface");
delete fPixelFormat;
delete fCKSrcOverlay;
delete fCKSrcBlt;
delete fCKDestOverlay;
delete fCKDestBlt;
delete fDescription;
}

// --------------------------------------------------------------------------
// Method:          AddAttachedSurface
// Description:     This method attaches another surface to the surface
//                  contained in this object
// Parameters:      surface - this is the object that is to be attached to
//                          the current surface
// --------------------------------------------------------------------------
HRESULT	            __fastcall	TDirectDrawSurface::AddAttachedSurface(
                                                TDirectDrawSurface* surface )
{
// check pre-requisites
if (!fCreated)
    {
    if (FOnError) FOnError(this,"TDirectDrawSurface::AddAttachedSurface()",
            "TDX_NOTCREATED",
                "The TDirectDrawSurface interface has NOT been created" );
    return TDX_NOTCREATED;
    }
// do something with DirectX
FOnMessage(this,"TDirectDrawSurface::AddAttachedSurface()",
                                        "about to add an attached surface");
//DJD
//fError = fSurfacePtr->AddAttachedSurface();
// everything worked as expected
FOnMessage(this,"TDirectDrawSurface::AddAttachedSurface()",
                                "the attached surface was added correctly");
// at this point fError must contain DD_OK, simply return it.
return fError;
}

// --------------------------------------------------------------------------
// Method:          AddOverlayDirtyRect
// Description:     This function is not yet implemented in the DirectX SDK
// --------------------------------------------------------------------------
HRESULT	            __fastcall	TDirectDrawSurface::AddOverlayDirtyRect()
{
// check pre-requisites
if (!fCreated)
    {
    if (FOnError) FOnError(this,"TDirectDrawSurface::AddOverlayDirtyRect()",
            "TDX_NOTCREATED",
                "The TDirectDrawSurface interface has NOT been created" );
    return TDX_NOTCREATED;
    }
// do something with DirectX
FOnMessage(this,"TDirectDrawSurface::AddOverlayDirtyRect()",
                                    "about to add an overlay dirty rect");
//DJD
//fError = fSurfacePtr->AddOverlayDirtyRect();
// everything worked as expected
FOnMessage(this,"TDirectDrawSurface::AddOverlayDirtyRect()",
                            "the overlay dirty rect was added correctly");
// at this point fError must contain DD_OK, simply return it.
return fError;
}

// --------------------------------------------------------------------------
// Method:          Blt
// Description:     This method performs a 'bitBlt', 'Blt', or bit-block
//                  transfer, to copy pixels from one surface to another -
//                  from a source surface to a destination surface
// Parameters:      destrect    - this defines a rectangular region on the
//                              destination surface that will be overwritten
//                              by pixels transferred during the Blt. If this
//                              rectangle is an empty rectangle
//                              (ie. Left=0,Top=0, Right=0,Bottom=0) then all
//                              pixels on the destination surface will be
//                              replaced with pixels from the source surface
//                  srcsurface  - this is the source surface containing the
//                              pixels to be transferred during the Blt
//                  srcrect     - this defines a rectangular region on the
//                              source surface that contains the pixels that
//                              will be copied during the Blt. If this
//                              rectangle is an empty rectangle
//                              (Left=0,Top=0,Right=0,Bottom=0)
//                               then all pixels on the source surface will
//                               be copied to the destination surface
//                  flags       - these flags define the various pixel
//                              manipulations that can occur during the Blt
//                  bltfx       - this structure contains additional effects
//                              to be used during the Blt
// --------------------------------------------------------------------------
HRESULT	            __fastcall	TDirectDrawSurface::Blt( TRect& destrect,
                                        TDirectDrawSurface* srcsurface,
                                        TRect& srcrect, dword flags,
                                        TDDBltFX* bltfx )
{
// check pre-requisites
if (!fCreated)
    {
    if (FOnError) FOnError(this,"TDirectDrawSurface::Blt()","TDX_NOTCREATED",
                "The TDirectDrawSurface interface has NOT been created" );
    return TDX_NOTCREATED;
    }
// do something with DirectX
FOnMessage(this,"TDirectDrawSurface::Blt()", "about to blit the surface");
//DJD
//fError = fSurfacePtr->Blt();
// everything worked as expected
FOnMessage(this,"TDirectDrawSurface::Blt()",
                                        "the surface was blitted correctly");
// at this point fError must contain DD_OK, simply return it.
return fError;
}

// --------------------------------------------------------------------------
// Method:          BltBatch
// Description:     Performs a sequence of Blt operations from several
//                  sources to a single destination
// Note:            This method has not yet been implemented
// --------------------------------------------------------------------------
HRESULT	            __fastcall	TDirectDrawSurface::BltBatch()
{
// check pre-requisites
if (!fCreated)
    {
    if (FOnError) FOnError(this,"TDirectDrawSurface::BltBatch()",
            "TDX_NOTCREATED",
                "The TDirectDrawSurface interface has NOT been created" );
    return TDX_NOTCREATED;
    }
// do something with DirectX
FOnMessage(this,"TDirectDrawSurface::BltBatch()",
                                        "about to batch blit the surface");
//DJD
//fError = fSurfacePtr->BltBatch();
// everything worked as expected
FOnMessage(this,"TDirectDrawSurface::BltBatch()",
                                "the surface was batch blitted correctly");
// at this point fError must contain DD_OK, simply return it.
return fError;
}

// --------------------------------------------------------------------------
// Method:          BltFast
// Description:     Performs a source copy blit or transparent blit by using
//                  a source color key or destination color key
// Note:            Always attempts an asynchronous blit if H/W supported
//                  Only works on display memory surfaces without clipping
//                  10% faster if emulated, no advantage if hardware used
// --------------------------------------------------------------------------
HRESULT	            __fastcall	TDirectDrawSurface::BltFast()
{
// check pre-requisites
if (!fCreated)
    {
    if (FOnError) FOnError(this,"TDirectDrawSurface::BltFast()",
            "TDX_NOTCREATED",
                "The TDirectDrawSurface interface has NOT been created" );
    return TDX_NOTCREATED;
    }
// do something with DirectX
FOnMessage(this,"TDirectDrawSurface::BltFast()",
                                        "about to fast blit the surface");
//DJD
//fError = fSurfacePtr->BltFast();
// everything worked as expected
FOnMessage(this,"TDirectDrawSurface::BltFast()",
                                "the surface was fast blitted correctly");
// at this point fError must contain DD_OK, simply return it.
return fError;
}

// --------------------------------------------------------------------------
// Method:          Create
// Description:     Call this method to create the internals of the
//                  TDirectDrawSurface object
// Notes:           Remember to call the Release() method when finished
// --------------------------------------------------------------------------
HRESULT	            __fastcall	TDirectDrawSurface::Create()
{
// check pre-requisites
if (fCreated)
    {
    if (FOnError) FOnError(this,"TDirectDrawSurface::Create()",
        "TDX_ALREADYCREATED",
            "The TDirectDrawSurface interface has already been created" );
    return TDX_ALREADYCREATED;
    }
// do something with DirectX
FOnMessage(this,"TDirectDrawSurface::Create()",
            "creating the internals of the TDirectDrawSurface interface");

// ___actually.... do nothing with DirectX____

// everything worked as expected
FOnMessage(this,"TDirectDrawSurface::Create()",
        "the TDirectDrawSurface interface internals were created correctly");
fCreated = (fSurfacePtr!=NULL);
// at this point fError must contain DD_OK, simply return it.
fError = DD_OK;
return fError;
}

// --------------------------------------------------------------------------
// Method:          DeleteAttachedSurface
// Description:     Detaches two attached surfaces
// Note:            The detached surface is not released
//                  Attachments not added with AddAttachedSurface cannot be
//                  detached
// --------------------------------------------------------------------------
HRESULT	            __fastcall	TDirectDrawSurface::DeleteAttachedSurface()
{
// check pre-requisites
if (!fCreated)
    {
    if (FOnError) FOnError(this,"TDirectDrawSurface::DeleteAttachedSurface()"
            ,"TDX_NOTCREATED",
                "The TDirectDrawSurface interface has NOT been created" );
    return TDX_NOTCREATED;
    }
// do something with DirectX
FOnMessage(this,"TDirectDrawSurface::DeleteAttachedSurface()",
                                    "about to delete an attached surface");
//fError = fSurfacePtr->DeleteAttachedSurface();
// everything worked as expected
FOnMessage(this,"TDirectDrawSurface::DeleteAttachedSurface()",
                            "the attached surface was deleted correctly");
// at this point fError must contain DD_OK, simply return it.
return fError;
}

// --------------------------------------------------------------------------
// Method:          EnumAttachedSurfaces
// Description:     Enumerates all the surfaces attached to a given surface
// --------------------------------------------------------------------------
HRESULT	            __fastcall	TDirectDrawSurface::EnumAttachedSurfaces()
{
// check pre-requisites
if (!fCreated)
    {
    if (FOnError) FOnError(this,"TDirectDrawSurface::EnumAttachedSurfaces()",
            "TDX_NOTCREATED",
                "The TDirectDrawSurface interface has NOT been created" );
    return TDX_NOTCREATED;
    }
// do something with DirectX
FOnMessage(this,"TDirectDrawSurface::EnumAttachedSurfaces()",
                                "about to enumerate the attached surfaces");
//DJD
//fError = fSurfacePtr->EnumAttachedSurfaces();
// everything worked as expected
FOnMessage(this,"TDirectDrawSurface::EnumAttachedSurfaces()",
                        "the attached surfaces were enumerated correctly");
// at this point fError must contain DD_OK, simply return it.
return fError;
}

// --------------------------------------------------------------------------
// Method:          EnumOverlayZOrders
// Description:     Enumerates overlay surfaces on a specified destination
// Note:            Overlays can be enumerated in front-to-back or
//                  back-to-front order
// --------------------------------------------------------------------------
HRESULT	            __fastcall	TDirectDrawSurface::EnumOverlayZOrders()
{
// check pre-requisites
if (!fCreated)
    {
    if (FOnError) FOnError(this,"TDirectDrawSurface::EnumOverlayZOrders()",
            "TDX_NOTCREATED",
                "The TDirectDrawSurface interface has NOT been created" );
    return TDX_NOTCREATED;
    }
// do something with DirectX
FOnMessage(this,"TDirectDrawSurface::EnumOverlayZOrders()",
                                "about to enumerate the overlay z orders");
//DJD
//fError = fSurfacePtr->EnumOverlayZOrders();
// everything worked as expected
FOnMessage(this,"TDirectDrawSurface::EnumOverlayZOrders()",
                        "the overlay z orders were enumerated correctly");
// at this point fError must contain DD_OK, simply return it.
return fError;
}

// --------------------------------------------------------------------------
// Method:          Flip
// Description:     Associates DDSCAPS_BACKBUFFER surface memory with the
//                  front-buffer surface
// Note:            Can only be called for surfaces that have DDSCAPS_FLIP
//                  and DDSCAPS_FRONTBUFFER capabilities
//                  Display memory previously associated with the front
//                  buffer is associated with the back buffer
//                  Always synchronized with the vertical blank
//                  If the surface has been assigned to a video port, this
//                  method updates the visible overlay surface and the video
//                  port's target surface
// --------------------------------------------------------------------------
HRESULT	            __fastcall	TDirectDrawSurface::Flip()
{
// check pre-requisites
if (!fCreated)
    {
    if (FOnError) FOnError(this,"TDirectDrawSurface::Flip()","TDX_NOTCREATED"
                ,"The TDirectDrawSurface interface has NOT been created" );
    return TDX_NOTCREATED;
    }
// do something with DirectX
FOnMessage(this,"TDirectDrawSurface::Flip()", "about to flip the surface");
//DJD
//fError = fSurfacePtr->Flip();
// everything worked as expected
FOnMessage(this,"TDirectDrawSurface::Flip()",
                                        "the surface was flipped correctly");
// at this point fError must contain DD_OK, simply return it.
return fError;
}

// --------------------------------------------------------------------------
// Method:          GetAttachedSurface
// Description:     Obtains the attached surface with the specified
//                  capabilities
// Note:            Fails if multiple surfaces match the specified
//                  capabilities
// --------------------------------------------------------------------------
HRESULT	            __fastcall	TDirectDrawSurface::GetAttachedSurface(
                                                TDDSCaps* scaps,
                                                TDirectDrawSurface* surface )
{
// check pre-requisites
if (!fCreated)
    {
    if (FOnError) FOnError(this,"TDirectDrawSurface::GetAttachedSurface()",
            "TDX_NOTCREATED",
                "The TDirectDrawSurface interface has NOT been created" );
    return TDX_NOTCREATED;
    }
if (scaps==NULL)
    {
    if (FOnError) FOnError(this,"TDirectDrawSurface::GetAttachedSurface()",
                            "TDX_BADPARAMS","the 'scaps' parameter is NULL");
    return TDX_BADPARAMS;
    }
if (surface==NULL)
    {
    if (FOnError) FOnError(this,"TDirectDrawSurface::GetAttachedSurface()",
                        "TDX_BADPARAMS","the 'surface' parameter is NULL");
    return TDX_BADPARAMS;
    }
// do something with DirectX
FOnMessage(this,"TDirectDrawSurface::GetAttachedSurface()",
                                    "about to retrieve an attached surface");
LPDIRECTDRAWSURFACE3 attached_surface_ptr;
fError = fSurfacePtr->GetAttachedSurface(scaps->Ptr(),&attached_surface_ptr);
if (fError!=DD_OK)
    {
    if (FOnError) FOnError(this,"TDirectDrawSurface::GetAttachedSurface()",
            DirectDrawErrorString(fError),DirectDrawErrorMessage(fError));
    return fError;
    }
// everything worked as expected
FOnMessage(this,"TDirectDrawSurface::GetAttachedSurface()",
                            "the attached surface was retrieved correctly");
surface->Ptr = attached_surface_ptr;
// at this point fError must contain DD_OK, simply return it.
return fError;
}

// --------------------------------------------------------------------------
// Method:          GetBltStatus
// Description:     Obtains the blitter status
//---------------------------------------------------------------------------
HRESULT	            __fastcall	TDirectDrawSurface::GetBltStatus()
{
// check pre-requisites
if (!fCreated)
    {
    if (FOnError) FOnError(this,"TDirectDrawSurface::GetBltStatus()",
            "TDX_NOTCREATED",
                "The TDirectDrawSurface interface has NOT been created" );
    return TDX_NOTCREATED;
    }
// do something with DirectX
FOnMessage(this,"TDirectDrawSurface::GetBltStatus()",
                                        "about to retrieve the blit status");
//DJD
//fError = fSurfacePtr->GetBltStatus();
// everything worked as expected
FOnMessage(this,"TDirectDrawSurface::GetBltStatus()",
                                "the blit status was retrieved correctly");
// at this point fError must contain DD_OK, simply return it.
return fError;
}

// --------------------------------------------------------------------------
// Method:          GetCaps
// Description:     Retrieves the capabilities of the surface
// Note:            These capabilities are not necessarily related to the
//                  capabilities of the display device
//---------------------------------------------------------------------------
HRESULT	            __fastcall	TDirectDrawSurface::GetCaps()
{
// check pre-requisites
if (!fCreated)
    {
    if (FOnError) FOnError(this,"TDirectDrawSurface::GetCaps()",
            "TDX_NOTCREATED",
                "The TDirectDrawSurface interface has NOT been created" );
    return TDX_NOTCREATED;
    }
// do something with DirectX
FOnMessage(this,"TDirectDrawSurface::GetCaps()",
                        "about to retrieve the capabilities of the surface");
//DJD
//fError = fSurfacePtr->GetCaps();
// everything worked as expected
FOnMessage(this,"TDirectDrawSurface::GetCaps()",
            "the capabilities of the surface were retrieved successfully");
// at this point fError must contain DD_OK, simply return it.
return fError;
}

// --------------------------------------------------------------------------
// Method:          GetClipper
// Description:     Retrieves the DirectDrawClipper object associated with
//                  this surface
//---------------------------------------------------------------------------
HRESULT	            __fastcall	TDirectDrawSurface::GetClipper()
{
// check pre-requisites
if (!fCreated)
    {
    if (FOnError) FOnError(this,"TDirectDrawSurface::GetClipper()",
            "TDX_NOTCREATED",
                "The TDirectDrawSurface interface has NOT been created" );
    return TDX_NOTCREATED;
    }
// do something with DirectX
FOnMessage(this,"TDirectDrawSurface::GetClipper()",
                                    "about to retrieve the surface clipper");
//DJD
//fError = fSurfacePtr->GetClipper();
// everything worked as expected
FOnMessage(this,"TDirectDrawSurface::GetClipper()",
                            "the surface clipper was retrieved correctly");
// at this point fError must contain DD_OK, simply return it.
return fError;
}

// --------------------------------------------------------------------------
// Method:          GetColorKey
// Description:     Retrieves the color key value for the surface object
//---------------------------------------------------------------------------
HRESULT	            __fastcall	TDirectDrawSurface::GetColorKey()
{
// check pre-requisites
if (!fCreated)
    {
    if (FOnError) FOnError(this,"TDirectDrawSurface::GetColorKey()",
            "TDX_NOTCREATED",
                "The TDirectDrawSurface interface has NOT been created" );
    return TDX_NOTCREATED;
    }
// do something with DirectX
FOnMessage(this,"TDirectDrawSurface::GetColorKey()",
                                "about to retrieve the surface color key");
//DJD
//fError = fSurfacePtr->GetColorKey();
// everything worked as expected
FOnMessage(this,"TDirectDrawSurface::GetColorKey()",
                            "the surface color key was retrieved correctly");
// at this point fError must contain DD_OK, simply return it.
return fError;
}

// --------------------------------------------------------------------------
// Method:          GetDC
// Description:     Creates a GDI-compatible handle of a device context for
//                  the surface
// Note:            Locks the surface
//                  Surface remains locked until the ReleaseDC is called
//---------------------------------------------------------------------------
HRESULT	            __fastcall	TDirectDrawSurface::GetDC()
{
// check pre-requisites
if (!fCreated)
    {
    if (FOnError) FOnError(this,"TDirectDrawSurface::GetDC()",
            "TDX_NOTCREATED",
                "The TDirectDrawSurface interface has NOT been created" );
    return TDX_NOTCREATED;
    }
// do something with DirectX
FOnMessage(this,"TDirectDrawSurface::GetDC()",
                            "about to retrieve the surface device context");
//DJD
//fError = fSurfacePtr->GetDC();
// everything worked as expected
FOnMessage(this,"TDirectDrawSurface::GetDC()",
                    "the surface device context was retrieved correctly");
// at this point fError must contain DD_OK, simply return it.
return fError;
}

// --------------------------------------------------------------------------
// Method:          GetDDInterface
// Description:     Retrieves an interface to the DirectDraw object that was
//                  used to create the surface
//---------------------------------------------------------------------------
HRESULT	            __fastcall	TDirectDrawSurface::GetDDInterface()
{
// check pre-requisites
if (!fCreated)
    {
    if (FOnError) FOnError(this,"TDirectDrawSurface::GetDDInterface()",
            "TDX_NOTCREATED",
                "The TDirectDrawSurface interface has NOT been created" );
    return TDX_NOTCREATED;
    }
// do something with DirectX
FOnMessage(this,"TDirectDrawSurface::GetDDInterface()",
                    "about to retrieve the surface's directdraw interface");
//DJD
//fError = fSurfacePtr->GetDDInterface();
// everything worked as expected
FOnMessage(this,"TDirectDrawSurface::GetDDInterface()",
            "the surface's directdraw interface was retrieved correctly");
// at this point fError must contain DD_OK, simply return it.
return fError;
}

// --------------------------------------------------------------------------
// Method:          GetFlipStatus
// Description:     Indicates whether the surface has finished flipping
//---------------------------------------------------------------------------
HRESULT	            __fastcall	TDirectDrawSurface::GetFlipStatus()
{
// check pre-requisites
if (!fCreated)
    {
    if (FOnError) FOnError(this,"TDirectDrawSurface::GetFlipStatus()",
            "TDX_NOTCREATED",
                "The TDirectDrawSurface interface has NOT been created" );
    return TDX_NOTCREATED;
    }
// do something with DirectX
FOnMessage(this,"TDirectDrawSurface::GetFlipStatus()",
                                        "about to retrieve the flip status");
//fError = fSurfacePtr->GetFlipStatus();
// everything worked as expected
FOnMessage(this,"TDirectDrawSurface::GetFlipStatus()",
                                "the flip status was retrieved correctly");
// at this point fError must contain DD_OK, simply return it.
return fError;
}

// --------------------------------------------------------------------------
// Method:          GetOverlayPosition
// Description:     Given a visible, active overlay surface (DDSCAPS_OVERLAY
//                  flag set), this returns the display coordinates of the
//                  surface
//---------------------------------------------------------------------------
HRESULT	            __fastcall	TDirectDrawSurface::GetOverlayPosition()
{
// check pre-requisites
if (!fCreated)
    {
    if (FOnError) FOnError(this,"TDirectDrawSurface::GetOverlayPosition()",
            "TDX_NOTCREATED",
                "The TDirectDrawSurface interface has NOT been created" );
    return TDX_NOTCREATED;
    }
// do something with DirectX
FOnMessage(this,"TDirectDrawSurface::GetOverlayPosition()",
                                "about to retrieve the overlay position");
//DJD
//fError = fSurfacePtr->GetOverlayPosition();
// everything worked as expected
FOnMessage(this,"TDirectDrawSurface::GetOverlayPosition()",
                            "the overlay position was retrieved correctly");
// at this point fError must contain DD_OK, simply return it.
return fError;
}

// --------------------------------------------------------------------------
// Method:          GetPalette
// Description:     Retrieves the palette structure associated with this
//                  surface and increments the reference count of the
//                  returned palette
//---------------------------------------------------------------------------
HRESULT	            __fastcall	TDirectDrawSurface::GetPalette()
{
// check pre-requisites
if (!fCreated)
    {
    if (FOnError) FOnError(this,"TDirectDrawSurface::GetPalette()",
            "TDX_NOTCREATED",
                "The TDirectDrawSurface interface has NOT been created" );
    return TDX_NOTCREATED;
    }
// do something with DirectX
FOnMessage(this,"TDirectDrawSurface::GetPalette()",
                                    "about to retrieve the surface palette");
//DJD
//fError = fSurfacePtr->GetPalette();
// everything worked as expected
FOnMessage(this,"TDirectDrawSurface::GetPalette()",
                            "the surface palette was retrieved correctly");
// at this point fError must contain DD_OK, simply return it.
return fError;
}

// --------------------------------------------------------------------------
// Method:          GetPixelFormat
// Description:     Retrieves the color and pixel format of the surface
//---------------------------------------------------------------------------
HRESULT	            __fastcall	TDirectDrawSurface::GetPixelFormat()
{
// check pre-requisites
if (!fCreated)
    {
    if (FOnError) FOnError(this,"TDirectDrawSurface::GetPixelFormat()",
            "TDX_NOTCREATED",
                "The TDirectDrawSurface interface has NOT been created" );
    return TDX_NOTCREATED;
    }
// do something with DirectX
FOnMessage(this,"TDirectDrawSurface::GetPixelFormat()",
                            "about to retrieve the surface pixel format");
//DJD
//fError = fSurfacePtr->GetPixelFormat();
// everything worked as expected
FOnMessage(this,"TDirectDrawSurface::GetPixelFormat()",
                        "the surface pixel format was retrieved correctly");
// at this point fError must contain DD_OK, simply return it.
return fError;
}

// --------------------------------------------------------------------------
// Method:          GetSurfaceDesc
// Description:     Retrieves a DDSURFACEDESC structure that describes the
//                  surface in its current condition
//---------------------------------------------------------------------------
HRESULT	            __fastcall	TDirectDrawSurface::GetSurfaceDesc()
{
// check pre-requisites
if (!fCreated)
    {
    if (FOnError) FOnError(this,"TDirectDrawSurface::GetSurfaceDesc()",
            "TDX_NOTCREATED",
                "The TDirectDrawSurface interface has NOT been created" );
    return TDX_NOTCREATED;
    }
// do something with DirectX
FOnMessage(this,"TDirectDrawSurface::GetSurfaceDesc()",
                                "about to retrieve the surface description");
//DJD
//fError = fSurfacePtr->GetSurfaceDesc();
// everything worked as expected
FOnMessage(this,"TDirectDrawSurface::GetSurfaceDesc()",
                        "the surface description was retrieved correctly");
// at this point fError must contain DD_OK, simply return it.
return fError;
}

// --------------------------------------------------------------------------
// Method:          IsLost
// Description:     Determines if the surface memory associated with a
//                  DirectDrawSurface object has been freed
//---------------------------------------------------------------------------
HRESULT	            __fastcall	TDirectDrawSurface::IsLost()
{
// check pre-requisites
if (!fCreated)
    {
    if (FOnError) FOnError(this,"TDirectDrawSurface::IsLost()",
            "TDX_NOTCREATED",
                "The TDirectDrawSurface interface has NOT been created" );
    return TDX_NOTCREATED;
    }
// do something with DirectX
FOnMessage(this,"TDirectDrawSurface::IsLost()",
                                "about to retrieve the surface lost status");
//DJD
//fError = fSurfacePtr->IsLost();
// everything worked as expected
FOnMessage(this,"TDirectDrawSurface::IsLost()",
                        "the surface lost status was retrieved correctly");
// at this point fError must contain DD_OK, simply return it.
return fError;
}

// --------------------------------------------------------------------------
// Method:          Lock
// Description:     Obtains a pointer to the surface memory
// Note:            After retrieving a surface memory pointer, you can access
//                  the surface memory until a corresponding unlock is called
//                  Once a surface is unlocked, the pointer becomes invalid
//                  Blits from a locked region of a surface will fail
//                  GDI blit functions silently fail when used on locked
//                  video memory surfaces
//---------------------------------------------------------------------------
HRESULT	            __fastcall	TDirectDrawSurface::Lock()
{
// check pre-requisites
if (!fCreated)
    {
    if (FOnError) FOnError(this,"TDirectDrawSurface::Lock()",
            "TDX_NOTCREATED",
                "The TDirectDrawSurface interface has NOT been created" );
    return TDX_NOTCREATED;
    }
// do something with DirectX
FOnMessage(this,"TDirectDrawSurface::Lock()", "about to lock the surface");
//DJD
//fError = fSurfacePtr->Lock();
// everything worked as expected
FOnMessage(this,"TDirectDrawSurface::Lock()",
                                        "the surface was locked correctly");
// at this point fError must contain DD_OK, simply return it.
return fError;
}

// --------------------------------------------------------------------------
// Method:          PageLock
// Description:     Prevents a system-memory surface from being paged out
//                  while a blit operation using direct memory access (DMA)
//                  transfers to or from system memory is in progress
// Note:            This method must be called to use DMA support otherwise
//                  the blit will use software emulation
//                  A lock count is maintained for each surface and is
//                  incremented each time PageLock is called for that surface
//                  Only works on system-memory surfaces, it won't page lock
//                  a display-memory surface or an emulated primary surface
//---------------------------------------------------------------------------
HRESULT	            __fastcall	TDirectDrawSurface::PageLock()
{
// check pre-requisites
if (!fCreated)
    {
    if (FOnError) FOnError(this,"TDirectDrawSurface::PageLock()",
            "TDX_NOTCREATED",
                "The TDirectDrawSurface interface has NOT been created" );
    return TDX_NOTCREATED;
    }
// do something with DirectX
FOnMessage(this,"TDirectDrawSurface::PageLock()",
                                        "about to page lock the surface");
//DJD
//fError = fSurfacePtr->PageLock();
// everything worked as expected
FOnMessage(this,"TDirectDrawSurface::PageLock()",
                                    "the surface was page locked correctly");
// at this point fError must contain DD_OK, simply return it.
return fError;
}

// --------------------------------------------------------------------------
// Method:          PageUnlock
// Description:     Unlocks a system-memory surface, allowing it to be paged
//                  out
// Note:            The lock count is decremented when PageUnlock is called
//                  When the count reaches 0, the memory is unlocked and can
//                  be paged by the operating system
//                  This method works only on system-memory surfaces, it
//                  won't page unlock a display-memory surface or an emulated
//                  primary surface
//---------------------------------------------------------------------------
HRESULT	            __fastcall	TDirectDrawSurface::PageUnlock()
{
// check pre-requisites
if (!fCreated)
    {
    if (FOnError) FOnError(this,"TDirectDrawSurface::PageUnlock()",
            "TDX_NOTCREATED",
                "The TDirectDrawSurface interface has NOT been created" );
    return TDX_NOTCREATED;
    }
// do something with DirectX
FOnMessage(this,"TDirectDrawSurface::PageUnlock()",
                                        "about to page unlock the surface");
//DJD
//fError = fSurfacePtr->PageUnlock();
// everything worked as expected
FOnMessage(this,"TDirectDrawSurface::PageUnlock()",
                                "the surface was page unlocked correctly");
// at this point fError must contain DD_OK, simply return it.
return fError;
}

// --------------------------------------------------------------------------
// Method:          Release
// Description:
// --------------------------------------------------------------------------
HRESULT	            __fastcall	TDirectDrawSurface::Release()
{
// check pre-requisites
if (!fCreated)
    {
    if (FOnError) FOnError(this,"TDirectDrawSurface::Release()",
            "TDX_NOTCREATED",
                "The TDirectDrawSurface interface has NOT been created" );
    return TDX_NOTCREATED;
    }
// do something with DirectX
if (fSurfacePtr!=NULL)
    {
    FOnMessage(this,"TDirectDrawSurface::Release()",
            "releasing the internals of the TDirectDrawSurface interface");
    if (fSurfaceType != _BackBuffer) fError = fSurfacePtr->Release();
    if (fError!=DD_OK)
        {
        if (FOnError) FOnError(this,"TDirectDrawSurface::Release()",
            DirectDrawErrorString(fError),DirectDrawErrorMessage(fError));
        return fError;
        }
    }
// everything worked as expected
FOnMessage(this,"TDirectDrawSurface::Release()",
    "the TDirectDrawSurface interface internals were released correctly");
fSurfacePtr = NULL;
fCreated = false;
// at this point fError must contain DD_OK, simply return it.
return fError;
}

// --------------------------------------------------------------------------
// Method:          ReleaseDC
// Description:     Releases the handle of a device context previously
//                  obtained by GetDC
// Note:            Also unlocks the surface locked when the GetDC was called
//---------------------------------------------------------------------------
HRESULT	            __fastcall	TDirectDrawSurface::ReleaseDC()
{
// check pre-requisites
if (!fCreated)
    {
    if (FOnError) FOnError(this,"TDirectDrawSurface::ReleaseDC()",
            "TDX_NOTCREATED",
                "The TDirectDrawSurface interface has NOT been created" );
    return TDX_NOTCREATED;
    }
// do something with DirectX
FOnMessage(this,"TDirectDrawSurface::ReleaseDC()",
                            "about to release the surface device context");
//DJD
//fError = fSurfacePtr->ReleaseDC();
// everything worked as expected
FOnMessage(this,"TDirectDrawSurface::ReleaseDC()",
                        "the surface device context was released correctly");
// at this point fError must contain DD_OK, simply return it.
return fError;
}

// --------------------------------------------------------------------------
// Method:          Restore
// Description:     Restores a surface that has been lost. Lost surfaces
//                  occur when the surface memory associated with the surface
//                  object has been freed
// Note:            Restores memory allocated for a surface, not the contents
//                  Restore will reallocate surface memory and reattach it to
//                  the DirectDrawSurface object
//                  One call to this method will restore a surface object's
//                  associated implicit surfaces (back buffers, and so on)
//                  Attempts to directly restore an implicitly created
//                  surface will result in an error
//                  Restore will not work across explicit attachments created
//                  using AddAttachedSurface, each of these surfaces must be
//                  restored individually
//---------------------------------------------------------------------------
HRESULT	            __fastcall	TDirectDrawSurface::Restore()
{
// check pre-requisites
if (!fCreated)
    {
    if (FOnError) FOnError(this,"TDirectDrawSurface::Restore()",
            "TDX_NOTCREATED",
                "The TDirectDrawSurface interface has NOT been created" );
    return TDX_NOTCREATED;
    }
// do something with DirectX
FOnMessage(this,"TDirectDrawSurface::Restore()",
                                            "about to restore the surface");
//DJD
//fError = fSurfacePtr->Restore();
// everything worked as expected
FOnMessage(this,"TDirectDrawSurface::Restore()",
                                    "the surface was restored correctly");
// at this point fError must contain DD_OK, simply return it.
return fError;
}

// --------------------------------------------------------------------------
// Method:          SetClipper
// Description:     Attaches a clipper to a surface object
//---------------------------------------------------------------------------
HRESULT	            __fastcall	TDirectDrawSurface::SetClipper()
{
// check pre-requisites
if (!fCreated)
    {
    if (FOnError) FOnError(this,"TDirectDrawSurface::SetClipper()",
            "TDX_NOTCREATED",
                "The TDirectDrawSurface interface has NOT been created" );
    return TDX_NOTCREATED;
    }
// do something with DirectX
FOnMessage(this,"TDirectDrawSurface::SetClipper()",
                                        "about to set the surface clipper");
//DJD
//fError = fSurfacePtr->SetClipper();
// everything worked as expected
FOnMessage(this,"TDirectDrawSurface::SetClipper()",
                                    "the surface clipper was set correctly");
// at this point fError must contain DD_OK, simply return it.
return fError;
}

// --------------------------------------------------------------------------
// Method:          SetColorKey
// Description:     Sets the color key value for the surface object if the
//                  hardware supports color keys on a per surface basis
//---------------------------------------------------------------------------
HRESULT	            __fastcall	TDirectDrawSurface::SetColorKey()
{
// check pre-requisites
if (!fCreated)
    {
    if (FOnError) FOnError(this,"TDirectDrawSurface::SetColorKey()",
            "TDX_NOTCREATED",
                "The TDirectDrawSurface interface has NOT been created" );
    return TDX_NOTCREATED;
    }
// do something with DirectX
FOnMessage(this,"TDirectDrawSurface::SetColorKey()",
                                    "about to set the surface color key");
//DJD
//fError = fSurfacePtr->SetColorKey();
// everything worked as expected
FOnMessage(this,"TDirectDrawSurface::SetColorKey()",
                                "the surface color key was set correctly");
// at this point fError must contain DD_OK, simply return it.
return fError;
}

// --------------------------------------------------------------------------
// Method:          SetOverlayPosition
// Description:     Changes the display coordinates of an overlay surface
//---------------------------------------------------------------------------
HRESULT	            __fastcall	TDirectDrawSurface::SetOverlayPosition()
{
// check pre-requisites
if (!fCreated)
    {
    if (FOnError) FOnError(this,"TDirectDrawSurface::SetOverlayPosition()",
            "TDX_NOTCREATED",
                "The TDirectDrawSurface interface has NOT been created" );
    return TDX_NOTCREATED;
    }
// do something with DirectX
FOnMessage(this,"TDirectDrawSurface::SetOverlayPosition()",
                                "about to set the surface overlay position");
//DJD
//fError = fSurfacePtr->SetOverlayPosition();
// everything worked as expected
FOnMessage(this,"TDirectDrawSurface::SetOverlayPosition()",
                        "the surface overlay position was set correctly");
// at this point fError must contain DD_OK, simply return it.
return fError;
}

// --------------------------------------------------------------------------
// Method:          SetPalette
// Description:     Attaches the specified palette object to a surface
// Note:            Surface uses this palette for all subsequent operations
//                  The palette change takes place immediately, without
//                  regard to refresh timing
//---------------------------------------------------------------------------
HRESULT	            __fastcall	TDirectDrawSurface::SetPalette()
{
// check pre-requisites
if (!fCreated)
    {
    if (FOnError) FOnError(this,"TDirectDrawSurface::SetPalette()",
            "TDX_NOTCREATED",
                "The TDirectDrawSurface interface has NOT been created" );
    return TDX_NOTCREATED;
    }
// do something with DirectX
FOnMessage(this,"TDirectDrawSurface::SetPalette()",
                                        "about to set the surface palette");
//DJD
//fError = fSurfacePtr->SetPalette();
// everything worked as expected
FOnMessage(this,"TDirectDrawSurface::SetPalette()",
                                    "the surface palette was set correctly");
// at this point fError must contain DD_OK, simply return it.
return fError;
}

// --------------------------------------------------------------------------
// Method:          Unlock
// Description:     Notifies DirectDraw that the direct surface manipulations
//                  are complete
//---------------------------------------------------------------------------
HRESULT	            __fastcall	TDirectDrawSurface::Unlock()
{
// check pre-requisites
if (!fCreated)
    {
    if (FOnError) FOnError(this,"TDirectDrawSurface::Unlock()",
            "TDX_NOTCREATED",
                "The TDirectDrawSurface interface has NOT been created" );
    return TDX_NOTCREATED;
    }
// do something with DirectX
FOnMessage(this,"TDirectDrawSurface::Unlock()",
                                            "about to unlock the surface");
//DJD
//fError = fSurfacePtr->Unlock();
// everything worked as expected
FOnMessage(this,"TDirectDrawSurface::Unlock()",
                                    "the surface was unlocked correctly");
// at this point fError must contain DD_OK, simply return it.
return fError;
}

// --------------------------------------------------------------------------
// Method:          UpdateOverlay
// Description:     Repositions or modifies the visual attributes of an
//                  overlay surface
// Note:            These surfaces must have the DDSCAPS_OVERLAY value set
//---------------------------------------------------------------------------
HRESULT	            __fastcall	TDirectDrawSurface::UpdateOverlay()
{
// check pre-requisites
if (!fCreated)
    {
    if (FOnError) FOnError(this,"TDirectDrawSurface::UpdateOverlay()",
            "TDX_NOTCREATED",
                "The TDirectDrawSurface interface has NOT been created" );
    return TDX_NOTCREATED;
    }
// do something with DirectX
FOnMessage(this,"TDirectDrawSurface::UpdateOverlay()",
                                                "about to update overlay");
//DJD
//fError = fSurfacePtr->UpdateOverlay();
// everything worked as expected
FOnMessage(this,"TDirectDrawSurface::UpdateOverlay()",
                                        "the overlay was updated correctly");
// at this point fError must contain DD_OK, simply return it.
return fError;
}

// --------------------------------------------------------------------------
// Method:          UpdateOverlayDisplay
// Description:     Repaints the rectangles in the dirty rectangle list of
//                  all active overlays
// Note:            Clears the dirty rectangle list
//                  Only for software emulation, does nothing if the hardware
//                  supports overlays
//---------------------------------------------------------------------------
HRESULT	            __fastcall	TDirectDrawSurface::UpdateOverlayDisplay()
{
// check pre-requisites
if (!fCreated)
    {
    if (FOnError) FOnError(this,"TDirectDrawSurface::UpdateOverlayDisplay()",
            "TDX_NOTCREATED",
                "The TDirectDrawSurface interface has NOT been created" );
    return TDX_NOTCREATED;
    }
// do something with DirectX
FOnMessage(this,"TDirectDrawSurface::UpdateOverlayDisplay()",
                                        "about to update overlay display");
//DJD
//fError = fSurfacePtr->UpdateOverlayDisplay();
// everything worked as expected
FOnMessage(this,"TDirectDrawSurface::UpdateOverlayDisplay()",
                                "the overlay display was updated correctly");
// at this point fError must contain DD_OK, simply return it.
return fError;
}

// --------------------------------------------------------------------------
// Method:          UpdateOverlayZOrder
// Description:     Sets the z-order of an overlay
//---------------------------------------------------------------------------
HRESULT	            __fastcall	TDirectDrawSurface::UpdateOverlayZOrder()
{
// check pre-requisites
if (!fCreated)
    {
    if (FOnError) FOnError(this,"TDirectDrawSurface::UpdateOverlayZOrder()",
            "TDX_NOTCREATED",
                "The TDirectDrawSurface interface has NOT been created" );
    return TDX_NOTCREATED;
    }
// do something with DirectX
FOnMessage(this,"TDirectDrawSurface::UpdateOverlayZOrder()",
                                    "about to update the overlay z-order");
//DJD
//fError = fSurfacePtr->UpdateOverlayZOrder();
// everything worked as expected
FOnMessage(this,"TDirectDrawSurface::UpdateOverlayZOrder()",
                                "the overlay z-order was updated correctly");
// at this point fError must contain DD_OK, simply return it.
return fError;
}

// --------------------------------------------------------------------------
// Method:          FGetAlphaBitDepth
// Description:     Returns alpha channel bit depth (1, 2, 4, or 8)
//---------------------------------------------------------------------------
dword               __fastcall TDirectDrawSurface::FGetAlphaBitDepth()
{
return fDescription->AlphaBitDepth;
}

// --------------------------------------------------------------------------
// Method:          FSetAlphaBitDepth
// Description:     Sets alpha channel bit depth (1, 2, 4, or 8)
//---------------------------------------------------------------------------
void                __fastcall TDirectDrawSurface::FSetAlphaBitDepth(
                                                    dword pAlphaBitDepth )
{
fDescription->AlphaBitDepth = pAlphaBitDepth;
}

// --------------------------------------------------------------------------
// Method:          FGetBackBufferCount
// Description:     Returns the number of back buffers
//---------------------------------------------------------------------------
dword               __fastcall TDirectDrawSurface::FGetBackBufferCount()
{
return fDescription->BackBufferCount;
}

// --------------------------------------------------------------------------
// Method:          FSetBackBufferCount
// Description:     Sets the number of back buffers
//---------------------------------------------------------------------------
void                __fastcall TDirectDrawSurface::FSetBackBufferCount(
                                                    dword pBackBufferCount )
{
fDescription->BackBufferCount = pBackBufferCount;
}

// --------------------------------------------------------------------------
// Method:          FGetCKDestBlt
// Description:     Returns the TDDColorKey structure that describes the
//                  destination color key for blit operations
//---------------------------------------------------------------------------
TDDColorKey*        __fastcall TDirectDrawSurface::FGetCKDestBlt()
{
return NULL;
}

// --------------------------------------------------------------------------
// Method:          FSetCKDestBlt
// Description:     Sets the TDDColorKey structure that describes the
//                  destination color key for blit operations
//---------------------------------------------------------------------------
void                __fastcall TDirectDrawSurface::FSetCKDestBlt(
                                                    TDDColorKey* pCKDestBlt )
{
if (pCKDestBlt!=NULL)
    {
    fCKDestBlt->LowValue = pCKDestBlt->LowValue;
    fCKDestBlt->HighValue = pCKDestBlt->HighValue;
    }
}

// --------------------------------------------------------------------------
// Method:          FGetCKDestOverlay
// Description:     Returns the TDDColorKey structure that describes the
//                  destination color key to be used for an overlay surface
//---------------------------------------------------------------------------
TDDColorKey*        __fastcall TDirectDrawSurface::FGetCKDestOverlay()
{
return NULL;
}

// --------------------------------------------------------------------------
// Method:          FSetCKDestOverlay
// Description:     Sets the TDDColorKey structure that describes the
//                  destination color key to be used for an overlay surface
//---------------------------------------------------------------------------
void                __fastcall TDirectDrawSurface::FSetCKDestOverlay(
                                                TDDColorKey* pCKDestOverlay )
{
if (pCKDestOverlay!=NULL)
    {
    fCKDestOverlay->LowValue = pCKDestOverlay->LowValue;
    fCKDestOverlay->HighValue = pCKDestOverlay->HighValue;
    }
}

// --------------------------------------------------------------------------
// Method:          FGetCKSrcBlt
// Description:     TDDColorKey structure that describes the source color key
//                  for blit operations
//---------------------------------------------------------------------------
TDDColorKey*        __fastcall TDirectDrawSurface::FGetCKSrcBlt()
{
return NULL;
}

// --------------------------------------------------------------------------
// Method:          FSetCKSrcBlt
// Description:     TDDColorKey structure that describes the source color key
//                  for blit operations
//---------------------------------------------------------------------------
void                __fastcall TDirectDrawSurface::FSetCKSrcBlt(
                                                    TDDColorKey* pCKSrcBlt )
{
if (pCKSrcBlt!=NULL)
    {
    fCKSrcBlt->LowValue = pCKSrcBlt->LowValue;
    fCKSrcBlt->HighValue = pCKSrcBlt->HighValue;
    }
}

// --------------------------------------------------------------------------
// Method:          FGetCKSrcOverlay
// Description:     TDDColorKey structure that describes the source color key
//                  to be used for an overlay surface
//---------------------------------------------------------------------------
TDDColorKey*        __fastcall TDirectDrawSurface::FGetCKSrcOverlay()
{
return NULL;
}

// --------------------------------------------------------------------------
// Method:          FSetCKSrcOverlay
// Description:     TDDColorKey structure that describes the source color key
//                  to be used for an overlay surface
//---------------------------------------------------------------------------
void                __fastcall TDirectDrawSurface::FSetCKSrcOverlay(
                                                TDDColorKey* pCKSrcOverlay )
{
if (pCKSrcOverlay!=NULL)
    {
    fCKSrcOverlay->LowValue = pCKSrcOverlay->LowValue;
    fCKSrcOverlay->HighValue = pCKSrcOverlay->HighValue;
    }
}

// --------------------------------------------------------------------------
// Method:          FGetDescription
// Description:     Returns the characteristics of an existing surface
//---------------------------------------------------------------------------
TDDSurfaceDesc*     __fastcall TDirectDrawSurface::FGetDescription()
{
return fDescription;
}

// --------------------------------------------------------------------------
// Method:          FSetDescription
// Description:     Sets the characteristics of an existing surface
//---------------------------------------------------------------------------
void                __fastcall  TDirectDrawSurface::FSetDescription(
                                            TDDSurfaceDesc* pDescription )
{
if (pDescription!=NULL)
    {
    ShowMessage("Setting properties");
    // manually copy each property
    Flags = pDescription->Flags;
    Height = pDescription->Height;
    Width = pDescription->Width;
    Pitch = pDescription->Pitch;
    BackBufferCount = pDescription->BackBufferCount;
    MipMapCount = pDescription->MipMapCount;
    ZBufferBitDepth = pDescription->ZBufferBitDepth;
    RefreshRate = pDescription->RefreshRate;
    AlphaBitDepth = pDescription->AlphaBitDepth;
    CKDestOverlay = pDescription->CKDestOverlay;
    CKDestBlt = pDescription->CKDestBlt;
    CKSrcOverlay = pDescription->CKSrcOverlay;
    CKSrcBlt = pDescription->CKSrcBlt;
    PixelFormat = pDescription->PixelFormat;
    SCaps = pDescription->SCaps->Caps;
    }
}

// --------------------------------------------------------------------------
// Method:          FGetFlags
// Description:     Returns indicators of the validity of various
//                  TDDSurfaceDesc members
//---------------------------------------------------------------------------
dword               __fastcall TDirectDrawSurface::FGetFlags()
{
return fDescription->Flags;
}

// --------------------------------------------------------------------------
// Method:          FSetFlags
// Description:     Sets indicators of the validity of various
//                  TDDSurfaceDesc members
//---------------------------------------------------------------------------
void                __fastcall TDirectDrawSurface::FSetFlags( dword pFlags )
{
fDescription->Flags = pFlags;
}

// --------------------------------------------------------------------------
// Method:          FGetHeight
// Description:     Vertical dimension of the surface to be created, in
//                  pixels
//---------------------------------------------------------------------------
dword               __fastcall TDirectDrawSurface::FGetHeight()
{
return fDescription->Height;
}

// --------------------------------------------------------------------------
// Method:          FSetHeight
// Description:     Vertical dimension of the surface to be created, in
//                  pixels
//---------------------------------------------------------------------------
void                __fastcall TDirectDrawSurface::FSetHeight( dword pHeight)
{
fDescription->Height = pHeight;
}

// --------------------------------------------------------------------------
// Method:          FGetMipMapCount
// Description:     Number of mipmap levels
//---------------------------------------------------------------------------
dword               __fastcall TDirectDrawSurface::FGetMipMapCount()
{
return fDescription->MipMapCount;
}

// --------------------------------------------------------------------------
// Method:          FSetMipMapCount
// Description:     Number of mipmap levels
//---------------------------------------------------------------------------
void                __fastcall TDirectDrawSurface::FSetMipMapCount(
                                                        dword pMipMapCount )
{
fDescription->MipMapCount = pMipMapCount;
}

// --------------------------------------------------------------------------
// Method:          FGetPitch
// Description:     Distance, in bytes, to the start of next line
//---------------------------------------------------------------------------
dword               __fastcall TDirectDrawSurface::FGetPitch()
{
return fDescription->Pitch;
}

// --------------------------------------------------------------------------
// Method:          FSetPitch
// Description:     Distance, in bytes, to the start of next line
//---------------------------------------------------------------------------
void                __fastcall TDirectDrawSurface::FSetPitch( dword pPitch )
{
fDescription->Pitch = pPitch;
}

// --------------------------------------------------------------------------
// Method:          FGetPixelFormat
// Description:     TDDPixelFormat structure that describes the surface's
//                  pixel format
//---------------------------------------------------------------------------
TDDPixelFormat*     __fastcall TDirectDrawSurface::FGetPixelFormat()
{
return NULL;
}

// --------------------------------------------------------------------------
// Method:          FSetPixelFormat
// Description:     TDDPixelFormat structure that describes the surface's
//                  pixel format
//---------------------------------------------------------------------------
void                __fastcall TDirectDrawSurface::FSetPixelFormat(
                                            TDDPixelFormat* pSetPixelFormat )
{
}

// --------------------------------------------------------------------------
// Method:          FGetPtr
// Description:     Use this property to access the directx
//                  LPDIRECTDRAWSURFACE3 variable
//---------------------------------------------------------------------------
LPDIRECTDRAWSURFACE3 __fastcall  TDirectDrawSurface::FGetPtr()
{
return fSurfacePtr;
}

// --------------------------------------------------------------------------
// Method:          FSetPtr
// Description:     Use this property to access the directx
//                  LPDIRECTDRAWSURFACE3 variable
//---------------------------------------------------------------------------
void                __fastcall  TDirectDrawSurface::FSetPtr(
                                                LPDIRECTDRAWSURFACE3 pPtr )
{
if (pPtr!=NULL)
    {
    if (fCreated) Release();
    fSurfacePtr = pPtr;
    fCreated = true;
    }
else
    {
    fSurfacePtr = NULL;
    fCreated = false;
    }
}

// --------------------------------------------------------------------------
// Method:          FGetRefreshRate
// Description:     Refresh rate (used when the display mode is described)
// Note:            The value of 0 indicates an adapter fault
//---------------------------------------------------------------------------
dword               __fastcall TDirectDrawSurface::FGetRefreshRate()
{
return fDescription->RefreshRate;
}

// --------------------------------------------------------------------------
// Method:          FSetRefreshRate
// Description:     Refresh rate (used when the display mode is described)
// Note:            The value of 0 indicates an adapter fault
//---------------------------------------------------------------------------
void                __fastcall TDirectDrawSurface::FSetRefreshRate(
                                                        dword pRefreshRate )
{
fDescription->RefreshRate = pRefreshRate;
}

// --------------------------------------------------------------------------
// Method:          FGetSCaps
// Description:     TDDSCaps structure containing the surface's capabilities
//---------------------------------------------------------------------------
dword               __fastcall TDirectDrawSurface::FGetSCaps()
{
return fDescription->SCaps->Caps;
}

// --------------------------------------------------------------------------
// Method:          FSetSCaps
// Description:     TDDSCaps structure containing the surface's capabilities
//---------------------------------------------------------------------------
void                __fastcall TDirectDrawSurface::FSetSCaps( dword pSCaps )
{
fDescription->SCaps->Caps = pSCaps;
}

// --------------------------------------------------------------------------
// Method:          FGetSurface
// Description:
//---------------------------------------------------------------------------
void*               __fastcall TDirectDrawSurface::FGetSurface()
{
return fDescription->Surface();
}

// --------------------------------------------------------------------------
// Method:          FGetType
// Description:     Sets the TDirectDrawSurface to be one of the predefined
//                  surface types. ie. _BackBuffer or _PrimarySurface
//---------------------------------------------------------------------------
TDDSurfaceType      __fastcall  TDirectDrawSurface::FGetType()
{
return fSurfaceType;
}

// --------------------------------------------------------------------------
// Method:          FSetType
// Description:     Sets the TDirectDrawSurface to be one of the predefined
//                  surface types. ie. _BackBuffer or _PrimarySurface
//---------------------------------------------------------------------------
void                __fastcall  TDirectDrawSurface::FSetType(
                                                    TDDSurfaceType pType )
{
fSurfaceType = pType;
switch (pType)
    {
    case _CustomSurface:
        break;
    case _BackBuffer:
        Flags = DDSD_CAPS;
        SCaps = DDSCAPS_BACKBUFFER;
        fDescription->SCaps->Caps = SCaps;
        BackBufferCount = 0;
        break;
    case _Bitmap:
        break;
    case _MipMap:
        break;
    case _OffscreenPlain:
        break;
    case _PrimarySurface:
        Flags = DDSD_CAPS | DDSD_BACKBUFFERCOUNT;
        SCaps = DDSCAPS_COMPLEX | DDSCAPS_FLIP | DDSCAPS_PRIMARYSURFACE;
        fDescription->SCaps->Caps = SCaps;
        BackBufferCount = 1;
        break;
    case _Texture:
        break;
    }
}

// --------------------------------------------------------------------------
// Method:          FGetWidth
// Description:     Horizontal dimension of the surface to be created, in
//                  pixels
//---------------------------------------------------------------------------
dword               __fastcall TDirectDrawSurface::FGetWidth()
{
return fDescription->Width;
}

// --------------------------------------------------------------------------
// Method:          FSetWidth
// Description:     Horizontal dimension of the surface to be created, in
//                  pixels
//---------------------------------------------------------------------------
void                __fastcall TDirectDrawSurface::FSetWidth( dword pWidth )
{
fDescription->Width = pWidth;
}

// --------------------------------------------------------------------------
// Method:          FGetZBufferBitDepth
// Description:     Depth of z-buffer
// Note:            32-bit z-buffers are not supported
//---------------------------------------------------------------------------
dword               __fastcall TDirectDrawSurface::FGetZBufferBitDepth()
{
return fDescription->ZBufferBitDepth;
}

// --------------------------------------------------------------------------
// Method:          FSetZBufferBitDepth
// Description:     Depth of z-buffer
// Note:            32-bit z-buffers are not supported
//---------------------------------------------------------------------------
void                __fastcall TDirectDrawSurface::FSetZBufferBitDepth(
                                                    dword pZBufferBitDepth )
{
fDescription->ZBufferBitDepth = pZBufferBitDepth;
}

//---------------------------------------------------------------------------

