// --------------------------------------------------------------------------
#ifndef TDirectDrawSurfaceH
#define TDirectDrawSurfaceH

// ==========================================================================
// File:            TDirectDrawSurface.H
// Description:     This file contains the definition for the
//                  TDirectDrawSurface component
//
// (c) 1998 A.S.C. Alternate Software Concepts, Sydney, Australia.
// All Rights Reserved. Refer to the file TDirectX_Licence.Txt for detailed
// licence & copyright information
// ==========================================================================

// --------------------------------------------------------------------------
// include the BCB related definitions, etc
// --------------------------------------------------------------------------
#include <vcl\SysUtils.hpp>
#include <vcl\Controls.hpp>
#include <vcl\Classes.hpp>
#include <vcl\Forms.hpp>

// --------------------------------------------------------------------------
// include the basic TDirectDraw definitions, etc
// --------------------------------------------------------------------------
#include "TDirectDrawInclude.h"
#include "TDirectDrawDefs.h"
#include "TDirectDrawErrors.h"
#include "TDirectDrawMisc.h"

// --------------------------------------------------------------------------
// include the TDirectDrawSurface method specific flags
// --------------------------------------------------------------------------
#include "TDDSurfaceType.h"

// --------------------------------------------------------------------------
// include the various DirectX Wrapper classes used by TDirectDrawSurface
// --------------------------------------------------------------------------
#include "TDDBltFX.h"
#include "TDDSCaps.h"

// --------------------------------------------------------------------------
// include the various DirectX Component classes used by TDirectDrawSurface
// --------------------------------------------------------------------------
#include "TDDSurfaceDesc.h"
#include "TDDVideoPortCaps.h"

// ==========================================================================
//
// Class:           TDirectDrawSurface
//
// Description:     This is a component that wraps the functionality of the
//                  IDIRECTDRAWSURFACE3 interface available within DirectX5
//
//                  All methods of this object attempt to call the relevant
//                  IDIRECTDRAWSURFACE3 interface functions
//
//                  All methods call the OnError event method if any errors
//                  are encountered during execution of the relevant DirectX
//                  functions
//
// --------------------------------------------------------------------------
//
// Published        AlphaBitDepth
// Properties:      BackBufferCount
//                  Description
//                  DestBltColorKey
//                  DestOverlayColorKey
//                  Flags
//                  Height
//                  MipMapCount
//                  Pitch
//                  PixelFormat
//                  RefreshRate
//                  SCaps
//                  SrcBltColorKey
//                  SrcOverlayColorKey
//                  Type
//                  Width
//                  ZBufferBitDepth
//
// --------------------------------------------------------------------------
//
// Public           Created
// Properties:      Ptr
//
// --------------------------------------------------------------------------
//
//                  AddAttachedSurface
//                  AddOverlayDirtyRect
//                  Blt
//                  BltBatch
//                  BltFast
//                  Create
//                  DeleteAttachedSurface
//                  EnumAttachedSurfaces
//                  EnumOverlayZOrders
//                  Flip
//                  GetAttachedSurface
//                  GetBltStatus
//                  GetCaps
//                  GetClipper
//                  GetColorKey
//                  GetDC
//                  GetDDInterface
//                  GetFlipStatus
//                  GetOverlayPosition
//                  GetPalette
//                  GetPixelFormat
//                  GetSurfaceDesc
//                  IsLost
//                  Lock
//                  PageLock
//                  PageUnlock
//                  Release
//                  ReleaseDC
//                  Restore
//                  SetClipper
//                  SetColorKey
//                  SetOverlayPosition
//                  SetPalette
//                  Unlock
//                  UpdateOverlay
//                  UpdateOverlayZOrder
//
// --------------------------------------------------------------------------
//
// Events:          OnCreate
//                  OnError
//                  OnMessage
//                  OnRelease
//
// ==========================================================================
class TDirectDrawSurface : public TComponent
{
__published:

// --------------------------------------------------------------------------
// Property:        AlphaBitDepth
// Description:     Alpha channel bit depth (1, 2, 4, or 8)
// --------------------------------------------------------------------------
    __property	    dword		      AlphaBitDepth = {
                                                    read=FGetAlphaBitDepth,
                                                    write=FSetAlphaBitDepth,
                                                    default=0 };

// --------------------------------------------------------------------------
// Property:        BackBufferCount
// Description:     Number of back buffers
// --------------------------------------------------------------------------
    __property	    dword           BackBufferCount = {
                                                    read=FGetBackBufferCount,
                                                    write=FSetBackBufferCount
                                                    ,default=0 };

// --------------------------------------------------------------------------
// Property:        Description
// Description:     The characteristics of an existing surface
// --------------------------------------------------------------------------
    __property      TDDSurfaceDesc*     Description = { read=FGetDescription,
                                                        write=FSetDescription
                                                        ,nodefault };

// --------------------------------------------------------------------------
// Property:        DestBltColorKey
// Description:     TDDColorKey structure that describes the destination
//                  color key for blit operations
// --------------------------------------------------------------------------
    __property	    TDDColorKey*          CKDestBlt = { read=FGetCKDestBlt,
                                                        write=FSetCKDestBlt,
                                                        nodefault };

// --------------------------------------------------------------------------
// Property:        DestOverlayColorKey
// Description:     TDDColorKey structure that describes the destination
//                  color key to be used for an overlay surface
// --------------------------------------------------------------------------
    __property	    TDDColorKey*	  CKDestOverlay = {
                                                    read=FGetCKDestOverlay,
                                                    write=FSetCKDestOverlay,
                                                    nodefault };

// --------------------------------------------------------------------------
// Property:        Flags
// Description:     Indicates validity of various TDDSurfaceDesc members
// --------------------------------------------------------------------------
    __property	    dword                     Flags = { read=FGetFlags,
                                                        write=FSetFlags,
                                                        nodefault };

// --------------------------------------------------------------------------
// Property:        Height
// Description:     Vertical dimension of the surface to be created, in
//                  pixels
// --------------------------------------------------------------------------
	__property	    dword                    Height = { read=FGetHeight,
                                                        write=FSetHeight,
                                                        default=640 };

// --------------------------------------------------------------------------
// Property:        MipMapCount
// Description:     Number of mipmap levels
// --------------------------------------------------------------------------
    __property	    dword	            MipMapCount = { read=FGetMipMapCount,
                                                        write=FSetMipMapCount
                                                        ,default=0 };

// --------------------------------------------------------------------------
// Event:           OnCreate
// Description:     This event method is called when the Create() method has
//                  created the IDIRECTDRAWSURFACE3 interface successfully
// --------------------------------------------------------------------------
    __property      TDirectDrawEvent       OnCreate = { read=FOnCreate,
                                                        write=FOnCreate,
                                                        nodefault };

// --------------------------------------------------------------------------
// Event:           OnError
// Description:     This event method is called when a valid DirectX error
//                  occurs
// Note:            TDirectDrawSurface supplies a default OnError event
//                  handler which opens a simple message box to display the
//                  details of the error
//                  To stop TDirectDrawSurface Error messages from occurring,
//                  set this property to NULL after you call Create()
// --------------------------------------------------------------------------
    __property	    TDirectDrawError	    OnError = { read=FOnError,
                                                        write=FOnError,
                                                        nodefault };

// --------------------------------------------------------------------------
// Event:           OnMessage
// Description:     This event method is called when the TDirectDraw object
//                  performs any kind of task. This is incredibly useful for
//                  debugging
// Note:            TDirectDrawSurface supplies a default OnMessage event
//                  handler which opens a simple message box to display the
//                  details of the error
//                  To stop TDirectDrawSurface messages from occurring, set
//                  this property to NULL after you call Create()
// --------------------------------------------------------------------------
    __property      TDirectDrawMessage    OnMessage = { read=FOnMessage,
                                                        write=FOnMessage,
                                                        nodefault };

// --------------------------------------------------------------------------
// Event:           OnRelease
// Description:     This event method is called before the Release() method
//                  releases the IDIRECTDRAWSURFACE3 interface
// --------------------------------------------------------------------------
    __property      TDirectDrawEvent      OnRelease = { read=FOnRelease,
                                                        write=FOnRelease,
                                                        nodefault };

// --------------------------------------------------------------------------
// Property:        Pitch
// Description:     Distance, in bytes, to the start of next line
// --------------------------------------------------------------------------
    __property	    dword	                  Pitch = { read=FGetPitch,
                                                        write=FSetPitch,
                                                        nodefault };

// --------------------------------------------------------------------------
// Property:        PixelFormat
// Description:     TDDPixelFormat structure that describes the surface's
//                  pixel format
// --------------------------------------------------------------------------
    __property      TDDPixelFormat*     PixelFormat = { read=FGetPixelFormat,
                                                        write=FSetPixelFormat
                                                        ,nodefault };

// --------------------------------------------------------------------------
// Property:        RefreshRate
// Description:     Refresh rate (used when the display mode is described)
// Note:            The value of 0 indicates an adapter fault
// --------------------------------------------------------------------------
    __property	    dword	            RefreshRate = { read=FGetRefreshRate,
                                                        write=FSetRefreshRate
                                                        ,default=0 };

// --------------------------------------------------------------------------
// Property:        SCaps
// Description:     TDDSCaps structure containing the surface's capabilities
// --------------------------------------------------------------------------
    __property      dword                     SCaps = { read=FGetSCaps,
                                                        write=FSetSCaps,
                                                        nodefault };

// --------------------------------------------------------------------------
// Property:        SrcBltColorKey
// Description:     TDDColorKey structure that describes the source color key
//                  for blit operations
// --------------------------------------------------------------------------
    __property	    TDDColorKey*	       CKSrcBlt = { read=FGetCKSrcBlt,
                                                        write=FSetCKSrcBlt,
                                                        nodefault };

// --------------------------------------------------------------------------
// Property:        SrcOverlayColorKey
// Description:     TDDColorKey structure that describes the source color key
//                  to be used for an overlay surface
// --------------------------------------------------------------------------
    __property	    TDDColorKey*	   CKSrcOverlay = {
                                                    read=FGetCKSrcOverlay,
                                                    write=FSetCKSrcOverlay,
                                                    nodefault };

// --------------------------------------------------------------------------
// Property:        Type
// Description:     Sets the TDirectDrawSurface to be one of the predefined
//                  surface types. ie. _BackBuffer or _PrimarySurface
// --------------------------------------------------------------------------
    __property      TDDSurfaceType             Type = { read=FGetType,
                                                    write=FSetType,
                                                    default=_CustomSurface };

// --------------------------------------------------------------------------
// Property:        Width
// Description:     Horizontal dimension of the surface to be created, in
//                  pixels
// --------------------------------------------------------------------------
    __property	    dword	                  Width = { read=FGetWidth,
                                                        write=FSetWidth,
                                                        default=480 };

// --------------------------------------------------------------------------
// Property:        ZBufferBitDepth
// Description:     Depth of z-buffer
// Note:            32-bit z-buffers are not supported
// --------------------------------------------------------------------------
    __property	    dword		    ZBufferBitDepth = {
                                                    read=FGetZBufferBitDepth,
                                                    write=FSetZBufferBitDepth
                                                    ,default=0 };

public:

// --------------------------------------------------------------------------
// Property:        Created                                                     //no fastcall in cpp
// Description:     Indicates whether the create method was successful
// --------------------------------------------------------------------------
    __property      bool                    Created = { read=fCreated };

// --------------------------------------------------------------------------
// Property:        Ptr
// Description:     Use this property to access the directx
//                  LPDIRECTDRAWSURFACE3 variable
// --------------------------------------------------------------------------
    __property      LPDIRECTDRAWSURFACE3        Ptr = { read=FGetPtr,
                                                        write=FSetPtr,
                                                        default=NULL };

// --------------------------------------------------------------------------
// Constructor & Destructor
// --------------------------------------------------------------------------
	                __fastcall TDirectDrawSurface(TComponent* Owner);
    virtual         __fastcall ~TDirectDrawSurface();

// --------------------------------------------------------------------------
// Method:          AddAttachedSurface
// Description:     This method attaches another surface to the surface
//                  contained in this object
// Parameters:      surface - this is the object that is to be attached to
//                          the current surface
// --------------------------------------------------------------------------
	virtual	HRESULT	__fastcall	AddAttachedSurface( TDirectDrawSurface*
                                                                surface );
    // LPDIRECTDRAWSURFACE3 lpDDSAttachedSurface

// --------------------------------------------------------------------------
// Method:          AddOverlayDirtyRect
// Description:     This function is not yet implemented in the DirectX SDK
// --------------------------------------------------------------------------
	virtual	HRESULT	__fastcall	AddOverlayDirtyRect();
    // LPRECT lpRect

// --------------------------------------------------------------------------
// Method:          Blt
// Description:     This method performs a 'bitBlt', 'Blt', or bit-block
//                  transfer, to copy pixels from one surface to another -
//                  from a source surface to a destination surface
// Parameters:      destrect    - this defines a rectangular region on the
//                              destination surface that will be overwritten
//                              by pixels transferred during the Blt. If this
//                              rectangle is an empty rectangle
//                              (ie. Left=0,Top=0, Right=0,Bottom=0) then all
//                              pixels on the destination surface will be
//                              replaced with pixels from the source surface
//                  srcsurface  - this is the source surface containing the
//                              pixels to be transferred during the Blt
//                  srcrect     - this defines a rectangular region on the
//                              source surface that contains the pixels that
//                              will be copied during the Blt. If this
//                              rectangle is an empty rectangle
//                              (Left=0,Top=0,Right=0,Bottom=0)
//                               then all pixels on the source surface will
//                               be copied to the destination surface
//                  flags       - these flags define the various pixel
//                              manipulations that can occur during the Blt
//                  bltfx       - this structure contains additional effects
//                              to be used during the Blt
// --------------------------------------------------------------------------
	virtual	HRESULT	__fastcall	Blt( TRect& destrect,
                                            TDirectDrawSurface* srcsurface,
                                            TRect& srcrect, dword flags,
                                            TDDBltFX* bltfx );
    // LPRECT lpDestRect, LPDIRECTDRAWSURFACE3 lpDDSrcSurface,
    // LPRECT lpSrcRect, DWORD dwFlags, LPDDBLTFX lpDDBltFx

// --------------------------------------------------------------------------
// Method:          BltBatch
// Description:     Performs a sequence of Blt operations from several
//                  sources to a single destination
// Note:            This method has not yet been implemented
// --------------------------------------------------------------------------
	virtual	HRESULT	__fastcall	BltBatch();
    // LPDDBLTBATCH lpDDBltBatch, DWORD dwCount, DWORD dwFlags

// --------------------------------------------------------------------------
// Method:          BltFast
// Description:     Performs a source copy blit or transparent blit by using
//                  a source color key or destination color key
// Note:            Always attempts an asynchronous blit if H/W supported
//                  Only works on display memory surfaces without clipping
//                  10% faster if emulated, no advantage if hardware used
// --------------------------------------------------------------------------
	virtual	HRESULT	__fastcall	BltFast();
    // DWORD dwX, DWORD dwY, LPDIRECTDRAWSURFACE3 lpDDSrcSurface,
    // LPRECT lpSrcRect, DWORD dwTrans

// --------------------------------------------------------------------------
// Method:          Create
// Description:     Call this method to create the internals of the
//                  TDirectDrawSurface object
// Notes:           Remember to call the Release() method when finished
// --------------------------------------------------------------------------
    virtual HRESULT __fastcall  Create();

// --------------------------------------------------------------------------
// Method:          DeleteAttachedSurface
// Description:     Detaches two attached surfaces
// Note:            The detached surface is not released
//                  Attachments not added with AddAttachedSurface cannot be
//                  detached
// --------------------------------------------------------------------------
	virtual	HRESULT	__fastcall	DeleteAttachedSurface();
    // DWORD dwFlags, LPDIRECTDRAWSURFACE3 lpDDSAttachedSurface

// --------------------------------------------------------------------------
// Method:          EnumAttachedSurfaces
// Description:     Enumerates all the surfaces attached to a given surface
// --------------------------------------------------------------------------
	virtual	HRESULT	__fastcall	EnumAttachedSurfaces();
    // LPVOID lpContext, LPDDENUMSURFACESCALLBACK lpEnumSurfacesCallback

// --------------------------------------------------------------------------
// Method:          EnumOverlayZOrders
// Description:     Enumerates overlay surfaces on a specified destination
// Note:            Overlays can be enumerated in front-to-back or
//                  back-to-front order
// --------------------------------------------------------------------------
	virtual	HRESULT	__fastcall	EnumOverlayZOrders();
    // DWORD dwFlags, LPVOID lpContext, LPDDENUMSURFACESCALLBACK lpfnCallback

// --------------------------------------------------------------------------
// Method:          Flip
// Description:     Associates DDSCAPS_BACKBUFFER surface memory with the
//                  front-buffer surface
// Note:            Can only be called for surfaces that have DDSCAPS_FLIP
//                  and DDSCAPS_FRONTBUFFER capabilities
//                  Display memory previously associated with the front
//                  buffer is associated with the back buffer
//                  Always synchronized with the vertical blank
//                  If the surface has been assigned to a video port, this
//                  method updates the visible overlay surface and the video
//                  port's target surface
// --------------------------------------------------------------------------
	virtual	HRESULT	__fastcall	Flip();
    // LPDIRECTDRAWSURFACE3 lpDDSurfaceTargetOverride, DWORD dwFlags

// --------------------------------------------------------------------------
// Method:          GetAttachedSurface
// Description:     Obtains the attached surface with the specified
//                  capabilities
// Note:            Fails if multiple surfaces match the specified
//                  capabilities
// --------------------------------------------------------------------------
	virtual	HRESULT	__fastcall	GetAttachedSurface( TDDSCaps* scaps,
                                            TDirectDrawSurface* surface );
    // LPDDSCAPS lpDDSCaps, LPDIRECTDRAWSURFACE3 FAR *lplpDDAttachedSurface

// --------------------------------------------------------------------------
// Method:          GetBltStatus
// Description:     Obtains the blitter status
// --------------------------------------------------------------------------
	virtual	HRESULT	__fastcall	GetBltStatus();
    // DWORD dwFlags

// --------------------------------------------------------------------------
// Method:          GetCaps
// Description:     Retrieves the capabilities of the surface
// Note:            These capabilities are not necessarily related to the
//                  capabilities of the display device
// --------------------------------------------------------------------------
	virtual	HRESULT	__fastcall	GetCaps();
    // LPDDSCAPS lpDDSCaps

// --------------------------------------------------------------------------
// Method:          GetClipper
// Description:     Retrieves the DirectDrawClipper object associated with
//                  this surface
// --------------------------------------------------------------------------
	virtual	HRESULT	__fastcall	GetClipper();
    // LPDIRECTDRAWCLIPPER FAR *lplpDDClipper

// --------------------------------------------------------------------------
// Method:          GetColorKey
// Description:     Retrieves the color key value for the surface object
// --------------------------------------------------------------------------
	virtual	HRESULT	__fastcall	GetColorKey();
    // DWORD dwFlags, LPDDCOLORKEY lpDDColorKey

// --------------------------------------------------------------------------
// Method:          GetDC
// Description:     Creates a GDI-compatible handle of a device context for
//                  the surface
// Note:            Locks the surface
//                  Surface remains locked until the ReleaseDC is called
// --------------------------------------------------------------------------
	virtual	HRESULT	__fastcall	GetDC();
    // HDC FAR *lphDC

// --------------------------------------------------------------------------
// Method:          GetDDInterface
// Description:     Retrieves an interface to the DirectDraw object that was
//                  used to create the surface
// --------------------------------------------------------------------------
	virtual	HRESULT	__fastcall	GetDDInterface();
    // LPVOID FAR *lplpDD

// --------------------------------------------------------------------------
// Method:          GetFlipStatus
// Description:     Indicates whether the surface has finished flipping
// --------------------------------------------------------------------------
	virtual	HRESULT	__fastcall	GetFlipStatus();
    // DWORD dwFlags

// --------------------------------------------------------------------------
// Method:          GetOverlayPosition
// Description:     Given a visible, active overlay surface (DDSCAPS_OVERLAY
//                  flag set), this returns the display coordinates of the
//                  surface
// --------------------------------------------------------------------------
	virtual	HRESULT	__fastcall	GetOverlayPosition();
    // LPLONG lplX, LPLONG lplY

// --------------------------------------------------------------------------
// Method:          GetPalette
// Description:     Retrieves the palette structure associated with this
//                  surface and increments the reference count of the
//                  returned palette
// --------------------------------------------------------------------------
	virtual	HRESULT	__fastcall	GetPalette();
    // LPDIRECTDRAWPALETTE FAR *lplpDDPalette

// --------------------------------------------------------------------------
// Method:          GetPixelFormat
// Description:     Retrieves the color and pixel format of the surface
// --------------------------------------------------------------------------
	virtual	HRESULT	__fastcall	GetPixelFormat();
    // LPDDPIXELFORMAT lpDDPixelFormat

// --------------------------------------------------------------------------
// Method:          GetSurfaceDesc
// Description:     Retrieves a DDSURFACEDESC structure that describes the
//                  surface in its current condition
// --------------------------------------------------------------------------
	virtual	HRESULT	__fastcall	GetSurfaceDesc();
    // LPDDSURFACEDESC lpDDSurfaceDesc

// --------------------------------------------------------------------------
// Method:          IsLost
// Description:     Determines if the surface memory associated with a
//                  DirectDrawSurface object has been freed
// --------------------------------------------------------------------------
	virtual	HRESULT	__fastcall	IsLost();
    // no params

// --------------------------------------------------------------------------
// Method:          Lock
// Description:     Obtains a pointer to the surface memory
// Note:            After retrieving a surface memory pointer, you can access
//                  the surface memory until a corresponding unlock is called
//                  Once a surface is unlocked, the pointer becomes invalid
//                  Blits from a locked region of a surface will fail
//                  GDI blit functions silently fail when used on locked
//                  video memory surfaces
// --------------------------------------------------------------------------
	virtual	HRESULT	__fastcall	Lock();
    // LPRECT lpDestRect, LPDDSURFACEDESC lpDDSurfaceDesc, DWORD dwFlags,
    // HANDLE hEvent

// --------------------------------------------------------------------------
// Method:          PageLock
// Description:     Prevents a system-memory surface from being paged out
//                  while a blit operation using direct memory access (DMA)
//                  transfers to or from system memory is in progress
// Note:            This method must be called to use DMA support otherwise
//                  the blit will use software emulation
//                  A lock count is maintained for each surface and is
//                  incremented each time PageLock is called for that surface
//                  Only works on system-memory surfaces, it won't page lock
//                  a display-memory surface or an emulated primary surface
// --------------------------------------------------------------------------
	virtual	HRESULT	__fastcall	PageLock();
    // DWORD dwFlags

// --------------------------------------------------------------------------
// Method:          PageUnlock
// Description:     Unlocks a system-memory surface, allowing it to be paged
//                  out
// Note:            The lock count is decremented when PageUnlock is called
//                  When the count reaches 0, the memory is unlocked and can
//                  be paged by the operating system
//                  This method works only on system-memory surfaces, it
//                  won't page unlock a display-memory surface or an emulated
//                  primary surface
// --------------------------------------------------------------------------
	virtual	HRESULT	__fastcall	PageUnlock();
    // DWORD dwFlags

// --------------------------------------------------------------------------
// Method:          Release
// Description:
// --------------------------------------------------------------------------
    virtual HRESULT __fastcall  Release();

// --------------------------------------------------------------------------
// Method:          ReleaseDC
// Description:     Releases the handle of a device context previously
//                  obtained by GetDC
// Note:            Also unlocks the surface locked when the GetDC was called
// --------------------------------------------------------------------------
	virtual	HRESULT	__fastcall	ReleaseDC();
    // HDC hDC

// --------------------------------------------------------------------------
// Method:          Restore
// Description:     Restores a surface that has been lost. Lost surfaces
//                  occur when the surface memory associated with the surface
//                  object has been freed
// Note:            Restores memory allocated for a surface, not the contents
//                  Restore will reallocate surface memory and reattach it to
//                  the DirectDrawSurface object
//                  One call to this method will restore a surface object's
//                  associated implicit surfaces (back buffers, and so on)
//                  Attempts to directly restore an implicitly created
//                  surface will result in an error
//                  Restore will not work across explicit attachments created
//                  using AddAttachedSurface, each of these surfaces must be
//                  restored individually
// --------------------------------------------------------------------------
	virtual	HRESULT	__fastcall	Restore();
    // no params

// --------------------------------------------------------------------------
// Method:          SetClipper
// Description:     Attaches a clipper to a surface object
// --------------------------------------------------------------------------
	virtual	HRESULT	__fastcall	SetClipper();
    // LPDIRECTDRAWCLIPPER lpDDClipper

// --------------------------------------------------------------------------
// Method:          SetColorKey
// Description:     Sets the color key value for the surface object if the
//                  hardware supports color keys on a per surface basis
// --------------------------------------------------------------------------
	virtual	HRESULT	__fastcall	SetColorKey();
    // DWORD dwFlags, LPDDCOLORKEY lpDDColorKey

// --------------------------------------------------------------------------
// Method:          SetOverlayPosition
// Description:     Changes the display coordinates of an overlay surface
// --------------------------------------------------------------------------
	virtual	HRESULT	__fastcall	SetOverlayPosition();
    // LONG lX, LONG lY

// --------------------------------------------------------------------------
// Method:          SetPalette
// Description:     Attaches the specified palette object to a surface
// Note:            Surface uses this palette for all subsequent operations
//                  The palette change takes place immediately, without
//                  regard to refresh timing
// --------------------------------------------------------------------------
	virtual	HRESULT	__fastcall	SetPalette();
    // LPDIRECTDRAWPALETTE lpDDPalette

// --------------------------------------------------------------------------
// Method:          Unlock
// Description:     Notifies DirectDraw that the direct surface manipulations
//                  are complete
// --------------------------------------------------------------------------
	virtual	HRESULT	__fastcall	Unlock();
    // LPVOID lpSurfaceData

// --------------------------------------------------------------------------
// Method:          UpdateOverlay
// Description:     Repositions or modifies the visual attributes of an
//                  overlay surface
// Note:            These surfaces must have the DDSCAPS_OVERLAY value set
// --------------------------------------------------------------------------
	virtual	HRESULT	__fastcall	UpdateOverlay();
    // LPRECT lpSrcRect, LPDIRECTDRAWSURFACE3 lpDDDestSurface,
    // LPRECT lpDestRect, DWORD dwFlags, LPDDOVERLAYFX lpDDOverlayFx

// --------------------------------------------------------------------------
// Method:          UpdateOverlayDisplay
// Description:     Repaints the rectangles in the dirty rectangle list of
//                  all active overlays
// Note:            Clears the dirty rectangle list
//                  Only for software emulation, does nothing if the hardware
//                  supports overlays
// --------------------------------------------------------------------------
	virtual	HRESULT	__fastcall	UpdateOverlayDisplay();
    // DWORD dwFlags

// --------------------------------------------------------------------------
// Method:          UpdateOverlayZOrder
// Description:     Sets the z-order of an overlay
// --------------------------------------------------------------------------
	virtual	HRESULT	__fastcall	UpdateOverlayZOrder();
    // DWORD dwFlags, LPDIRECTDRAWSURFACE3 lpDDSReference

// --------------------------------------------------------------------------
// End of the TDirectDrawSurface interface
// --------------------------------------------------------------------------

protected:

    dword           __fastcall  FGetAlphaBitDepth();
    void            __fastcall  FSetAlphaBitDepth( dword pAlphaBitDepth );
    dword           __fastcall  FGetBackBufferCount();
    void            __fastcall  FSetBackBufferCount( dword
                                                        pBackBufferCount );
    TDDColorKey*    __fastcall  FGetCKDestBlt();
    void            __fastcall  FSetCKDestBlt( TDDColorKey* pCKDestBlt );
    TDDColorKey*    __fastcall  FGetCKDestOverlay();
    void            __fastcall  FSetCKDestOverlay( TDDColorKey*
                                                            pCKDestOverlay );
    TDDColorKey*    __fastcall  FGetCKSrcBlt();
    void            __fastcall  FSetCKSrcBlt( TDDColorKey* pCKSrcBlt );
    TDDColorKey*    __fastcall  FGetCKSrcOverlay();
    void            __fastcall  FSetCKSrcOverlay( TDDColorKey*
                                                            pCKSrcOverlay );
    TDDSurfaceDesc* __fastcall  FGetDescription();
    void            __fastcall  FSetDescription( TDDSurfaceDesc*
                                                            pDescription );
    dword           __fastcall  FGetFlags();
    void            __fastcall  FSetFlags( dword pFlags );
    dword           __fastcall  FGetHeight();
    void            __fastcall  FSetHeight( dword pHeight );
    dword           __fastcall  FGetMipMapCount();
    void            __fastcall  FSetMipMapCount( dword pMipMapCount );
    dword           __fastcall  FGetPitch();
    void            __fastcall  FSetPitch( dword pPitch );
    TDDPixelFormat* __fastcall  FGetPixelFormat();
    void            __fastcall  FSetPixelFormat( TDDPixelFormat*
                                                            pPixelFormat );
LPDIRECTDRAWSURFACE3
                    __fastcall  FGetPtr();
    void            __fastcall  FSetPtr( LPDIRECTDRAWSURFACE3 pPtr );
    dword           __fastcall  FGetRefreshRate();
    void            __fastcall  FSetRefreshRate( dword pRefreshRate );
    dword           __fastcall  FGetSCaps();
    void            __fastcall  FSetSCaps( dword pSCaps );
    void*           __fastcall  FGetSurface();                                  //no property/etc
    TDDSurfaceType  __fastcall  FGetType();
    void            __fastcall  FSetType( TDDSurfaceType pType );
    dword           __fastcall  FGetWidth();
    void            __fastcall  FSetWidth( dword pWidth );
    dword           __fastcall  FGetZBufferBitDepth();
    void            __fastcall  FSetZBufferBitDepth( dword
                                                        pZBufferBitDepth );

private:

    TDirectDrawEvent            FOnCreate;
    TDirectDrawEvent            FOnRelease;
    TDirectDrawMessage          FOnMessage;
    TDirectDrawError	        FOnError;

	HRESULT			            fError;
    bool                        fCreated;
    LPDIRECTDRAWSURFACE3        fSurfacePtr;
    TDDSurfaceDesc*             fDescription;
    TDDSurfaceType              fSurfaceType;
    TDDColorKey*                fCKDestBlt;
    TDDColorKey*                fCKDestOverlay;
    TDDColorKey*                fCKSrcBlt;
    TDDColorKey*                fCKSrcOverlay;
    TDDPixelFormat*             fPixelFormat;

};
// --------------------------------------------------------------------------
#endif
