//---------------------------------------------------------------------------
#include <vcl\vcl.h>
#pragma hdrstop

// ==========================================================================
// File:            TDDTemplatePropertyEditor.CPP
// Description:     This file contains the code for the property editor
//                  used to edit TDDTemplate type properties of the
//                  various components in the TDirectX library.
//
// (c) 1998 A.S.C. Alternate Software Concepts, Sydney, Australia.
// All Rights Reserved. Refer to the file TDirectX_Licence.Txt for detailed
// licence & copyright information.
// ==========================================================================

// --------------------------------------------------------------------------
// include the various TDDTemplate classes.
// --------------------------------------------------------------------------
#include "TDDSCapsPropertyEditor.h"
#include "TDDSCaps.h"
#include "TDDSCapsEditor.h"

//---------------------------------------------------------------------------

__fastcall      TDDSCapsPropertyEditor::TDDSCapsPropertyEditor()
    : TPropertyEditor()
{
}

//---------------------------------------------------------------------------
// Method:          GetValue()
// Description:     This method returns the string that the object
//                  inspector displays next to the property name.
//---------------------------------------------------------------------------
AnsiString          __fastcall      TDDSCapsPropertyEditor::GetValue()
{
// retrieve a pointer to the property the object inspector is talking about
TDDSCaps* caps = (TDDSCaps*) GetOrdValue();

// return a string for the object inspector to display
return To_TDDSCapsCapsString( caps->Caps );
}

//---------------------------------------------------------------------------
// Method:          SetValue()
// Description:     This method sets the TDDTemplate's internal values
//                  by parsing the string as required.
//---------------------------------------------------------------------------
void                __fastcall      TDDSCapsPropertyEditor::SetValue(
                                                        AnsiString value )
{
// get a pointer to the property the object inspector is talking about
TDDSCaps* caps = (TDDSCaps*) GetOrdValue();

// set the relevant parameters based on what the object inspector supplied.
caps->Caps = From_TDDSCapsCapsString( value );
}

//---------------------------------------------------------------------------
// Method:          GetAttributes()
// Description:     This method returns the attributes that tell the
//                  object inspector what functions to perform when
//                  editing the TDDTemplate property
//---------------------------------------------------------------------------
TPropertyAttributes __fastcall      TDDSCapsPropertyEditor::GetAttributes()
{
// we want to open a dialog & the property should stay intact if unchanged
return TPropertyAttributes() << paDialog << paRevertable;
}

//---------------------------------------------------------------------------
// Method:          Edit()
// Description:     This method opens a custom TDDTemplate
//                  dialog box editor which is used to modify the various
//                  properties of the TDDTemplate class.
//---------------------------------------------------------------------------
void                __fastcall      TDDSCapsPropertyEditor::Edit()
{
// try creating the editor
TDDSCapsEditor* editor = new TDDSCapsEditor(Application);
if (!editor) return;

// get a pointer to the property the object inspector is talking about
TDDSCaps* caps = (TDDSCaps*) GetOrdValue();

// setup the default values used by the editor
editor->SCaps->Caps = caps->Caps;

// if the dialog was clicked OK.... update the appropriate values, then tell
// the designer that the values have changed
if (editor->ShowModal() == ID_OK)
    {
    caps->Caps = editor->SCaps->Caps;
    Designer->Modified();
    }
delete editor;
}
//---------------------------------------------------------------------------

