// --------------------------------------------------------------------------
#include <vcl\vcl.h>
#pragma hdrstop

// ==========================================================================
// File:            TDDBltBatch.CPP
// Description:     This file contains the code for the TDDBltBatch wrapper
//                  object used by the TDirectDrawSurface::BltBatch() method.
//
// (c) 1998 A.S.C. Alternate Software Concepts, Sydney, Australia.
// All Rights Reserved. Refer to the file TDirectX_Licence.Txt for detailed
// licence & copyright information.
// ==========================================================================

// --------------------------------------------------------------------------
// include the TDDBltBatch H file.
// --------------------------------------------------------------------------
#include "TDDBltBatch.h"

// ==========================================================================
// Class:           TDDBltBatch
// Description:     This object is a BCB-style wrapper for the DirectX
//                  DDBLTBATCH structure
// Notes:           By setting the various properties of a TDDBltBatch object
//                  you can specify the type of blit operations to be
//                  performed when calling the TDirectDrawSurface::BltBatch()
//                  method.
// ==========================================================================

// --------------------------------------------------------------------------
// Constructor
// --------------------------------------------------------------------------
                    __fastcall  TDDBltBatch::TDDBltBatch() : TPersistent()
{
}

// --------------------------------------------------------------------------
// Destructor
// --------------------------------------------------------------------------
                    __fastcall  TDDBltBatch::~TDDBltBatch()
{
}

// --------------------------------------------------------------------------
// Method:          Ptr
// Description:     This method returns a pointer to the internal
//                  DDBLTBATCH structure.
// --------------------------------------------------------------------------
DDBLTBATCH*         __fastcall  TDDBltBatch::Ptr()
{
return &fBltBatch;
}

// --------------------------------------------------------------------------
// Method:          FGetBltFX                                                   //added get
// Description:     Returns the additional effects to be used during the blit
// --------------------------------------------------------------------------
TDDBltFX*           __fastcall  TDDBltBatch::FGetBltFX()
{
return NULL;
}

// --------------------------------------------------------------------------
// Method:          FSetBltFX
// Description:     Sets additional effects to be used during the blit.
// --------------------------------------------------------------------------
void                __fastcall  TDDBltBatch::FSetBltFX( TDDBltFX* pBltFX )
{
if (pBltFX!=NULL) fBltBatch.lpDDBltFx = pBltFX->Ptr();
}

// --------------------------------------------------------------------------
// Method:          FGetDestRect
// Description:     Gets the destination rectangle to use for the blit.
// Note:            Only the pixels on the destination surface within the
//                  rectangle are to be filled with new pixel data.
// --------------------------------------------------------------------------
TRect               __fastcall  TDDBltBatch::FGetDestRect()
{
TRect rect = Rect( fBltBatch.lprDest->left,
                    fBltBatch.lprDest->top,
                    fBltBatch.lprDest->right,
                    fBltBatch.lprDest->bottom );
return rect;
}

// --------------------------------------------------------------------------
// Method:          FSetDestRect
// Description:     Sets the destination rectangle to use for the blit.
// Note:            Only the pixels on the destination surface within the
//                  rectangle are filled with new pixel data.
// --------------------------------------------------------------------------
void                __fastcall  TDDBltBatch::FSetDestRect( TRect pDestRect )
{
fBltBatch.lprDest->top = pDestRect.Top;
fBltBatch.lprDest->left = pDestRect.Left;
fBltBatch.lprDest->right = pDestRect.Right;
fBltBatch.lprDest->bottom = pDestRect.Bottom;
}

// --------------------------------------------------------------------------
// Method:          FGetFlags
// Description:     Gets the current blit control flag settings.
// --------------------------------------------------------------------------
dword               __fastcall  TDDBltBatch::FGetFlags()
{
return fBltBatch.dwFlags;
}

// --------------------------------------------------------------------------
// Method:          FSetFlags
// Description:     Sets the blit control flags to the type of blit required.
// --------------------------------------------------------------------------
void                __fastcall  TDDBltBatch::FSetFlags( dword pFlags )
{
fBltBatch.dwFlags = pFlags;
}

// --------------------------------------------------------------------------
// Method:          FGetSize
// Description:     This returns the size of the internal DDBLTBATCH
//                  structure.
// --------------------------------------------------------------------------
dword               __fastcall  TDDBltBatch::FGetSize()
{
return sizeof(DDBLTBATCH);
}

// --------------------------------------------------------------------------
// Method:          FSetSize                                                    //added set
// Description:     This sets the size of the internal DDBLTBATCH structure.
// --------------------------------------------------------------------------
void               __fastcall  TDDBltBatch::FSetSize( dword pSize )
{
return;
}

// --------------------------------------------------------------------------
// Property:        FGetSrcRect
// Description:     Gets the source rectangle to be used for the blit.
// Note:            Only the pixels on the source surface within the
//                  rectangle are copied to the destination surface.
// --------------------------------------------------------------------------
TRect               __fastcall  TDDBltBatch::FGetSrcRect()
{
TRect rect = Rect( fBltBatch.lprDest->left,
                    fBltBatch.lprDest->top,
                    fBltBatch.lprDest->right,
                    fBltBatch.lprDest->bottom );
return rect;
}

// --------------------------------------------------------------------------
// Property:        FSetSrcRect
// Description:     Sets the source rectangle to be used for the blit.
// Note:            Only the pixels on the source surface within the
//                  rectangle are copied to the destination surface.
// --------------------------------------------------------------------------
void                __fastcall  TDDBltBatch::FSetSrcRect( TRect pSrcRect )
{
fBltBatch.lprDest->top = pSrcRect.Top;
fBltBatch.lprDest->left = pSrcRect.Left;
fBltBatch.lprDest->right = pSrcRect.Right;
fBltBatch.lprDest->bottom = pSrcRect.Bottom;
}

// --------------------------------------------------------------------------
// Method:          FGetSrcSurface                                              //added get
// Description:     Returns a pointer to a TDirectDrawSurface used as the
//                  pixel data source for the blit
// --------------------------------------------------------------------------
TDirectDrawSurface* __fastcall  TDDBltBatch::FGetSrcSurface()
{
return NULL;
}

// --------------------------------------------------------------------------
// Method:          FSetSrcSurface
// Description:     Sets a pointer to a TDirectDrawSurface to use as the
//                  pixel data source for the blit.
// --------------------------------------------------------------------------
void                __fastcall  TDDBltBatch::FSetSrcSurface(
                                            TDirectDrawSurface* pSrcSurface )
{
if (pSrcSurface!=NULL) fBltBatch.lpDDSSrc = (LPDIRECTDRAWSURFACE)
                                                            pSrcSurface->Ptr;
}

// --------------------------------------------------------------------------

