// --------------------------------------------------------------------------
#ifndef TDDBltBatchH
#define TDDBltBatchH

// ==========================================================================
// File:            TDDBltBatch.H
// Description:     This file contains the definition for the TDDBltBatch
//                  wrapper object used by the TDirectDrawSurface::BltBatch()
//                  method.
//
// (c) 1998 A.S.C. Alternate Software Concepts, Sydney, Australia.
// All Rights Reserved. Refer to the file TDirectX_Licence.Txt for detailed
// licence & copyright information.
// ==========================================================================

// --------------------------------------------------------------------------
// include the basic TDirectDraw definitions, etc.
// --------------------------------------------------------------------------
#include "TDirectDrawInclude.h"
#include "TDirectDrawDefs.h"
#include "TDirectDrawErrors.h"
#include "TDirectDrawMisc.h"
// --------------------------------------------------------------------------
// include the various DirectX Wrapper classes used by TDDBltBatch
// --------------------------------------------------------------------------
#include "TDDBltFX.h"
// --------------------------------------------------------------------------
// include the various DirectX Component classes used by TDDBltBatch
// --------------------------------------------------------------------------
#include "TDirectDrawSurface.h"

// ==========================================================================
// Class:           TDDBltBatch
// Description:     This object is a BCB-style wrapper for the DirectX
//                  DDBLTBATCH structure
// Notes:           By setting the various properties of a TDDBltBatch object
//                  you can specify the type of blit operations to be
//                  performed when calling the TDirectDrawSurface::BltBatch()
//                  method.
// ==========================================================================
class TDDBltBatch : public TPersistent
{
__published:
public:

// --------------------------------------------------------------------------
// Property:        BltFX
// Description:     This is a pointer to a TDDBltFX object containing
//                  additional effects to be used during the blit.
// --------------------------------------------------------------------------
    __property      TDDBltFX*                 BltFX = { read=FGetBltFX,         //added read
                                                        write=FSetBltFX,
                                                        nodefault };

// --------------------------------------------------------------------------
// Property:        DestRect
// Description:     This is the destination rectangle used during the blit.
// Note:            Only the pixels on the destination surface within the
//                  rectangle are filled with new pixel data.
// --------------------------------------------------------------------------
    __property      TRect                  DestRect = { read=FGetDestRect,
                                                        write=FSetDestRect,
                                                        nodefault };

// --------------------------------------------------------------------------
// Property:        Flags
// Description:     These are optional blit control flags that are set to the
//                  type of blit required.
// --------------------------------------------------------------------------
    __property      dword                     Flags = { read=FGetFlags,
                                                        write=FSetFlags,
                                                        nodefault };

// --------------------------------------------------------------------------
// Property:        Size
// Description:     This contains the size of the internal DDBLTBATCH
//                  structure.
// --------------------------------------------------------------------------
    __property      dword                      Size = { read=FGetSize,
                                                        write=FSetSize,         //added write
                                                        nodefault };            //added default

// --------------------------------------------------------------------------
// Property:        SrcRect
// Description:     This is the source rectangle used during the blit.
// Note:            Only the pixels on the source surface within the
//                  rectangle are copied to the destination surface.
// --------------------------------------------------------------------------
    __property      TRect                   SrcRect = { read=FGetSrcRect,
                                                        write=FSetSrcRect,
                                                        nodefault };

// --------------------------------------------------------------------------
// Property:        SrcSurface
// Description:     This is a pointer to a TDirectDrawSurface that will be
//                  used as the pixel data source for the blit.
// --------------------------------------------------------------------------
    __property      TDirectDrawSurface*  SrcSurface = { read=FGetSrcSurface,    //added read
                                                        write=FSetSrcSurface,
                                                        nodefault };

// --------------------------------------------------------------------------
// Constructor & Destructor
// --------------------------------------------------------------------------
                    __fastcall   TDDBltBatch();
    virtual         __fastcall   ~TDDBltBatch();

// --------------------------------------------------------------------------
// Method:          Ptr
// Description:     This returns a pointer to the internal
//                  DDBLTBATCH structure.
// --------------------------------------------------------------------------
    DDBLTBATCH*     __fastcall  Ptr();

// --------------------------------------------------------------------------
// End of the TDDBltBatch wrapper
// --------------------------------------------------------------------------

protected:

    TDDBltFX*       __fastcall  FGetBltFX ();                                   //added get
    void            __fastcall  FSetBltFX( TDDBltFX* pBltFX );
    TRect           __fastcall  FGetDestRect();
    void            __fastcall  FSetDestRect( TRect pDestRect );
    dword           __fastcall  FGetFlags();
    void            __fastcall  FSetFlags( dword pFlags );
    dword           __fastcall  FGetSize();
    void            __fastcall  FSetSize( dword pSize );                        //added set
    TRect           __fastcall  FGetSrcRect();
    void            __fastcall  FSetSrcRect( TRect pSrcRect );
TDirectDrawSurface* __fastcall  FGetSrcSurface();                               //added get
    void            __fastcall  FSetSrcSurface( TDirectDrawSurface*
                                                            pSrcSurface );

private:

    DDBLTBATCH      fBltBatch;

};
// --------------------------------------------------------------------------
#endif
