// --------------------------------------------------------------------------
#ifndef TDDBltFXH
#define TDDBltFXH

// ==========================================================================
// File:            TDDBltFX.H
// Description:
//
// (c) 1998 A.S.C. Alternate Software Concepts, Sydney, Australia. All Rights
// Reserved. Refer to the file TDirectX_Licence.Txt for detailed licence &
// copyright information.
// ==========================================================================

// --------------------------------------------------------------------------
// include the basic TDirectDraw definitions, etc.
// --------------------------------------------------------------------------
#include "TDirectDrawInclude.h"
#include "TDirectDrawDefs.h"
#include "TDirectDrawErrors.h"
#include "TDirectDrawMisc.h"
// --------------------------------------------------------------------------
// include the various DirectX Wrapper classes used by TDDBltFX.
// --------------------------------------------------------------------------
#include "TDDColorKey.h"
// --------------------------------------------------------------------------
// include the various DirectX Component classes used by TDDBltFX.
// --------------------------------------------------------------------------
// circular reference problem
// #include "TDirectDrawSurface.h"
class TDirectDrawSurface;

// --------------------------------------------------------------------------
// Class:           TDDBltFX
// Description:     The TDDBltFX object passes raster operations, effects and
//                  override information to the TDirectDrawSurface::Blt
//                  method. This structure is also part of the DDBLTBATCH
//                  structure used with the TDirectDrawSurface::BltBatch
//                  method.
// --------------------------------------------------------------------------
class TDDBltFX : public TPersistent
{
__published:
public:

// --------------------------------------------------------------------------
// Property:        AlphaDest
// Description:     Surface used as the alpha channel destination
// --------------------------------------------------------------------------
    __property      TDirectDrawSurface*   AlphaDest = { read=FGetAlphaDest,     //added read
                                                        write=FSetAlphaDest,
                                                        nodefault };

// --------------------------------------------------------------------------
// Property:        AlphaDestConst
// Description:     Constant used as the alpha channel destination
// --------------------------------------------------------------------------
    __property      dword            AlphaDestConst = {
                                                    read=FGetAlphaDestConst,
                                                    write=FSetAlphaDestConst,
                                                    nodefault };

// --------------------------------------------------------------------------
// Property:        AlphaDestConstBitDepth
// Description:     Bit depth of the destination alpha constant
// --------------------------------------------------------------------------
    __property      dword    AlphaDestConstBitDepth = {
                                            read=FGetAlphaDestConstBitDepth,
                                            write=FSetAlphaDestConstBitDepth,
                                            nodefault };

// --------------------------------------------------------------------------
// Property:        AlphaEdgeBlend
// Description:     Alpha constant used for edge blending
// --------------------------------------------------------------------------
    __property      dword            AlphaEdgeBlend = {
                                                    read=FGetAlphaEdgeBlend,
                                                    write=FSetAlphaEdgeBlend,
                                                    nodefault };

// --------------------------------------------------------------------------
// Property:        AlphaEdgeBlendBitDepth
// Description:     Bit depth of the constant for an alpha edge blend
// --------------------------------------------------------------------------
    __property      dword    AlphaEdgeBlendBitDepth = {
                                            read=FGetAlphaEdgeBlendBitDepth,
                                            write=FSetAlphaEdgeBlendBitDepth,
                                            nodefault };

// --------------------------------------------------------------------------
// Property:        AlphaSrc
// Description:     Surface used as the alpha channel source
// --------------------------------------------------------------------------
    __property      TDirectDrawSurface*    AlphaSrc = { read=FGetAlphaSrc,      //added read
                                                        write=FSetAlphaSrc,
                                                        nodefault };

// --------------------------------------------------------------------------
// Property:        AlphaSrcConst
// Description:     Constant used as the alpha channel source
// --------------------------------------------------------------------------
    __property      dword             AlphaSrcConst = {
                                                    read=FGetAlphaSrcConst,
                                                    write=FSetAlphaSrcConst,
                                                    nodefault };

// --------------------------------------------------------------------------
// Property:        AlphaSrcConstBitDepth
// Description:     Bit depth of the source alpha constant
// --------------------------------------------------------------------------
    __property      dword     AlphaSrcConstBitDepth = {
                                            read=FGetAlphaSrcConstBitDepth,
                                            write=FSetAlphaSrcConstBitDepth,
                                            nodefault };

// --------------------------------------------------------------------------
// Property:        DDFX
// Description:     Type of FX operations
// --------------------------------------------------------------------------
    __property      dword                      DDFX = { read=FGetDDFX,
                                                        write=FSetDDFX,
                                                        nodefault };

// --------------------------------------------------------------------------
// Property:        DDRop
// Description:     DirectDraw raster operations
// --------------------------------------------------------------------------
    __property      dword                     DDRop = { read=FGetDDRop,
                                                        write=FSetDDRop,
                                                        nodefault };

// --------------------------------------------------------------------------
// Property:        DestColorKey
// Description:     Destination color key override
// --------------------------------------------------------------------------
    __property      TDDColorKey*       DestColorKey = {
                                                    read=FGetDestColorKey,      //added read
                                                    write=FSetDestColorKey,
                                                    nodefault};

// --------------------------------------------------------------------------
// Property:        FillColor
// Description:     Color used to fill a surface when ddblt_ColorFill is
//                  specified
// Notes:           This value must be a pixel appropriate to the pixel
//                  format of the destination surface. For a palettized
//                  surface it would be a palette index, and for a 16-bit RGB
//                  surface it would be a 16-bit pixel value
// --------------------------------------------------------------------------
    __property      dword                 FillColor = { read=FGetFillColor,
                                                        write=FSetFillColor,
                                                        nodefault };

// --------------------------------------------------------------------------
// Property:        FillDepth
// Description:     Depth value of the z-buffer
// --------------------------------------------------------------------------
    __property      dword                 FillDepth = { read=FGetFillDepth,
                                                        write=FSetFillDepth,
                                                        nodefault };

// --------------------------------------------------------------------------
// Property:        FillPixel
// Description:     Pixel value for RGBA or RGBZ fills
// Notes:           Applications that use RGBZ fills should use this property
//                  rather than FillColor or FillDepth
// --------------------------------------------------------------------------
    __property      dword                 FillPixel = { read=FGetFillPixel,
                                                        write=FSetFillPixel,
                                                        nodefault };

// --------------------------------------------------------------------------
// Property:        Pattern
// Description:     Surface to use as a pattern
// Note:            The pattern can be used in certain blit operations that
//                  combine a source and a destination
// --------------------------------------------------------------------------
    __property      TDirectDrawSurface*     Pattern = { read=FGetPattern,       //added read
                                                        write=FSetPattern,
                                                        nodefault };

// --------------------------------------------------------------------------
// Property:        Rop
// Description:     Win32 raster operations
// Notes:           You can retrieve a list of supported raster operations by
//                  calling the TDirectDraw::GetCaps method
// --------------------------------------------------------------------------
    __property      dword                       Rop = { read=FGetRop,
                                                        write=FSetRop,
                                                        nodefault };

// --------------------------------------------------------------------------
// Property:        RotationAngle
// Description:     Rotation angle for the blit
// --------------------------------------------------------------------------
    __property      dword             RotationAngle = {
                                                    read=FGetRotationAngle,
                                                    write=FSetRotationAngle,
                                                    nodefault };

// --------------------------------------------------------------------------
// Property:        Size
// Description:     This contains the size of the internal DDBLTFX
//                  structure.
// --------------------------------------------------------------------------
    __property      dword                      Size = { read=FGetSize,
                                                        write=FSetSize,         //added write
                                                        nodefault };            //added default

// --------------------------------------------------------------------------
// Property:        SrcColorKey
// Description:     Source color key override
// --------------------------------------------------------------------------
    __property      TDDColorKey*        SrcColorKey = { read=FGetSrcColorKey,   //added read
                                                        write=FSetSrcColorKey
                                                        ,nodefault};

// --------------------------------------------------------------------------
// Property:        ZBufferBaseDest
// Description:     Destination base value of a z-buffer
// --------------------------------------------------------------------------
    __property      dword           ZBufferBaseDest = {
                                                    read=FGetZBufferBaseDest,
                                                    write=FSetZBufferBaseDest
                                                    ,nodefault };

// --------------------------------------------------------------------------
// Property:        ZBufferDest
// Description:     Surface used as the z-buffer destination
// --------------------------------------------------------------------------
    __property      TDirectDrawSurface* ZBufferDest = { read=FGetZBufferDest,   //added read
                                                        write=FSetZBufferDest
                                                        ,nodefault };

// --------------------------------------------------------------------------
// Property:        ZBufferHigh
// Description:     High Limit of a z-buffer
// --------------------------------------------------------------------------
    __property      dword               ZBufferHigh = { read=FGetZBufferHigh,
                                                        write=FSetZBufferHigh
                                                        ,nodefault };

// --------------------------------------------------------------------------
// Property:        ZBufferLow
// Description:     Low limit of a z-buffer
// --------------------------------------------------------------------------
    __property      dword                ZBufferLow = { read=FGetZBufferLow,
                                                        write=FSetZBufferLow,
                                                        nodefault };

// --------------------------------------------------------------------------
// Property:        ZBufferOpCode
// Description:     Z-Buffer Compares
// --------------------------------------------------------------------------
    __property      dword             ZBufferOpCode = {
                                                    read=FGetZBufferOpCode,
                                                    write=FSetZBufferOpCode,
                                                    nodefault };

// --------------------------------------------------------------------------
// Property:        ZBufferSrc
// Description:     Surface used as the z-buffer source
// --------------------------------------------------------------------------
    __property      TDirectDrawSurface* ZBufferSrc  = { read=FGetZBufferSrc,    //added read
                                                        write=FSetZBufferSrc,
                                                        nodefault };

// --------------------------------------------------------------------------
// Property:        ZDestConst
// Description:     Constant used as the z-buffer destination
// --------------------------------------------------------------------------
    __property      dword                ZDestConst = { read=FGetZDestConst,
                                                        write=FSetZDestConst,
                                                        nodefault };

// --------------------------------------------------------------------------
// Property:        ZDestConstBitDepth
// Description:     Bit depth of the destination z-constant
// --------------------------------------------------------------------------
    __property      dword        ZDestConstBitDepth = {
                                                read=FGetZDestConstBitDepth,
                                                write=FSetZDestConstBitDepth,
                                                nodefault };

// --------------------------------------------------------------------------
// Property:        ZSrcConst
// Description:     Constant used as the zbuffer source
// --------------------------------------------------------------------------
    __property      dword                 ZSrcConst = { read=FGetZSrcConst,
                                                        write=FSetZSrcConst,
                                                        nodefault };

// --------------------------------------------------------------------------
// Property:        ZSrcConstBitDepth
// Description:     Bit depth of the source z-constant
// --------------------------------------------------------------------------
    __property      dword         ZSrcConstBitDepth = {
                                                read=FGetZSrcConstBitDepth,
                                                write=FSetZSrcConstBitDepth,
                                                nodefault };

// --------------------------------------------------------------------------
// Constructor & Destructor
// --------------------------------------------------------------------------
                        __fastcall  TDDBltFX();
    virtual             __fastcall  ~TDDBltFX();

// --------------------------------------------------------------------------
// Method:          Ptr
// Description:     This property returns a pointer to the internal
//                  DDBLTFX structure
// --------------------------------------------------------------------------
    DDBLTFX*            __fastcall  Ptr();

protected:

TDirectDrawSurface* __fastcall  FGetAlphaDest();                                //added Get
    void            __fastcall  FSetAlphaDest( TDirectDrawSurface*
                                                                alphadest );
    dword           __fastcall  FGetAlphaDestConst();
    void            __fastcall  FSetAlphaDestConst( dword alphadestconst );
    dword           __fastcall  FGetAlphaDestConstBitDepth();
    void            __fastcall  FSetAlphaDestConstBitDepth( dword
                                                    alphadestconstbitdepth );
    dword           __fastcall  FGetAlphaEdgeBlend();
    void            __fastcall  FSetAlphaEdgeBlend( dword alphaedgeblend );
    dword           __fastcall  FGetAlphaEdgeBlendBitDepth();
    void            __fastcall  FSetAlphaEdgeBlendBitDepth( dword
                                                    alphaedgeblendbitdepth );
TDirectDrawSurface* __fastcall  FGetAlphaSrc();                                 //added Get
    void            __fastcall  FSetAlphaSrc( TDirectDrawSurface* alphasrc );
    dword           __fastcall  FGetAlphaSrcConst();
    void            __fastcall  FSetAlphaSrcConst( dword alphasrcconst );
    dword           __fastcall  FGetAlphaSrcConstBitDepth();
    void            __fastcall  FSetAlphaSrcConstBitDepth( dword
                                                    alphasrcconstbitdepth );
    dword           __fastcall  FGetDDFX();
    void            __fastcall  FSetDDFX( dword ddfx );
    dword           __fastcall  FGetDDRop();
    void            __fastcall  FSetDDRop( dword ddrop );
    TDDColorKey*    __fastcall  FGetDestColorKey();                             //added Get
    void            __fastcall  FSetDestColorKey( TDDColorKey*
                                                            destcolorkey );
    dword           __fastcall  FGetFillColor();
    void            __fastcall  FSetFillColor( dword fillcolor );
    dword           __fastcall  FGetFillDepth();
    void            __fastcall  FSetFillDepth( dword filldepth );
    dword           __fastcall  FGetFillPixel();
    void            __fastcall  FSetFillPixel( dword fillpixel );
TDirectDrawSurface* __fastcall  FGetPattern();                                  //added Get
    void            __fastcall  FSetPattern( TDirectDrawSurface* pattern );
    dword           __fastcall  FGetRop();
    void            __fastcall  FSetRop( dword rop );
    dword           __fastcall  FGetRotationAngle();
    void            __fastcall  FSetRotationAngle( dword rotationangle );
    dword           __fastcall  FGetSize();
    void            __fastcall  FSetSize( dword pSize );                        //added set
    TDDColorKey*    __fastcall  FGetSrcColorKey();                              //added Get
    void            __fastcall  FSetSrcColorKey( TDDColorKey* srccolorkey );
    dword           __fastcall  FGetZBufferBaseDest();
    void            __fastcall  FSetZBufferBaseDest( dword zbufferbasedest );
TDirectDrawSurface* __fastcall  FGetZBufferDest();                              //added Get
    void            __fastcall  FSetZBufferDest( TDirectDrawSurface*
                                                            zbufferdest );
    dword           __fastcall  FGetZBufferHigh();
    void            __fastcall  FSetZBufferHigh( dword zbufferhigh );
    dword           __fastcall  FGetZBufferLow();
    void            __fastcall  FSetZBufferLow( dword zbufferlow );
    dword           __fastcall  FGetZBufferOpCode();
    void            __fastcall  FSetZBufferOpCode( dword zbufferopcode );
TDirectDrawSurface* __fastcall  FGetZBufferSrc();                               //added Get
    void            __fastcall  FSetZBufferSrc( TDirectDrawSurface*
                                                                zbuffersrc );
    dword           __fastcall  FGetZDestConst();
    void            __fastcall  FSetZDestConst( dword zdestconst );
    dword           __fastcall  FGetZDestConstBitDepth();
    void            __fastcall  FSetZDestConstBitDepth( dword
                                                        zdestconstbitdepth );
    dword           __fastcall  FGetZSrcConst();
    void            __fastcall  FSetZSrcConst( dword zsrcconst );
    dword           __fastcall  FGetZSrcConstBitDepth();
    void            __fastcall  FSetZSrcConstBitDepth( dword
                                                        zsrcconstbitdepth );

private:

    DDBLTFX         fBltFX;

};
// --------------------------------------------------------------------------
#endif


