// --------------------------------------------------------------------------
#include <vcl\vcl.h>
#pragma hdrstop
// ==========================================================================
// File:            TDDColorControl.CPP
// Description:
//
// (c) 1998 A.S.C. Alternate Software Concepts, Sydney, Australia.
// All Rights Reserved. Refer to the file TDirectX_Licence.Txt for detailed
// licence & copyright information.
// ==========================================================================

// --------------------------------------------------------------------------
// include the TDDColorControl H file.
// --------------------------------------------------------------------------
#include "TDDColorControl.h"

// ==========================================================================
// Class:           TDDBltBatch
// Description:     This object is a BCB-style wrapper for the DirectX
//                  DDBLTBATCH structure
// Notes:           By setting the various properties of a TDDBltBatch object
//                  you can specify the type of blit operations to be
//                  performed when calling the TDirectDrawSurface::BltBatch()
//                  method.
// ==========================================================================

// --------------------------------------------------------------------------
// Constructor
// --------------------------------------------------------------------------
                    __fastcall      TDDColorControl::TDDColorControl() :
                                                                TPersistent()
{
fColorControl.dwSize = sizeof(DDCOLORCONTROL);
}

// --------------------------------------------------------------------------
// Destructor
//---------------------------------------------------------------------------
                    __fastcall      TDDColorControl::~TDDColorControl()
{
}

// --------------------------------------------------------------------------
// Method:          Ptr
// Description:     This method returns a pointer to the internal
//                  DDCOLORCONTROL structure.
//---------------------------------------------------------------------------
DDCOLORCONTROL*     __fastcall      TDDColorControl::Ptr()
{
return &fColorControl;
}

// --------------------------------------------------------------------------
// Method:          FGetBrightness
// Description:     Returns the luminance intensity (Black Level) in IRE
//                  units*100.
// Note:            Range is 0 to 10,000. The default is 750 (7.5 IRE)
//---------------------------------------------------------------------------
long                __fastcall      TDDColorControl::FGetBrightness()
{
return fColorControl.lBrightness;
}

// --------------------------------------------------------------------------
// Method:          FSetBrightness
// Description:     Sets the luminance intensity (Black Level) in IRE
//                  units*100.
// Note:            Range is 0 to 10,000. The default is 750 (7.5 IRE)
//---------------------------------------------------------------------------
void                __fastcall      TDDColorControl::FSetBrightness( long
                                                                brightness )
{
fColorControl.lBrightness = brightness;
}

// --------------------------------------------------------------------------
// Method:          FGetColorEnable
// Description:     Returns a flag indicating whether color is used or not.
// Note:            If this member is zero, color is not used; if it is 1,
//                  then color is used. The default value is 1
//---------------------------------------------------------------------------
long                __fastcall      TDDColorControl::FGetColorEnable()
{
return fColorControl.lColorEnable;
}

// --------------------------------------------------------------------------
// Method:          FSetColorEnable
// Description:     Sets a flag indicating whether color is used or not.
// Note:            If this member is zero, color is not used; if it is 1,
//                  then color is used. The default value is 1
//---------------------------------------------------------------------------
void                __fastcall      TDDColorControl::FSetColorEnable( long
                                                                colorenable )
{
fColorControl.lColorEnable = colorenable;
}

// --------------------------------------------------------------------------
// Method:          FGetContrast
// Description:     Returns current difference between higher and lower
//                  intensity luminance values in IRE units*100.
// Note:            The valid range is 0 to 20,000. The default value is
//                  10,000 (100 IRE). Higher values of contrast cause darker
//                  luminance values to tend towards black, and cause lighter
//                  luminance values to tend towards white. Lower values of
//                  contrast cause all luminance values to move towards the
//                  middle luminance values
//---------------------------------------------------------------------------
long                __fastcall      TDDColorControl::FGetContrast()
{
return fColorControl.lContrast;
}

// --------------------------------------------------------------------------
// Method:          FSetContrast
// Description:     Sets the relative difference between higher and lower
//                  intensity luminance values in IRE units*100.
// Note:            The valid range is 0 to 20,000. The default value is
//                  10,000 (100 IRE). Higher values of contrast cause darker
//                  luminance values to tend towards black, and cause lighter
//                  luminance values to tend towards white. Lower values of
//                  contrast cause all luminance values to move towards the
//                  middle luminance values
//---------------------------------------------------------------------------
void                __fastcall      TDDColorControl::FSetContrast( long
                                                                contrast )
{
fColorControl.lContrast = contrast;
}

// --------------------------------------------------------------------------
// Method:          FGetFlags
// Description:     Returns flags specifying which object properties contain
//                  valid data.
// Note:            When the object is returned by the
//                  TDirectDrawColorControl::GetColorControls method, it also
//                  indicates which options are supported by the device
//---------------------------------------------------------------------------
dword       __fastcall      TDDColorControl::FGetFlags()
{
return fColorControl.dwFlags;
}

// --------------------------------------------------------------------------
// Method:          FSetFlags
// Description:     Sets flags specifying which object properties contain
//                  valid data.
//---------------------------------------------------------------------------
void                __fastcall      TDDColorControl::FSetFlags( dword flags )
{
fColorControl.dwFlags = flags;
}

// --------------------------------------------------------------------------
// Method:          FGetGamma
// Description:     Returns the current amount of gamma correction applied to
//                  the luminance values.
// Note:            The valid range is 1 to 500 gamma units, with a default
//                  of 1
//---------------------------------------------------------------------------
long                __fastcall      TDDColorControl::FGetGamma()
{
return fColorControl.lGamma;
}

// --------------------------------------------------------------------------
// Method:          FSetGamma
// Description:     Sets the amount of gamma correction to be applied to the
//                  luminance values.
// Note:            The valid range is 1 to 500 gamma units, with a default
//                  of 1
//---------------------------------------------------------------------------
void                __fastcall      TDDColorControl::FSetGamma( long gamma )
{
fColorControl.lGamma = gamma;
}

// --------------------------------------------------------------------------
// Method:          FGetHue
// Description:     Returns current phase relationship of the chrominance
//                  components.
// Note:            Hue is specified in degrees and the valid range is -180
//                  to 180. The default is 0
//---------------------------------------------------------------------------
long                __fastcall      TDDColorControl::FGetHue()
{
return fColorControl.lHue;
}

// --------------------------------------------------------------------------
// Method:          FSetHue
// Description:     Sets the phase relationship of the chrominance
//                  components.
// Note:            Hue is specified in degrees and the valid range is -180
//                  to 180. The default is 0
//---------------------------------------------------------------------------
void                __fastcall      TDDColorControl::FSetHue( long hue )
{
fColorControl.lHue = hue;
}

// --------------------------------------------------------------------------
// Method:          FGetSaturation
// Description:     Returns current color intensity in IRE units*100.
// Note:            The valid range is 0 to 20,000. The default value is
//                  10,000 (100 IRE)
//---------------------------------------------------------------------------
long                __fastcall      TDDColorControl::FGetSaturation()
{
return fColorControl.lSaturation;
}

// --------------------------------------------------------------------------
// Method:          FSetSaturation
// Description:     Sets the color intensity in IRE units*100.
// Note:            The valid range is 0 to 20,000. The default value is
//                  10,000 (100 IRE)
//---------------------------------------------------------------------------
void                __fastcall      TDDColorControl::FSetSaturation( long
                                                                saturation )
{
fColorControl.lSaturation = saturation;
}

// --------------------------------------------------------------------------
// Method:          FGetSharpness
// Description:     Returns the current sharpness in arbitrary units.
// Note:            The valid range is 0 to 10. The default value is 5
//---------------------------------------------------------------------------
long                __fastcall      TDDColorControl::FGetSharpness()
{
return fColorControl.lSharpness;
}

// --------------------------------------------------------------------------
// Method:          FSetSharpness
// Description:     Sets the sharpness in arbitrary units.
// Note:            The valid range is 0 to 10. The default value is 5
// --------------------------------------------------------------------------
void                __fastcall      TDDColorControl::FSetSharpness( long
                                                                sharpness )
{
fColorControl.lSharpness = sharpness;
}

// --------------------------------------------------------------------------
// Method:          FGetSize
// Description:     This returns the size of the internal DDCOLORCONTROL
//                  structure
// --------------------------------------------------------------------------
dword               __fastcall  TDDColorControl::FGetSize()
{
return sizeof(DDCOLORCONTROL);
}

// --------------------------------------------------------------------------
// Method:          FSetSize                                                    //added set
// Description:     This sets the size of the internal DDCOLORCONTROL
//                  structure
// --------------------------------------------------------------------------
void               __fastcall  TDDColorControl::FSetSize( dword pSize )
{
return;
}

// --------------------------------------------------------------------------
