//---------------------------------------------------------------------------
#ifndef TDDColorControlH
#define TDDColorControlH

// ==========================================================================
// File:            TDDColorControl.H
// Description:
//
// (c) 1998 A.S.C. Alternate Software Concepts, Sydney, Australia.
// All Rights Reserved. Refer to the file TDirectX_Licence.Txt for detailed
// licence & copyright information.
// ==========================================================================

// --------------------------------------------------------------------------
// include the basic TDirectDraw definitions, etc.
// --------------------------------------------------------------------------
#include "TDirectDrawInclude.h"
#include "TDirectDrawDefs.h"
#include "TDirectDrawErrors.h"
// ==========================================================================
// Class:           TDDColorControl
// Description:     The TDDColorControl object defines the color controls
//                  associated with a TDirectDrawVideoPortObject, an overlay
//                  surface, or a primary surface
// Notes:
// ==========================================================================
class TDDColorControl : public TPersistent
{
__published:
public:

// --------------------------------------------------------------------------
// Property:        Brightness
// Description:     Luminance intensity (Black Level) in IRE units*100.
// Note:            Range is 0 to 10,000. The default is 750 (7.5 IRE)
// --------------------------------------------------------------------------
    __property      long                 Brightness = { read=FGetBrightness,
                                                        write=FSetBrightness,
                                                        nodefault };

// --------------------------------------------------------------------------
// Property:        ColorEnable
// Description:     Flag indicating whether color is used.
// Note:            If this member is zero, color is not used; if it is 1,
//                  then color is used. The default value is 1
// --------------------------------------------------------------------------
    __property      long                ColorEnable = { read=FGetColorEnable,
                                                        write=FSetColorEnable
                                                        ,nodefault};

// --------------------------------------------------------------------------
// Property:        Contrast
// Description:     Relative difference between higher and lower intensity
//                  luminance values in IRE units*100.
// Note:            The valid range is 0 to 20,000. The default value is
//                  10,000 (100 IRE). Higher values of contrast cause darker
//                  luminance values to tend towards black, and cause lighter
//                  luminance values to tend towards white. Lower values of
//                  contrast cause all luminance values to move towards the
//                  middle luminance values
// --------------------------------------------------------------------------
    __property      long                   Contrast = { read=FGetContrast,
                                                        write=FSetContrast,
                                                        nodefault };

// --------------------------------------------------------------------------
// Property:        Flags
// Description:     Flags specifying which object properties contain valid
//                  data.
// Note:            When the object is returned by the
//                  TDirectDrawColorControl::GetColorControls method, it also
//                  indicates which options are supported by the device
// --------------------------------------------------------------------------
    __property      dword                     Flags = { read=FGetFlags,
                                                        write=FSetFlags,
                                                        nodefault };

// --------------------------------------------------------------------------
// Property:        Gamma
// Description:     Controls the amount of gamma correction applied to the
//                  luminance values.
// Note:            The valid range is 1 to 500 gamma units, with a default
//                  of 1
// --------------------------------------------------------------------------
    __property      long                      Gamma = { read=FGetGamma,
                                                        write=FSetGamma,
                                                        nodefault };

// --------------------------------------------------------------------------
// Property:        Hue
// Description:     Phase relationship of the chrominance components.
// Note:            Hue is specified in degrees and the valid range is -180
//                  to 180. The default is 0
// --------------------------------------------------------------------------
    __property      long                        Hue = { read=FGetHue,
                                                        write=FSetHue,
                                                        nodefault };

// --------------------------------------------------------------------------
// Property:        Saturation
// Description:     Color intensity in IRE units*100.
// Note:            The valid range is 0 to 20,000. The default value is
//                  10,000 (100 IRE)
// --------------------------------------------------------------------------
    __property      long                 Saturation = { read=FGetSaturation,
                                                        write=FSetSaturation,
                                                        nodefault };

// --------------------------------------------------------------------------
// Property:        Sharpness
// Description:     Sharpness in arbitrary units.
// Note:            The valid range is 0 to 10. The default value is 5
// --------------------------------------------------------------------------
    __property      long                  Sharpness = { read=FGetSharpness,
                                                        write=FSetSharpness,
                                                        nodefault };

// --------------------------------------------------------------------------
// Property:        Size
// Description:     This contains the size of the internal DDCOLORCONTROL
//                  structure.
// --------------------------------------------------------------------------
    __property      dword                      Size = { read=FGetSize,
                                                        write=FSetSize,         //added write
                                                        nodefault };            //added default

// --------------------------------------------------------------------------
// Constructor & Destructor
// --------------------------------------------------------------------------
                    __fastcall      TDDColorControl();
    virtual         __fastcall      ~TDDColorControl();

// --------------------------------------------------------------------------
// Method:          Ptr
// Description:     This property returns a pointer to the internal
//                  DDCOLORCONTROL structure.
// --------------------------------------------------------------------------
    DDCOLORCONTROL* __fastcall     Ptr();

protected:

    long            __fastcall  FGetBrightness();
    void            __fastcall  FSetBrightness( long brightness );
    long            __fastcall  FGetColorEnable();
    void            __fastcall  FSetColorEnable( long colorenable );
    long            __fastcall  FGetContrast();
    void            __fastcall  FSetContrast( long contrast );
    dword           __fastcall  FGetFlags();
    void            __fastcall  FSetFlags( dword flags );
    long            __fastcall  FGetGamma();
    void            __fastcall  FSetGamma( long gamma );
    long            __fastcall  FGetHue();
    void            __fastcall  FSetHue( long hue );
    long            __fastcall  FGetSaturation();
    void            __fastcall  FSetSaturation( long saturation );
    long            __fastcall  FGetSharpness();
    void            __fastcall  FSetSharpness( long sharpness );
    dword           __fastcall  FGetSize();
    void            __fastcall  FSetSize( dword pSize );                        //added set

private:

    DDCOLORCONTROL  fColorControl;

};
// --------------------------------------------------------------------------
#endif
