//---------------------------------------------------------------------------
#ifndef TDDOverlayFXH
#define TDDOverlayFXH

// ==========================================================================
// File:            TDDOverlayFX.H
// Description:
//
// (c) 1998 A.S.C. Alternate Software Concepts, Sydney, Australia.
// All Rights Reserved. Refer to the file TDirectX_Licence.Txt for detailed
// licence & copyright information.
// ==========================================================================

// --------------------------------------------------------------------------
// include the basic TDirectDraw definitions, etc.
// --------------------------------------------------------------------------
#include "TDirectDrawInclude.h"
#include "TDirectDrawDefs.h"
#include "TDirectDrawErrors.h"

// --------------------------------------------------------------------------
// include the various DirectX Wrapper classes used by TDDOverlayFX
// --------------------------------------------------------------------------
#include "TDDColorKey.h"

// --------------------------------------------------------------------------
// include the various DirectX Component classes used by TDDOverlayFX
// --------------------------------------------------------------------------
#include "TDirectDrawSurface.h"

// ==========================================================================
// Class:           TDDOverlayFX
// Description:     The TDDOverlayFX object passes override information to
//                  the TDirectDrawSurface::UpdateOverlay method
// ==========================================================================
class TDDOverlayFX : public TPersistent
{
__published:
public:

// --------------------------------------------------------------------------
// Property:        AlphaDest
// Description:     Address of a surface to use as the alpha channel for a
//                  destination
// --------------------------------------------------------------------------
    __property      TDirectDrawSurface*   AlphaDest = { read=FGetAlphaDest,     //added read
                                                        write=FSetAlphaDest,
                                                        nodefault };

// --------------------------------------------------------------------------
// Property:        AlphaDestConst
// Description:     Constant to use as the alpha channel for a destination
// --------------------------------------------------------------------------
    __property      dword            AlphaDestConst = {
                                                    read=FGetAlphaDestConst,
                                                    write=FSetAlphaDestConst,
                                                    nodefault };

// --------------------------------------------------------------------------
// Property:        AlphaDestConstBitDepth
// Description:     Bit depth used to specify the alpha constant for a
//                  destination
// --------------------------------------------------------------------------
    __property      dword    AlphaDestConstBitDepth = {
                                            read=FGetAlphaDestConstBitDepth,
                                            write=FSetAlphaDestConstBitDepth,
                                            nodefault };

// --------------------------------------------------------------------------
// Property:        AlphaEdgeBlend
// Description:     Constant to use as the alpha for an edge blend
// --------------------------------------------------------------------------
    __property      dword            AlphaEdgeBlend = {
                                                    read=FGetAlphaEdgeBlend,
                                                    write=FSetAlphaEdgeBlend,
                                                    nodefault };

// --------------------------------------------------------------------------
// Property:        AlphaEdgeBlendBitDepth
// Description:     Bit depth used to specify the constant for an alpha edge
//                  blend
// --------------------------------------------------------------------------
    __property      dword    AlphaEdgeBlendBitDepth = {
                                            read=FGetAlphaEdgeBlendBitDepth,
                                            write=FSetAlphaEdgeBlendBitDepth,
                                            nodefault };

// --------------------------------------------------------------------------
// Property:        AlphaSrc
// Description:     Address of a surface to use as the alpha channel for a
//                  source
// --------------------------------------------------------------------------
    __property      TDirectDrawSurface*    AlphaSrc = { read=FGetAlphaSrc,      //added read
                                                        write=FSetAlphaSrc,
                                                        nodefault };


// --------------------------------------------------------------------------
// Property:        AlphaSrcConst
// Description:     Constant to use as the alpha channel for a source
// --------------------------------------------------------------------------
    __property      dword             AlphaSrcConst = {
                                                    read=FGetAlphaSrcConst,
                                                    write=FSetAlphaSrcConst,
                                                    nodefault };

// --------------------------------------------------------------------------
// Property:        AlphaSrcConstBitDepth
// Description:     Bit depth used to specify the alpha constant for a source
// --------------------------------------------------------------------------
    __property      dword     AlphaSrcConstBitDepth = {
                                            read=FGetAlphaSrcConstBitDepth,
                                            write=FSetAlphaSrcConstBitDepth,
                                            nodefault };

// --------------------------------------------------------------------------
// Property:        DDFX
// Description:     Overlay FX flags
// --------------------------------------------------------------------------
    __property      dword                      DDFX = { read=FGetDDFX,
                                                        write=FSetDDFX,
                                                        nodefault };

// --------------------------------------------------------------------------
// Property:        DestColorKey
// Description:     Destination color key override
// --------------------------------------------------------------------------
    __property      TDDColorKey*       DestColorKey = {
                                                    read=FGetDestColorKey,     //added read
                                                    write=FSetDestColorKey,
                                                    nodefault };

// --------------------------------------------------------------------------
// Property:        Size
// Description:     This contains the size of the internal DDOVERLAYFX
//                  structure.
// --------------------------------------------------------------------------
    __property      dword                      Size = { read=FGetSize,
                                                        write=FSetSize,         //added write
                                                        nodefault };            //added default

// --------------------------------------------------------------------------
// Property:        SrcColorKey
// Description:     Source color key override
// --------------------------------------------------------------------------
    __property      TDDColorKey*        SrcColorKey = { read=FGetSrcColorKey,   //added read
                                                        write=FSetSrcColorKey
                                                        ,nodefault };

// --------------------------------------------------------------------------
// Constructor & Destructor
// --------------------------------------------------------------------------
                    __fastcall          TDDOverlayFX();
    virtual         __fastcall          ~TDDOverlayFX();

// --------------------------------------------------------------------------
// Method:          Ptr
// Description:     This property returns a pointer to the internal
//                  DDOVERLAYFX structure.
// --------------------------------------------------------------------------
    DDOVERLAYFX*    __fastcall          Ptr();

// --------------------------------------------------------------------------
// End of the TDDOverlayFX wrapper
// --------------------------------------------------------------------------


protected:

TDirectDrawSurface* __fastcall  FGetAlphaDest();                                //added get
    void            __fastcall  FSetAlphaDest( TDirectDrawSurface*
                                                                alphadest );
    dword           __fastcall  FGetAlphaDestConst();
    void            __fastcall  FSetAlphaDestConst( dword alphadestconst );
    dword           __fastcall  FGetAlphaDestConstBitDepth();
    void            __fastcall  FSetAlphaDestConstBitDepth( dword
                                                    alphadestconstbitdepth );
    dword           __fastcall  FGetAlphaEdgeBlend();
    void            __fastcall  FSetAlphaEdgeBlend( dword
                                                            alphaedgeblend );
    dword           __fastcall  FGetAlphaEdgeBlendBitDepth();
    void            __fastcall  FSetAlphaEdgeBlendBitDepth( dword
                                                    alphaedgeblendbitdepth );
TDirectDrawSurface* __fastcall  FGetAlphaSrc();                                 //added get
    void            __fastcall  FSetAlphaSrc( TDirectDrawSurface* alphasrc );
    dword           __fastcall  FGetAlphaSrcConst();
    void            __fastcall  FSetAlphaSrcConst( dword alphasrcconst );
    dword           __fastcall  FGetAlphaSrcConstBitDepth();
    void            __fastcall  FSetAlphaSrcConstBitDepth( dword
                                                    alphasrcconstbitdepth );
    dword           __fastcall  FGetDDFX();
    void            __fastcall  FSetDDFX( dword ddfx );
    TDDColorKey*    __fastcall  FGetDestColorKey();                             //added get
    void            __fastcall  FSetDestColorKey( TDDColorKey*
                                                            destcolorkey );
    dword           __fastcall  FGetSize();
    void            __fastcall  FSetSize( dword pSize );                        //added set
    TDDColorKey*    __fastcall  FGetSrcColorKey();                              //added get
    void            __fastcall  FSetSrcColorKey( TDDColorKey* srccolorkey );

private:

    DDOVERLAYFX     fOverlayFX;
};
//---------------------------------------------------------------------------
#endif

