//---------------------------------------------------------------------------
#ifndef TDDPixelFormatH
#define TDDPixelFormatH

// ==========================================================================
// File:            TDDPixelFormat.H
// Description:
//
// (c) 1998 A.S.C. Alternate Software Concepts, Sydney, Australia.
// All Rights Reserved. Refer to the file TDirectX_Licence.Txt for detailed
// licence & copyright information.
// ==========================================================================

// --------------------------------------------------------------------------
// include the basic TDirectDraw definitions, etc.
// --------------------------------------------------------------------------
#include "TDirectDrawInclude.h"
#include "TDirectDrawDefs.h"
#include "TDirectDrawErrors.h"
#include "TDirectDrawMisc.h"

// ==========================================================================
// Class:           TDDPixelFormat
// Description:     This object describes the pixel format of a
//                  TDirectDrawSurface object for the
//                  TDirectDrawSurface::GetPixelFormat method
// ==========================================================================
class TDDPixelFormat : public TPersistent
{
__published:
public:

// --------------------------------------------------------------------------
// Property:        Flags
// Description:     Optional control flags
// --------------------------------------------------------------------------
    __property      dword                     Flags = { read=FGetFlags,
                                                        write=FSetFlags,
                                                        nodefault };

// --------------------------------------------------------------------------
// Property:        FourCC
// Description:     FourCC code
// --------------------------------------------------------------------------
    __property      dword                    FourCC = { read=FGetFourCC,
                                                        write=FSetFourCC,
                                                        nodefault };

// --------------------------------------------------------------------------
// Property:        RGBBitCount
// Description:     RGB bits per pixel
// Note:            4, 8, 16, 24 or 32
// --------------------------------------------------------------------------
    __property      dword               RGBBitCount = { read=FGetRGBBitCount,
                                                        write=FSetRGBBitCount
                                                        ,nodefault };

// --------------------------------------------------------------------------
// Property:        Size
// Description:     This contains the size of the internal DDPIXELFORMAT
//                  structure
// --------------------------------------------------------------------------
    __property      dword                      Size = { read=FGetSize,
                                                        write=FSetSize,         //added write
                                                        nodefault };            //added default

// --------------------------------------------------------------------------
// Property:        YUVBitCount
// Description:     YUV bits per pixel
// Note:            4, 8, 16, 24 or 32
// --------------------------------------------------------------------------
    __property      dword               YUVBitCount = { read=FGetYUVBitCount,
                                                        write=FSetYUVBitCount
                                                        ,nodefault };

// --------------------------------------------------------------------------
// Property:        ZBufferBitDepth
// Description:     Z-buffer bit depth
// Note:            8, 16 or 24. 32-bit z-buffers are not supported
// --------------------------------------------------------------------------
    __property      dword           ZBufferBitDepth = {
                                                    read=FGetZBufferBitDepth,
                                                    write=FSetZBufferBitDepth
                                                    ,nodefault };

// --------------------------------------------------------------------------
// Property:        AlphaBitDepth
// Description:     Alpha channel bit depth
// Type:            1, 2, 4 or 8
// --------------------------------------------------------------------------
    __property      dword             AlphaBitDepth = {
                                                    read=FGetAlphaBitDepth,
                                                    write=FSetAlphaBitDepth,
                                                    nodefault };

// --------------------------------------------------------------------------
// Property:        RBitMask
// Description:     Mask for red bits
// --------------------------------------------------------------------------
    __property      dword                  RBitMask = { read=FGetRBitMask,
                                                        write=FSetRBitMask,
                                                        nodefault };

// --------------------------------------------------------------------------
// Property:        YBitMask
// Description:     Mask for Y bits
// --------------------------------------------------------------------------
    __property      dword                  YBitMask = { read=FGetYBitMask,
                                                        write=FSetYBitMask,
                                                        nodefault };

// --------------------------------------------------------------------------
// Property:        GBitMask
// Description:     Mask for green bits
// --------------------------------------------------------------------------
    __property      dword                  GBitMask = { read=FGetGBitMask,
                                                        write=FSetGBitMask,
                                                        nodefault };

// --------------------------------------------------------------------------
// Property:        UBitMask
// Description:     Mask for U bits
// --------------------------------------------------------------------------
    __property      dword                  UBitMask = { read=FGetUBitMask,
                                                        write=FSetUBitMask,
                                                        nodefault };

// --------------------------------------------------------------------------
// Property:        BBitMask
// Description:     Mask for blue bits
// --------------------------------------------------------------------------
    __property      dword                  BBitMask = { read=FGetBBitMask,
                                                        write=FSetBBitMask,
                                                        nodefault };

// --------------------------------------------------------------------------
// Property:        VBitMask
// Description:     Mask for V bits
// --------------------------------------------------------------------------
    __property      dword                  VBitMask = { read=FGetVBitMask,
                                                        write=FSetVBitMask,
                                                        nodefault };

// --------------------------------------------------------------------------
// Property:        RGBAlphaBitMask
// Description:     Mask for alpha channel
// --------------------------------------------------------------------------
    __property      dword           RGBAlphaBitMask = {
                                                    read=FGetRGBAlphaBitMask,
                                                    write=FSetRGBAlphaBitMask
                                                    ,nodefault };

// --------------------------------------------------------------------------
// Property:        YUVAlphaBitMask
// Description:     Mask for alpha channel
// --------------------------------------------------------------------------
    __property      dword           YUVAlphaBitMask = {
                                                    read=FGetYUVAlphaBitMask,
                                                    write=FSetYUVAlphaBitMask
                                                    ,nodefault };

// --------------------------------------------------------------------------
// Property:        RGBZBitMask
// Description:     Mask for Z channel
// --------------------------------------------------------------------------
    __property      dword               RGBZBitMask = { read=FGetRGBZBitMask,
                                                        write=FSetRGBZBitMask
                                                        ,nodefault };

// --------------------------------------------------------------------------
// Property:        YUVZBitMask
// Description:     Mask for Z channel
// --------------------------------------------------------------------------
    __property      dword               YUVZBitMask = { read=FGetYUVZBitMask,
                                                        write=FSetYUVZBitMask
                                                        ,nodefault };

// --------------------------------------------------------------------------
// Constructor & Destructor
// --------------------------------------------------------------------------
                    __fastcall          TDDPixelFormat();
    virtual         __fastcall          ~TDDPixelFormat();

// --------------------------------------------------------------------------
// Method:          Ptr
// Description:     This property returns a pointer to the internal
//                  DDPIXELFORMAT structure.
// --------------------------------------------------------------------------
    DDPIXELFORMAT*  __fastcall          Ptr();

// --------------------------------------------------------------------------
// End of the TDDPixelFormat wrapper
// --------------------------------------------------------------------------

protected:

    dword           __fastcall  FGetAlphaBitDepth();
    void            __fastcall  FSetAlphaBitDepth( dword alphabitdepth );
    dword           __fastcall  FGetBBitMask();
    void            __fastcall  FSetBBitMask( dword bbitmask );
    dword           __fastcall  FGetFlags();
    void            __fastcall  FSetFlags( dword flags );
    dword           __fastcall  FGetFourCC();
    void            __fastcall  FSetFourCC( dword fourcc );
    dword           __fastcall  FGetGBitMask();
    void            __fastcall  FSetGBitMask( dword gbitmask );
    dword           __fastcall  FGetRBitMask();
    void            __fastcall  FSetRBitMask( dword rbitmask );
    dword           __fastcall  FGetRGBAlphaBitMask();
    void            __fastcall  FSetRGBAlphaBitMask( dword rgbalphabitmask );
    dword           __fastcall  FGetRGBBitCount();
    void            __fastcall  FSetRGBBitCount( dword rgbbitcount );
    dword           __fastcall  FGetRGBZBitMask();
    void            __fastcall  FSetRGBZBitMask( dword rgbzbitmask );
    dword           __fastcall  FGetSize();
    void            __fastcall  FSetSize( dword pSize );                        //added set
    dword           __fastcall  FGetUBitMask();
    void            __fastcall  FSetUBitMask( dword ubitmask );
    dword           __fastcall  FGetVBitMask();
    void            __fastcall  FSetVBitMask( dword vbitmask );
    dword           __fastcall  FGetYBitMask();
    void            __fastcall  FSetYBitMask( dword ybitmask );
    dword           __fastcall  FGetYUVAlphaBitMask();
    void            __fastcall  FSetYUVAlphaBitMask( dword yuvalphabitmask );
    dword           __fastcall  FGetYUVBitCount();
    void            __fastcall  FSetYUVBitCount( dword yuvbitcount );
    dword           __fastcall  FGetYUVZBitMask();
    void            __fastcall  FSetYUVZBitMask( dword yuvzbitmask );
    dword           __fastcall  FGetZBufferBitDepth();
    void            __fastcall  FSetZBufferBitDepth( dword zbufferbitdepth );

private:

    DDPIXELFORMAT   fPixelFormat;
};
//---------------------------------------------------------------------------
#endif

