//---------------------------------------------------------------------------
#ifndef TDDVideoPortBandwidthH
#define TDDVideoPortBandwidthH

// ==========================================================================
// File:            TDDVideoPortBandwidth.H
// Description:
//
// (c) 1998 A.S.C. Alternate Software Concepts, Sydney, Australia.
// All Rights Reserved. Refer to the file TDirectX_Licence.Txt for detailed
// licence & copyright information.
// ==========================================================================

// --------------------------------------------------------------------------
// include the basic TDirectDraw definitions, etc.
// --------------------------------------------------------------------------
#include "TDirectDrawInclude.h"
#include "TDirectDrawDefs.h"
#include "TDirectDrawErrors.h"

// ==========================================================================
// Class:           TDDVideoPortBandwidth
// Description:     The TDDVideoPortBandwidth object describes the bandwidth
//                  characteristics of an overlay surface when used with a
//                  particular video port and pixel format configuration
// Note:            This object is used with the
//                  TDirectDrawVideoPort::GetBandwidthInfo method
// ==========================================================================
class TDDVideoPortBandwidth : public TPersistent
{
__published:
public:

// --------------------------------------------------------------------------
// Property:        Caps
// Description:     Flag values specifying device dependency
// Note:            This member can be one of TDDVideoPortBandwidthCaps
//                  values
// --------------------------------------------------------------------------
    __property      dword                      Caps = { read=FGetCaps,
                                                        write=FSetCaps,
                                                        nodefault };

// --------------------------------------------------------------------------
// Property:        ColorKey
// Description:     Stretch factor or overlay source size at which an overlay
//                  with color keying is supported multiplied by 1000.
// Note:            Example 1.3 = 1300, or .75 = 750
// --------------------------------------------------------------------------
    __property      dword                  ColorKey = { read=FGetColorKey,
                                                        write=FSetColorKey,
                                                        nodefault };

// --------------------------------------------------------------------------
// Property:        Overlay
// Description:     Stretch factor or overlay source size at which an overlay
//                  is supported multiplied by 1000.
// Note:            Example 1.3 = 1300, or .75 = 750
// --------------------------------------------------------------------------
    __property      dword                   Overlay = { read=FGetOverlay,
                                                        write=FSetOverlay,
                                                        nodefault };

// --------------------------------------------------------------------------
// Property:        Size
// Description:     This contains the size of the internal
//                  DDVIDEOPORTBANDWIDTH structure
// --------------------------------------------------------------------------
    __property      dword                      Size = { read=FGetSize,
                                                        write=FSetSize,         //added write
                                                        nodefault };            //added default

// --------------------------------------------------------------------------
// Property:        YInterpAndColorKey
// Description:     Stretch factor or overlay source size at which an overlay
//                  with Y-axis interpolation and color keying is supported
//                  multiplied by 1000
// Note:            Example 1.3 = 1300, or .75 = 750
// --------------------------------------------------------------------------
    __property      dword        YInterpAndColorKey = {
                                                read=FGetYInterpAndColorKey,
                                                write=FSetYInterpAndColorKey,
                                                nodefault };

// --------------------------------------------------------------------------
// Property:        YInterpolate
// Description:     Stretch factor or overlay source size at which an overlay
//                  with Y-axis interpolation is supported multiplied by 1000
// Note:            Example 1.3 = 1300, or .75 = 750
// --------------------------------------------------------------------------
    __property      dword              YInterpolate = {
                                                    read=FGetYInterpolate,
                                                    write=FSetYInterpolate,
                                                    nodefault };

// --------------------------------------------------------------------------
// Constructor & Destructor
// --------------------------------------------------------------------------
                    __fastcall TDDVideoPortBandwidth();
    virtual         __fastcall ~TDDVideoPortBandwidth();

// --------------------------------------------------------------------------
// Method:          Ptr
// Description:     This property returns a pointer to the internal
//                  DDBLTBATCH structure.
// --------------------------------------------------------------------------
DDVIDEOPORTBANDWIDTH* __fastcall Ptr();

// --------------------------------------------------------------------------
// End of the TDDBltBatch wrapper
// --------------------------------------------------------------------------

protected:

    dword           __fastcall  FGetCaps();
    void            __fastcall  FSetCaps( dword caps );
    dword           __fastcall  FGetColorKey();
    void            __fastcall  FSetColorKey( dword colorkey );
    dword           __fastcall  FGetOverlay();
    void            __fastcall  FSetOverlay( dword overlay );
    dword           __fastcall  FGetSize();
    void            __fastcall  FSetSize( dword pSize );                        //added set
    dword           __fastcall  FGetYInterpAndColorKey();
    void            __fastcall  FSetYInterpAndColorKey( dword
                                                        yinterpandcolorkey );
    dword           __fastcall  FGetYInterpolate();
    void            __fastcall  FSetYInterpolate( dword yinterpolate );

private:

DDVIDEOPORTBANDWIDTH fVideoPortBandwidth;

};
//---------------------------------------------------------------------------
#endif
