//---------------------------------------------------------------------------
#ifndef TDDVideoPortCapsH
#define TDDVideoPortCapsH

// ==========================================================================
// File:            TDDVideoPortCaps.H
// Description:
//
// (c) 1998 A.S.C. Alternate Software Concepts, Sydney, Australia.
// All Rights Reserved. Refer to the file TDirectX_Licence.Txt for detailed
// licence & copyright information.
// ==========================================================================

// --------------------------------------------------------------------------
// include the basic TDirectDraw definitions, etc.
// --------------------------------------------------------------------------
#include "TDirectDrawInclude.h"
#include "TDirectDrawDefs.h"
#include "TDirectDrawErrors.h"
#include "TDirectDrawMisc.h"

// ==========================================================================
// Class:           TDDVideoPortCaps
// Description:     The TDDVideoPortCaps object describes the capabilities
//                  and alignment restrictions of a video port.
// Note:            This object is used with the
//                  TDirectDrawVideoPortContainer::EnumVideoPorts method
// ==========================================================================
class TDDVideoPortCaps : public TPersistent
{
__published:
public:

// --------------------------------------------------------------------------
// Property:        AlignVideoPortBoundary
// Description:     Byte restriction of placement within the surface
// --------------------------------------------------------------------------
    __property      dword    AlignVideoPortBoundary = {
                                        read=FGetAlignVideoPortBoundary,
                                        write=FSetAlignVideoPortBoundary,       //added write and nodefault
                                        nodefault };

// --------------------------------------------------------------------------
// Property:        AlignVideoPortCropBoundary
// Description:     Byte restriction of left cropping
// --------------------------------------------------------------------------
    __property     dword AlignVideoPortCropBoundary = {
                                    read=FGetAlignVideoPortCropBoundary,
                                    write=FSetAlignVideoPortCropBoundary,       //added write and nodefault
                                    nodefault };

// --------------------------------------------------------------------------
// Property:        AlignVideoPortCropWidth
// Description:     Byte restriction of cropping width
// --------------------------------------------------------------------------
    __property      dword   AlignVideoPortCropWidth = {
                                        read=FGetAlignVideoPortCropWidth,
                                        write=FSetAlignVideoPortCropWidth,      //added write and nodefault
                                        nodefault };

// --------------------------------------------------------------------------
// Property:        AlignVideoPortPrescaleWidth
// Description:     Byte restriction of width after prescaling
// --------------------------------------------------------------------------
    __property    dword AlignVideoPortPrescaleWidth = {
                                    read=FGetAlignVideoPortPrescaleWidth,
                                    write=FSetAlignVideoPortPrescaleWidth,      //added write and nodefault
                                    nodefault };

// --------------------------------------------------------------------------
// Property:        Caps
// Description:     Video port capabilities
// --------------------------------------------------------------------------
    __property      dword                      Caps = { read=FGetCaps,
                                                        write=FSetCaps,         //added write and nodefault
                                                        nodefault };

// --------------------------------------------------------------------------
// Property:        Flags
// Description:     Flag values indicating the fields that contain valid data
// --------------------------------------------------------------------------
    __property      dword                     Flags = { read=FGetFlags,
                                                        write=FSetFlags,        //added write and nodefault
                                                        nodefault };

// --------------------------------------------------------------------------
// Property:        FX
// Description:     Additional video port capabilities
// --------------------------------------------------------------------------
    __property      dword                        FX = { read=FGetFX,
                                                        write=FSetFX,           //added write and nodefault
                                                        nodefault };

// --------------------------------------------------------------------------
// Property:        MaxHeight
// Description:     Maximum height of the video port field
// --------------------------------------------------------------------------
    __property      dword                 MaxHeight = { read=FGetMaxHeight,
                                                        write=FSetMaxHeight,    //added write and nodefault
                                                        nodefault };

// --------------------------------------------------------------------------
// Property:        MaxVBIWidth
// Description:     Maximum width of the VBI data
// --------------------------------------------------------------------------
    __property      dword               MaxVBIWidth = {
                                                    read=FGetMaxVBIWidth,
                                                    write=FSetMaxVBIWidth,      //added write and nodefault
                                                    nodefault };

// --------------------------------------------------------------------------
// Property:        MaxWidth
// Description:     Maximum width of the video port field
// --------------------------------------------------------------------------
    __property      dword                  MaxWidth = { read=FGetMaxWidth,
                                                        write=FSetMaxWidth,     //added write and nodefault
                                                        nodefault };

// --------------------------------------------------------------------------
// Property:        NumAutoFlipSurfaces
// Description:     Number of autoflippable surfaces supported by the video
//                  port
// --------------------------------------------------------------------------
    __property      dword       NumAutoFlipSurfaces = {
                                            read=FGetNumAutoFlipSurfaces,
                                            write=FSetNumAutoFlipSurfaces,      //added write and nodefault
                                            nodefault };

// --------------------------------------------------------------------------
// Property:        NumVBIAutoFlipSurfaces
// Description:     Number of autoflipping surfaces capable of receiving data
//                  transmitted during the vertical blanking interval (VBI)
//                  independent from the remainder of the video stream.
// Note:            When constructing the autoflip chain, the number of VBI
//                  surfaces must equal the number of surfaces receiving the
//                  remainder of the video data
// --------------------------------------------------------------------------
    __property      dword    NumVBIAutoFlipSurfaces = {
                                        read=FGetNumVBIAutoFlipSurfaces,
                                        write=FSetNumVBIAutoFlipSurfaces,       //added write and nodefault
                                        nodefault };

// --------------------------------------------------------------------------
// Property:        PreShrinkXStep
// Description:     Width can be shrunk in the x direction in steps of
//                  1/dwPreshrinkXStep
// --------------------------------------------------------------------------
    __property      dword            PreShrinkXStep = {
                                                read=FGetPreShrinkXStep,
                                                write=FSetPreShrinkXStep,       //added write and nodefault
                                                nodefault };

// --------------------------------------------------------------------------
// Property:        PreShrinkYStep
// Description:     Height can be shrunk in the y direction in steps of
//                  1/dwPreshrinkYStep
// --------------------------------------------------------------------------
    __property      dword            PreShrinkYStep = {
                                                read=FGetPreShrinkYStep,
                                                write=FSetPreShrinkYStep,       //added write and nodefault
                                                nodefault };

// --------------------------------------------------------------------------
// Property:        Size
// Description:     This contains the size of the internal DDVIDEOPORTCAPS
//                  structure.
// --------------------------------------------------------------------------
    __property      dword                      Size = { read=FGetSize,
                                                        write=FSetSize,         //added write
                                                        nodefault };            //added default

// --------------------------------------------------------------------------
// Property:        VideoPortID
// Description:     Zero based index identifying the video port
// --------------------------------------------------------------------------
    __property      dword               VideoPortID = {
                                                    read=FGetVideoPortID,
                                                    write=FSetVideoPortID,      //added write and nodefault
                                                    nodefault };

// --------------------------------------------------------------------------
// Constructor & Destructor
// --------------------------------------------------------------------------
                    __fastcall  TDDVideoPortCaps();
    virtual         __fastcall  ~TDDVideoPortCaps();

// --------------------------------------------------------------------------
// Method:          Ptr
// Description:     This property returns a pointer to the internal
//                  DDVIDEOPORTCAPS structure.
// --------------------------------------------------------------------------
DDVIDEOPORTCAPS*    __fastcall  Ptr();

// --------------------------------------------------------------------------
// End of the TDDBltBatch wrapper
// --------------------------------------------------------------------------

protected:

    dword           __fastcall  FGetAlignVideoPortBoundary();
    void            __fastcall  FSetAlignVideoPortBoundary( dword
                                                pAlignVideoPortBoundary );      //added set
    dword           __fastcall  FGetAlignVideoPortCropBoundary();
    void            __fastcall  FSetAlignVideoPortCropBoundary( dword
                                            pAlignVideoPortCropBoundary );      //added set
    dword           __fastcall  FGetAlignVideoPortCropWidth();
    void            __fastcall  FSetAlignVideoPortCropWidth( dword
                                                pAlignVideoPortCropWidth );     //added set
    dword           __fastcall  FGetAlignVideoPortPrescaleWidth();
    void            __fastcall  FSetAlignVideoPortPrescaleWidth( dword
                                            pAlignVideoPortPrescaleWidth );     //added set
    dword           __fastcall  FGetCaps();
    void            __fastcall  FSetCaps( dword pCaps );                        //added set
    dword           __fastcall  FGetFlags();
    void            __fastcall  FSetFlags( dword pFlags );                      //added set
    dword           __fastcall  FGetFX();
    void            __fastcall  FSetFX( dword pFX );                            //added set
    dword           __fastcall  FGetMaxHeight();
    void            __fastcall  FSetMaxHeight( dword pMaxHeight );              //added set
    dword           __fastcall  FGetMaxVBIWidth();
    void            __fastcall  FSetMaxVBIWidth( dword pMaxVBIWidth );          //added set
    dword           __fastcall  FGetMaxWidth();
    void            __fastcall  FSetMaxWidth( dword pMaxWidth );                //added set
    dword           __fastcall  FGetNumAutoFlipSurfaces();
    void            __fastcall  FSetNumAutoFlipSurfaces( dword
                                                    pNumAutoFlipSurfaces );     //added set
    dword           __fastcall  FGetNumVBIAutoFlipSurfaces();
    void            __fastcall  FSetNumVBIAutoFlipSurfaces( dword
                                                pNumVBIAutoFlipSurfaces );      //added set
    dword           __fastcall  FGetPreShrinkXStep();
    void            __fastcall  FSetPreShrinkXStep( dword pPreShrinkXStep );    //added set
    dword           __fastcall  FGetPreShrinkYStep();
    void            __fastcall  FSetPreShrinkYStep( dword pPreShrinkYStep );    //added set
    dword           __fastcall  FGetSize();
    void            __fastcall  FSetSize( dword pSize );                        //added set
    dword           __fastcall  FGetVideoPortID();
    void            __fastcall  FSetVideoPortID( dword pVideoPortID );          //added set

private:

DDVIDEOPORTCAPS     fVideoPortCaps;

};
//---------------------------------------------------------------------------
#endif
