// --------------------------------------------------------------------------
#include <vcl\vcl.h>
#pragma hdrstop

// ==========================================================================
// File:            TDVideoPortInfo.CPP
// Description:
//
// (c) 1998 A.S.C. Alternate Software Concepts, Sydney, Australia.
// All Rights Reserved. Refer to the file TDirectX_Licence.Txt for detailed
// licence & copyright information.
// ==========================================================================

// --------------------------------------------------------------------------
// include the TDDBltBatch H file.
// --------------------------------------------------------------------------
#include "TDDVideoPortInfo.h"

// ==========================================================================
// Class:           TDDVideoPortInfo
// Description:     This object describes the transfer of video data to a
//                  surface
// Note:            This object is used with the
//                  TDirectDrawVideoPort::StartVideo method
// ==========================================================================

// --------------------------------------------------------------------------
// Constructor
// --------------------------------------------------------------------------
                    __fastcall  TDDVideoPortInfo::TDDVideoPortInfo()
                                                            : TPersistent()
{
fVideoPortInfo.dwSize = sizeof(DDVIDEOPORTINFO);
}

// --------------------------------------------------------------------------
// Destructor
// --------------------------------------------------------------------------
__fastcall  TDDVideoPortInfo::~TDDVideoPortInfo()
{
}

// --------------------------------------------------------------------------
// Method:          Ptr
// Description:     This method returns a pointer to the internal
//                  DDVIDEOPORTINFO structure.
// --------------------------------------------------------------------------
DDVIDEOPORTINFO*    __fastcall  TDDVideoPortInfo::Ptr()
{
return &fVideoPortInfo;
}

// --------------------------------------------------------------------------
// Method:          FGetCrop
// Description:     Returns the cropping rectangle
// Note:            This property is optional
// --------------------------------------------------------------------------
TRect               __fastcall  TDDVideoPortInfo::FGetCrop()
{
TRect r = Rect( fVideoPortInfo.rCrop.left, fVideoPortInfo.rCrop.top,
                fVideoPortInfo.rCrop.right, fVideoPortInfo.rCrop.bottom );
return r;
}

// --------------------------------------------------------------------------
// Method:          FSetCrop
// Description:     Sets the cropping rectangle
// Note:            This property is optional
// --------------------------------------------------------------------------
void                __fastcall  TDDVideoPortInfo::FSetCrop( TRect crop )
{
fVideoPortInfo.rCrop.top = crop.Top;
fVideoPortInfo.rCrop.left = crop.Left;
fVideoPortInfo.rCrop.right = crop.Right;
fVideoPortInfo.rCrop.bottom = crop.Bottom;
}

// --------------------------------------------------------------------------
// Method:          FGetInputFormat                                             //added get
// Description:     Returns a TDDPixelFormat object describing the pixel
//                  format to be written to the video port
// Note:            This will often be identical to the surface's pixel
//                  format, but can differ if the video port is to perform
//                  conversion
// --------------------------------------------------------------------------
TDDPixelFormat*     __fastcall  TDDVideoPortInfo::FGetInputFormat()
{
return NULL;
}

// --------------------------------------------------------------------------
// Method:          FSetInputFormat
// Description:     Sets a TDDPixelFormat object describing the pixel format
//                  to be written to the video port
// Note:            This will often be identical to the surface's pixel
//                  format, but can differ if the video port is to perform
//                  conversion
// --------------------------------------------------------------------------
void                __fastcall  TDDVideoPortInfo::FSetInputFormat(
                                                TDDPixelFormat* inputformat )
{
if (inputformat!=NULL) fVideoPortInfo.lpddpfInputFormat = inputformat->Ptr();
}

// --------------------------------------------------------------------------
// Method:          FGetOriginX
// Description:     Returns the X coordinate for the origin of the video data
//                  in the surface
// --------------------------------------------------------------------------
dword               __fastcall  TDDVideoPortInfo::FGetOriginX()
{
return fVideoPortInfo.dwOriginX;
}

// --------------------------------------------------------------------------
// Method:          FSetOriginX
// Description:     Sets the X coordinate for the origin of the video data in
//                  the surface
// --------------------------------------------------------------------------
void                __fastcall  TDDVideoPortInfo::FSetOriginX( dword
                                                                    originx )
{
fVideoPortInfo.dwOriginX = originx;
}

// --------------------------------------------------------------------------
// Method:          FGetOriginY
// Description:     Returns the Y coordinate for the origin of the video data
//                  in the surface
// --------------------------------------------------------------------------
dword               __fastcall  TDDVideoPortInfo::FGetOriginY()
{
return fVideoPortInfo.dwOriginY;
}

// --------------------------------------------------------------------------
// Method:          FSetOriginY
// Description:     Sets the Y coordinate for the origin of the video data in
//                  the surface
// --------------------------------------------------------------------------
void                __fastcall  TDDVideoPortInfo::FSetOriginY( dword
                                                                    originy )
{
fVideoPortInfo.dwOriginY = originy;
}

// --------------------------------------------------------------------------
// Method:          FGetPrescaleHeight
// Description:     Returns the pre-scaling or zooming in the y direction
// Note:            This property is optional
// --------------------------------------------------------------------------
dword       __fastcall  TDDVideoPortInfo::FGetPrescaleHeight()
{
return fVideoPortInfo.dwPrescaleHeight;
}

// --------------------------------------------------------------------------
// Method:          FSetPrescaleHeight
// Description:     Sets the pre-scaling or zooming in the y direction
// Note:            This property is optional
// --------------------------------------------------------------------------
void                __fastcall  TDDVideoPortInfo::FSetPrescaleHeight( dword
                                                            prescaleheight )
{
fVideoPortInfo.dwPrescaleHeight = prescaleheight;
}

// --------------------------------------------------------------------------
// Method:          FGetPrescaleWidth
// Description:     Returns the pre-scaling or zooming in the x direction
// Note:            This property is optional
// --------------------------------------------------------------------------
dword               __fastcall  TDDVideoPortInfo::FGetPrescaleWidth()
{
return fVideoPortInfo.dwPrescaleWidth;
}

// --------------------------------------------------------------------------
// Method:          FSetPrescaleWidth
// Description:     Sets the pre-scaling or zooming in the x direction
// Note:            This property is optional
// --------------------------------------------------------------------------
void                __fastcall  TDDVideoPortInfo::FSetPrescaleWidth( dword
                                                            prescalewidth )
{
fVideoPortInfo.dwPrescaleWidth = prescalewidth;
}

// --------------------------------------------------------------------------
// Method:          FGetSize
// Description:     This returns the size of the internal DDVIDEOPORTINFO
//                  structure
// --------------------------------------------------------------------------
dword               __fastcall  TDDVideoPortInfo::FGetSize()
{
return sizeof(DDVIDEOPORTINFO);
}

// --------------------------------------------------------------------------
// Method:          FSetSize                                                    //added set
// Description:     This sets the size of the internal DDVIDEOPORTINFO
//                  structure
// --------------------------------------------------------------------------
void               __fastcall  TDDVideoPortInfo::FSetSize( dword pSize )
{
return;
}

// --------------------------------------------------------------------------
// Method:          FGetVBIHeight
// Description:     Returns the amount of data within the vertical blanking
//                  interval, in scan lines
// --------------------------------------------------------------------------
dword               __fastcall  TDDVideoPortInfo::FGetVBIHeight()
{
return fVideoPortInfo.dwVBIHeight;
}

// --------------------------------------------------------------------------
// Method:          FSetVBIHeight
// Description:     Sets the amount of data within the vertical blanking
//                  interval, in scan lines
// --------------------------------------------------------------------------
void                __fastcall  TDDVideoPortInfo::FSetVBIHeight( dword
                                                                vbiheight )
{
fVideoPortInfo.dwVBIHeight = vbiheight;
}

// --------------------------------------------------------------------------
// Method:          FGetVBIInputFormat                                          //added get
// Description:     Returns a TDDPixelFormat object describing the input
//                  pixel format of the data within the vertical blanking
//                  interval
// --------------------------------------------------------------------------
TDDPixelFormat*     __fastcall  TDDVideoPortInfo::FGetVBIInputFormat()
{
return NULL;
}

// --------------------------------------------------------------------------
// Method:          FSetVBIInputFormat
// Description:     Sets a TDDPixelFormat object describing the input pixel
//                  format of the data within the vertical blanking interval
// --------------------------------------------------------------------------
void                __fastcall  TDDVideoPortInfo::FSetVBIInputFormat(
                                            TDDPixelFormat* vbiinputformat )
{
if (vbiinputformat!=NULL) fVideoPortInfo.lpddpfVBIInputFormat =
                                                    vbiinputformat->Ptr();
}

// --------------------------------------------------------------------------
// Method:          FGetVBIOutputFormat                                         //added get
// Description:     Returns TDDPixelFormat object describing the output pixel
//                  format of the data within the vertical blanking interval
// --------------------------------------------------------------------------
TDDPixelFormat*     __fastcall  TDDVideoPortInfo::FGetVBIOutputFormat()
{
return NULL;
}

// --------------------------------------------------------------------------
// Method:          FSetVBIOutputFormat
// Description:     Sets a TDDPixelFormat object describing the output pixel
//                  format of the data within the vertical blanking interval
// --------------------------------------------------------------------------
void                __fastcall  TDDVideoPortInfo::FSetVBIOutputFormat(
                                            TDDPixelFormat* vbioutputformat )
{
if (vbioutputformat!=NULL) fVideoPortInfo.lpddpfVBIOutputFormat =
                                                    vbioutputformat->Ptr();
}

// --------------------------------------------------------------------------
// Method:          FGetVPFlags
// Description:     Returns the video port options
// --------------------------------------------------------------------------
dword               __fastcall  TDDVideoPortInfo::FGetVPFlags()
{
return fVideoPortInfo.dwVPFlags;
}

// --------------------------------------------------------------------------
// Method:          FSetVPFlags
// Description:     Sets the video port options
// --------------------------------------------------------------------------
void                __fastcall  TDDVideoPortInfo::FSetVPFlags( dword flags )
{
fVideoPortInfo.dwVPFlags = flags;
}

// --------------------------------------------------------------------------

