// --------------------------------------------------------------------------
#ifndef TDDVideoPortStatusH
#define TDDVideoPortStatusH

// ==========================================================================
// File:            TDDVideoPortStatus.H
// Description:
//
// (c) 1998 A.S.C. Alternate Software Concepts, Sydney, Australia.
// All Rights Reserved. Refer to the file TDirectX_Licence.Txt for detailed
// licence & copyright information.
// ==========================================================================

// --------------------------------------------------------------------------
// include the basic TDirectDraw definitions, etc.
// --------------------------------------------------------------------------
#include "TDirectDrawInclude.h"
#include "TDirectDrawDefs.h"
#include "TDirectDrawErrors.h"

// --------------------------------------------------------------------------
// include the various DirectX Wrapper classes used by TDDBltBatch
// --------------------------------------------------------------------------
#include "TDDVideoPortConnect.h"

// ==========================================================================
// Class:           TDDVideoPortStatus
// Description:     This object describes the status of a video-port object
// Note:            This object is used with the
//                  TDirectDrawVideoPortContainer::QueryVideoPortStatus
//                  method
// ==========================================================================
class TDDVideoPortStatus : public TPersistent
{
__published:
public:

// --------------------------------------------------------------------------
// Property:        Flags
// Description:     Video port control flags
// Note:            Not currently used
// --------------------------------------------------------------------------
    __property      dword                     Flags = { read=FGetFlags,
                                                        write=FSetFlags,        //added write, nodefault
                                                        nodefault };

// --------------------------------------------------------------------------
// Property:        InUse
// Description:     Value indicating the current status of the video port
// Note:            This property is TRUE if the video port is currently
//                  being used, and FALSE otherwise
// --------------------------------------------------------------------------
    __property      bool                      InUse = { read=FGetInUse,
                                                        write=FSetInUse,        //added write, nodefault
                                                        nodefault };

// --------------------------------------------------------------------------
// Property:        Size
// Description:     This contains the size of the internal DDVIDEOPORTSTATUS
//                  structure
// --------------------------------------------------------------------------
    __property      dword                      Size = { read=FGetSize,
                                                        write=FSetSize,         //added write
                                                        nodefault };            //added default

// --------------------------------------------------------------------------
// Property:        VideoPortType
// Description:     A TDDVideoPortConnect object that receives information
//                  about the video-port connection
// --------------------------------------------------------------------------
    __property   TDDVideoPortConnect* VideoPortType = {
                                                    read=FGetVideoPortType,     //added read
                                                    write=FSetVideoPortType,
                                                    nodefault };

// --------------------------------------------------------------------------
// Constructor & Destructor
// --------------------------------------------------------------------------
                    __fastcall  TDDVideoPortStatus();
    virtual         __fastcall  ~TDDVideoPortStatus();

// --------------------------------------------------------------------------
// Method:          Ptr
// Description:     This returns a pointer to the internal DDVIDEOPORTSTATUS
//                  structure.
// --------------------------------------------------------------------------
DDVIDEOPORTSTATUS*  __fastcall  Ptr();

// --------------------------------------------------------------------------
// End of the TDDBltBatch wrapper
// --------------------------------------------------------------------------

protected:

    dword           __fastcall  FGetFlags();
    void            __fastcall  FSetFlags( dword pFlags );                      //added set
    bool            __fastcall  FGetInUse();
    void            __fastcall  FSetInUse( bool pInUse );                       //added set
    dword           __fastcall  FGetSize();
    void            __fastcall  FSetSize( dword pSize );                        //added set
TDDVideoPortConnect*
                    __fastcall  FGetVideoPortType();                            //added get
    void            __fastcall  FSetVideoPortType( TDDVideoPortConnect*
                                                            pVideoPortType );

private:

DDVIDEOPORTSTATUS   fVideoPortStatus;
};
//---------------------------------------------------------------------------
#endif
